# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class Regression2(Template):
    'Calculate More advanced descriptive Statistics in SPSS (equivalent to Analyze->Descriptive Statistics->Explore... in SPSS 23)'

    __version__ = '1.0.0'

    tags = [ 'SPSS']
    engines = ['SPSS_script']

    inputs = '''
var1 = DataVector('First variable: ')
var2 = DataVector('Second variable: ')
'''

    spssscript = '''
GRAPH
  /SCATTERPLOT(BIVAR)=${var2} WITH ${var1}
  /MISSING=LISTWISE.

EXAMINE VARIABLES=${var1} ${var2}
  /PLOT HISTOGRAM NPPLOT
  /COMPARE GROUPS
  /STATISTICS DESCRIPTIVES
  /CINTERVAL 95
  /MISSING LISTWISE
  /NOTOTAL.

GGRAPH
  /GRAPHDATASET NAME="graphdataset" VARIABLES=${var1} MISSING=LISTWISE REPORTMISSING=NO
  /GRAPHSPEC SOURCE=INLINE.
BEGIN GPL
  SOURCE: s=userSource(id("graphdataset"))
  DATA: ${var1}=col(source(s), name("${var1}"))
  DATA: id=col(source(s), name("$CASENUM"), unit.category())
  GUIDE: text.footnote(label("${datafile.variables[var1]['varlabel']}"))
  ELEMENT: schema(position(bin.quantile.letter(1*${var1})), color.interior(color."D3CE97"), label(id))
END GPL.

GGRAPH
  /GRAPHDATASET NAME="graphdataset" VARIABLES=${var2} MISSING=LISTWISE REPORTMISSING=NO
  /GRAPHSPEC SOURCE=INLINE.
BEGIN GPL
  SOURCE: s=userSource(id("graphdataset"))
  DATA: ${var2}=col(source(s), name("${var2}"))
  DATA: id=col(source(s), name("$CASENUM"), unit.category())
  GUIDE: text.footnote(label("${datafile.variables[var2]['varlabel']}"))
  ELEMENT: schema(position(bin.quantile.letter(1*${var2})), color.interior(color."D3CE97"), label(id))
END GPL.


CORRELATIONS
  /VARIABLES=${var1} ${var2}
  /PRINT=TWOTAIL NOSIG
  /STATISTICS DESCRIPTIVES
  /MISSING=PAIRWISE.

NONPAR CORR
  /VARIABLES=${var1} ${var2}
  /PRINT=KENDALL TWOTAIL NOSIG
  /MISSING=PAIRWISE.

NONPAR CORR
  /VARIABLES=${var1} ${var2}
  /PRINT=SPEARMAN TWOTAIL NOSIG
  /MISSING=PAIRWISE.

'''


    processoutputs = '''
import EStat
from EStat.Templating import *
import base64

from lxml import etree
from io import BytesIO

ns = {'x': 'http://www.ibm.com/software/analytics/spss/xml/oms'}

def str2float(s):
    try:
        return float(s)
    except ValueError:
        return float('nan')

caseout =  etree.parse(BytesIO(outputs['Table_Case Processing Summary.xml'].content))
print(etree.tostring(caseout, pretty_print=True))

path = "string(//x:dimension[@axis='row']/x:category/@text)"
casetext = caseout.xpath(path, namespaces=ns)
print(casetext)

path = "string(//x:dimension[@axis='row']/x:category/x:dimension[@text='Cases']/x:category[@text='Valid']/x:dimension[@text='Statistics']/x:category[@text='N']/x:cell/@number)"
Nvalid = int(caseout.xpath(path, namespaces=ns))
print(Nvalid)

path = "string(//x:dimension[@axis='row']/x:category/x:dimension[@text='Cases']/x:category[@text='Missing']/x:dimension[@text='Statistics']/x:category[@text='N']/x:cell/@number)"
Nmissing = int(caseout.xpath(path, namespaces=ns))
print(Nmissing)

path = "string(//x:dimension[@axis='row']/x:category/x:dimension[@text='Cases']/x:category[@text='Total']/x:dimension[@text='Statistics']/x:category[@text='N']/x:cell/@number)"
Ntotal = int(caseout.xpath(path, namespaces=ns))
print(Ntotal)

descsout = etree.parse(BytesIO(outputs['Table_Descriptives.xml'].content))
print(etree.tostring(descsout, pretty_print=True))

path = "string(//x:dimension[@text='Dependent Variables']/x:category[@varName='" + str(var1) + "']/@label)"
var1lab = descsout.xpath(path, namespaces=ns)
print(var1lab)

path = "string(//x:dimension[@text='Dependent Variables']/x:category[@varName='" + str(var2) + "']/@label)"
var2lab = descsout.xpath(path, namespaces=ns)
print(var2lab)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number"
mean_num = [str2float(x) for x in descsout.xpath(path, namespaces=ns)]
print(mean_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text"
mean_str = descsout.xpath(path, namespaces=ns)
print(mean_str)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@number"
ses_num = [str2float(x) for x in descsout.xpath(path, namespaces=ns)]
print(ses_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@text"
ses_str = descsout.xpath(path, namespaces=ns)
print(ses_str)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval for Mean']/x:category[@text='Lower Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number"
lci_num = [str2float(x) for x in descsout.xpath(path, namespaces=ns)]
print(lci_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval for Mean']/x:category[@text='Lower Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text"
lci_str = descsout.xpath(path, namespaces=ns)
print(lci_str)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval for Mean']/x:category[@text='Upper Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number"
uci_num = [str2float(x) for x in descsout.xpath(path, namespaces=ns)]
print(uci_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval for Mean']/x:category[@text='Upper Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text"
uci_str = descsout.xpath(path, namespaces=ns)
print(uci_str)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='5% Trimmed Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number"
tms_num = [str2float(x) for x in descsout.xpath(path, namespaces=ns)]
print(tms_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='5% Trimmed Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text"
tms_str = descsout.xpath(path, namespaces=ns)
print(tms_str)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Median']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number"
meds_num = [str2float(x) for x in descsout.xpath(path, namespaces=ns)]
print(meds_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Median']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text"
meds_str = descsout.xpath(path, namespaces=ns)
print(meds_str)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Variance']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number"
varis_num = [str2float(x) for x in descsout.xpath(path, namespaces=ns)]
print(varis_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Variance']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text"
varis_str = descsout.xpath(path, namespaces=ns)
print(varis_str)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Std. Deviation']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number"
sds_num = [str2float(x) for x in descsout.xpath(path, namespaces=ns)]
print(sds_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Std. Deviation']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text"
sds_str = descsout.xpath(path, namespaces=ns)
print(sds_str)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Minimum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number"
mins_num = [str2float(x) for x in descsout.xpath(path, namespaces=ns)]
print(mins_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Minimum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text"
mins_str = descsout.xpath(path, namespaces=ns)
print(mins_str)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Maximum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number"
maxs_num = [str2float(x) for x in descsout.xpath(path, namespaces=ns)]
print(maxs_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Maximum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text"
maxs_str = descsout.xpath(path, namespaces=ns)
print(maxs_str)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number"
ranges_num = [str2float(x) for x in descsout.xpath(path, namespaces=ns)]
print(ranges_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text"
ranges_str = descsout.xpath(path, namespaces=ns)
print(ranges_str)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Interquartile Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number"
iqrs_num = [str2float(x) for x in descsout.xpath(path, namespaces=ns)]
print(iqrs_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Interquartile Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text"
iqrs_str = descsout.xpath(path, namespaces=ns)
print(iqrs_str)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number"
skews_num = [str2float(x) for x in descsout.xpath(path, namespaces=ns)]
print(skews_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text"
skews_str = descsout.xpath(path, namespaces=ns)
print(skews_str)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@number"
skewses_num = [str2float(x) for x in descsout.xpath(path, namespaces=ns)]
print(skewses_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@text"
skewses_str = descsout.xpath(path, namespaces=ns)
print(skewses_str)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number"
kurts_num = [str2float(x) for x in descsout.xpath(path, namespaces=ns)]
print(kurts_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text"
kurts_str = descsout.xpath(path, namespaces=ns)
print(kurts_str)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@number"
kurtses_num = [str2float(x) for x in descsout.xpath(path, namespaces=ns)]
print(kurtses_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@text"
kurtses_str = descsout.xpath(path, namespaces=ns)
print(kurtses_str)

statsout = etree.parse(BytesIO(outputs['Table_Tests of Normality.xml'].content))
print(etree.tostring(statsout, pretty_print=True))

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='Statistic']/x:cell/@number"
kmstats_num = [str2float(x) for x in statsout.xpath(path, namespaces=ns)]
print(kmstats_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='Statistic']/x:cell/@text"
kmstats_str = statsout.xpath(path, namespaces=ns)
print(kmstats_str)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='df']/x:cell/@number"
kmdfs = [int(x) for x in statsout.xpath(path, namespaces=ns)]
print(kmdfs)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@number"
kmsigs_num = [str2float(x) for x in statsout.xpath(path, namespaces=ns)]
print(kmsigs_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@text"
kmsigs_str = statsout.xpath(path, namespaces=ns)
print(kmsigs_str)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='Statistic']/x:cell/@number"
swstats_num = [str2float(x) for x in statsout.xpath(path, namespaces=ns)]
print(swstats_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='Statistic']/x:cell/@text"
swstats_str = statsout.xpath(path, namespaces=ns)
print(swstats_str)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='df']/x:cell/@number"
swdfs = [int(x) for x in statsout.xpath(path, namespaces=ns)]
print(swdfs)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@number"
swsigs_num = [str2float(x) for x in statsout.xpath(path, namespaces=ns)]
print(swsigs_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@text"
swsigs_str = [str2float(x) for x in statsout.xpath(path, namespaces=ns)]
print(swsigs_str)

corrout = etree.parse(BytesIO(outputs['Table_Correlations.xml'].content))
print(etree.tostring(corrout, pretty_print=True))

path = "//x:dimension[@text='Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Pearson Correlation']/x:dimension[@text='Variables']/x:category/x:cell/@number"
pearcorr_num = [str2float(x) for x in corrout.xpath(path, namespaces=ns)]
print(pearcorr_num)

path = "//x:dimension[@text='Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Pearson Correlation']/x:dimension[@text='Variables']/x:category/x:cell/@text"
pearcorr_str = corrout.xpath(path, namespaces=ns)
print(pearcorr_str)

path = "//x:dimension[@text='Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Sig. (2-tailed)']/x:dimension[@text='Variables']/x:category/x:cell/@number"
sig2tail_num = [str2float(x) for x in corrout.xpath(path, namespaces=ns)]
print(sig2tail_num)

path = "//x:dimension[@text='Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Sig. (2-tailed)']/x:dimension[@text='Variables']/x:category/x:cell/@text"
sig2tail_str = corrout.xpath(path, namespaces=ns)
print(sig2tail_str)

path = "//x:dimension[@text='Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='N']/x:dimension[@text='Variables']/x:category/x:cell/@number"
pearn_num = [str2float(x) for x in corrout.xpath(path, namespaces=ns)]
print(pearn_num)

path = "//x:dimension[@text='Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='N']/x:dimension[@text='Variables']/x:category/x:cell/@text"
pearn_str = corrout.xpath(path, namespaces=ns)
print(pearn_str)

path = "//x:dimension[@text='Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Sum of Squares and Cross-products']/x:dimension[@text='Variables']/x:category/x:cell/@number"
sscp_num = [str2float(x) for x in corrout.xpath(path, namespaces=ns)]
print(sscp_num)

path = "//x:dimension[@text='Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Sum of Squares and Cross-products']/x:dimension[@text='Variables']/x:category/x:cell/@text"
sscp_str = corrout.xpath(path, namespaces=ns)
print(sscp_str)

path = "//x:dimension[@text='Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Covariance']/x:dimension[@text='Variables']/x:category/x:cell/@number"
covar_num = [str2float(x) for x in corrout.xpath(path, namespaces=ns)]
print(covar_num)

path = "//x:dimension[@text='Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Covariance']/x:dimension[@text='Variables']/x:category/x:cell/@text"
covar_str = corrout.xpath(path, namespaces=ns)
print(covar_str)

kendall_corrout = etree.parse(BytesIO(outputs['Table_Correlations_1.xml'].content))
print(etree.tostring(kendall_corrout, pretty_print=True))

path = "//x:dimension[@text='Type']/x:category[contains(@text, 'Kendall')]/x:dimension[@text='Variables1']/x:category/x:dimension[@text='Statistics']/x:category[@text='Correlation Coefficient']/x:dimension[@text='Variables2']/x:category/x:cell/@number"
kentau_b_num = [str2float(x) for x in kendall_corrout.xpath(path, namespaces=ns)]
print(kentau_b_num)

path = "//x:dimension[@text='Type']/x:category[contains(@text, 'Kendall')]/x:dimension[@text='Variables1']/x:category/x:dimension[@text='Statistics']/x:category[@text='Correlation Coefficient']/x:dimension[@text='Variables2']/x:category/x:cell/@text"
kentau_b_str = kendall_corrout.xpath(path, namespaces=ns)
print(kentau_b_str)

path = "//x:dimension[@text='Type']/x:category[contains(@text, 'Kendall')]/x:dimension[@text='Variables1']/x:category/x:dimension[@text='Statistics']/x:category[@text='Sig. (2-tailed)']/x:dimension[@text='Variables2']/x:category/x:cell/@number"
kentau_bsig2tail_num = [str2float(x) for x in kendall_corrout.xpath(path, namespaces=ns)]
print(kentau_bsig2tail_num)

path = "//x:dimension[@text='Type']/x:category[contains(@text, 'Kendall')]/x:dimension[@text='Variables1']/x:category/x:dimension[@text='Statistics']/x:category[@text='Sig. (2-tailed)']/x:dimension[@text='Variables2']/x:category/x:cell/@text"
kentau_bsig2tail_str = kendall_corrout.xpath(path, namespaces=ns)
print(kentau_bsig2tail_str)

spearman_corrout = etree.parse(BytesIO(outputs['Table_Correlations_2.xml'].content))
print(etree.tostring(spearman_corrout, pretty_print=True))

path = "//x:dimension[@text='Type']/x:category[contains(@text, 'Spearman')]/x:dimension[@text='Variables1']/x:category/x:dimension[@text='Statistics']/x:category[@text='Correlation Coefficient']/x:dimension[@text='Variables2']/x:category/x:cell/@number"
spearrho_num = [str2float(x) for x in spearman_corrout.xpath(path, namespaces=ns)]
print(spearrho_num)

path = "//x:dimension[@text='Type']/x:category[contains(@text, 'Spearman')]/x:dimension[@text='Variables1']/x:category/x:dimension[@text='Statistics']/x:category[@text='Correlation Coefficient']/x:dimension[@text='Variables2']/x:category/x:cell/@text"
spearrho_str = spearman_corrout.xpath(path, namespaces=ns)
print(spearrho_str)

path = "//x:dimension[@text='Type']/x:category[contains(@text, 'Spearman')]/x:dimension[@text='Variables1']/x:category/x:dimension[@text='Statistics']/x:category[@text='Sig. (2-tailed)']/x:dimension[@text='Variables2']/x:category/x:cell/@number"
spearrhosig2tail_num = [str2float(x) for x in spearman_corrout.xpath(path, namespaces=ns)]
print(spearrhosig2tail_num)

path = "//x:dimension[@text='Type']/x:category[contains(@text, 'Spearman')]/x:dimension[@text='Variables1']/x:category/x:dimension[@text='Statistics']/x:category[@text='Sig. (2-tailed)']/x:dimension[@text='Variables2']/x:category/x:cell/@text"
spearrhosig2tail_str = spearman_corrout.xpath(path, namespaces=ns)
print(spearrhosig2tail_str)

# Outputs:
# Chart_000.png : Scatter plot
# Chart_1_000.png : histogram (var1)
# Chart_2_000.png : Q-Q plot (var1)
# Chart_3_000.png : Detrended Q-Q plot (var1)
# Chart_4_000.png : histogram (var2)
# Chart_5_000.png : Q-Q plot (var2)
# Chart_6_000.png : Detrended Q-Q plot (var2)
# Chart_7_000.png : boxplot (var1)
# Chart_8_000.png : boxplot (var2)

# Table_Case Processing Summary.htm : Cases (valid/missing/total)
# Table_Correlations.htm : Pearson Correlations
# Table_Correlations_1.htm : Kendall's tau_b
# Table_Correlations_2.htm : Spearman's rho
# Table_Descriptives.htm : Mean + sd
# Table_Notes_1.html : notes for examine command
# Table_Notes_2.html : notes for Pearson Correlations
# Table_Notes_3.html : notes for Kendall tau_b
# Table_Notes_4.html : notes for Spearman's rho
# Table_Tests of Normality.htm : Kolmogorov-Smirmov and Shapiro-Wilk tests

varname1short = str(var1)
varname1long = str(var1)
if var1lab != '':
    varname1long = var1lab + '[' + varname1long + ']'

varname1short = '<strong>' + varname1short + '</strong>'
varname1long = '<strong>' + varname1long + '</strong>'

varname2short = str(var2)
varname2long = str(var2)
if var2lab != '':
    varname2long = var2lab + '[' + varname2long + ']'

varname2short = '<strong>' + varname2short + '</strong>'
varname2long = '<strong>' + varname2long + '</strong>'



introhtml = '<h2>Correlation practical</h2>'
introhtml += '<p>'
introhtml += 'In this practical we will investigate whether there is a relationship between two variables by looking at how correlated they are.'
introhtml += '</p>'
outputs['introduction.html'] = HTMLOutput(introhtml, "Introduction text")

scatterprachtml = '<p>'
scatterprachtml += 'To do this we will begin by simply plotting the two variables in SPSS:'
scatterprachtml += '</p>'

scatterinstrhtml = '<ul class = "instructions">'
scatterinstrhtml += '<li>Select <strong>Scatter/Dot</strong> from the <strong>Legacy diagnostics</strong> available from the <strong>Graphs</strong> menu.</li>'
scatterinstrhtml += '<li>Select Simple Scatter and click on Define to bring up the Simple Scatterplot window.</li>'
scatterinstrhtml += '<li>Copy the ' + varname1long + ' variable into the <strong>Y Axis</strong> box.</li>'
scatterinstrhtml += '<li>Copy the ' + varname2long + ' variable into the <strong>X Axis</strong> box.</li>'
scatterinstrhtml += '<li>Click on the <strong>OK</strong> button.</li>'
scatterinstrhtml += '</ul>'

scatterprachtml += scatterinstrhtml

scatterprachtml += '<p>'
scatterprachtml += 'SPSS will then draw a scatterplot of the two variables which can be seen below:'
scatterprachtml += '</p>'

scatterprachtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_000.png'].content)  + '">'

scatterprachtml += '<p>'
if pearcorr_num[1] > 0:
    if pearcorr_num[1] > 0.5:
        scatterprachtml += 'Looking at the scatterplot there appears to be a reasonably strong positive correlation between the variables with larger values of ' + varname1short + ' associated with larger values of ' + varname2short + ' (an upward sloping relationship) as there are often in this case a considerable numbers of points in the bottom-left and top-right quarters of the plot. '
    else:
        scatterprachtml += 'Looking at the scatterplot there appears to be a positive correlation between the variables with larger values of ' + varname1short + ' associated with larger values of ' + varname2short + ' (an upward sloping relationship) but this relationship is not that strong with possibly a few more points in the bottom-left and top-right quarters of the plot. '
else:
    
    if pearcorr_num[1] < -0.5:
        scatterprachtml += 'Looking at the scatterplot there appears to be a reasonably strong negative correlation between the variables with larger values of ' + varname1short + ' associated with smaller values of ' + varname2short + ' (a downward sloping relationship) as there are often in this case a considerable numbers of points in the top-left and bottom-right quarters of the plot. '
    else:
        scatterprachtml += 'Looking at the scatterplot there appears to be a negative correlation between the variables with larger values of ' + varname1short + ' associated with smaller values of ' + varname2short + ' (a downward sloping relationship) but this relationship is not that strong with possibly a few more points in the top-left and bottom-right quarters of the plot. '
scatterprachtml += '</p>'

outputs['scatterplot.html'] = HTMLOutput(scatterprachtml, "Scatterplot text")

scatterquesthtml = '<p>'
scatterquesthtml += 'Firstly use SPSS to create a scatterplot of ' + varname1short + ' and ' + varname2short + ' and answer the following: '
scatterquesthtml += '</p>'
scatterquesthtml += '<ul>'
scatterquesthtml += '<li>Question: What does the scatterplot say about the relationship between ' + varname1short + ' and ' + varname2short + '?</li>'
scatterquesthtml += '</ul>'
outputs['scatterquestions.html'] = HTMLOutput(scatterquesthtml, "Scatter questions text")

scattersolhtml = '<p>'
scattersolhtml += 'The SPSS instructions are as follows: '
scattersolhtml += '</p>'
scattersolhtml += scatterinstrhtml
scattersolhtml += '<ul>'
scattersolhtml += '<li>Question: What does the scatterplot say about the relationship between ' + varname1short + ' and ' + varname2short + '?</li>'
scattersolhtml += '</ul>'
scattersolhtml += '<p>'
scattersolhtml += 'Solution: The output from SPSS is as follows:'
scattersolhtml += '</p>'

scattersolhtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_000.png'].content)  + '">'

scattersolhtml += '<p>'
if pearcorr_num[1] > 0:
    if pearcorr_num[1] > 0.5:
        scattersolhtml += 'Looking at the scatterplot there appears to be a reasonably strong positive correlation between the variables with larger values of ' + varname1short + ' associated with larger values of ' + varname2short + ' (an upward sloping relationship) as there are often in this case a considerable numbers of points in the bottom-left and top-right quarters of the plot. '
    else:
        scattersolhtml += 'Looking at the scatterplot there appears to be a positive correlation between the variables with larger values of ' + varname1short + ' associated with larger values of ' + varname2short + ' (an upward sloping relationship) but this relationship is not that strong with possibly a few more points in the bottom-left and top-right quarters of the plot. '
else:
    
    if pearcorr_num[1] < -0.5:
        scattersolhtml += 'Looking at the scatterplot there appears to be a reasonably strong negative correlation between the variables with larger values of ' + varname1short + ' associated with smaller values of ' + varname2short + ' (a downward sloping relationship) as there are often in this case a considerable numbers of points in the top-left and bottom-right quarters of the plot. '
    else:
        scattersolhtml += 'Looking at the scatterplot there appears to be a negative correlation between the variables with larger values of ' + varname1short + ' associated with smaller values of ' + varname2short + ' (a downward sloping relationship) but this relationship is not that strong with possibly a few more points in the top-left and bottom-right quarters of the plot. '
scattersolhtml += '</p>'
outputs['scatteranswers.html'] = HTMLOutput(scattersolhtml, "Scatterplot solution text")

normtestprachtml = '<p>'
normtestprachtml += 'We want to test whether any correlation we observe in the scatterplot is significant but there are several different correlation coefficients for different situations. '
normtestprachtml += 'The first correlation coefficient that we will look at is the Pearson correlation coefficient. '
normtestprachtml += 'This correlation requires the variables to be continuous and, in smaller samples, to be normally distributed so we will firstly look at whether a normal distribution is suitable. '
normtestprachtml += '</p>'

normtestprachtml += '<p>'
normtestprachtml += 'To do this we need to the following in SPSS:'
normtestprachtml += '</p>'

normtestinstrhtml = '<ul class = "instructions">'
normtestinstrhtml += '<li>Select <strong>Descriptive Statistics</strong> from the <strong>Analyze</strong> menu.</li>'
normtestinstrhtml += '<li>Select <strong>Explore</strong> from the <strong>Descriptive Statistics</strong> sub-menu.</li>'
normtestinstrhtml += '<li>Click on the <strong>Reset</strong> button.</li>'
normtestinstrhtml += '<li>Copy the ' + varname1long + ' and ' + varname2long + ' variables into the <strong>Dependent List:</strong> box.</li>'
normtestinstrhtml += '<li>Click on the <strong>Plots...</strong> button.</li>'
normtestinstrhtml += '<li>On the screen that appears select the <strong>Histogram</strong> tick box.</li>'
normtestinstrhtml += '<li>Unselect the <strong>Stem and leaf</strong> button.</li>'
normtestinstrhtml += '<li>Select the <strong>Normality plots with tests</strong> button.</li>'
normtestinstrhtml += '<li>Click on the <strong>Continue</strong> button.</li>'
normtestinstrhtml += '<li>Click on the <strong>OK</strong> button.</li>'
normtestinstrhtml += '</ul>'

normtestprachtml += normtestinstrhtml

normtestprachtml += '<p>'
normtestprachtml += 'This set of instructions will create a whole list of outputs - both tables and figures - in SPSS. '
normtestprachtml += 'We will focus on two figures each for our two variables and then one table. '
normtestprachtml += '</p>'

normtestprachtml += '<p>'
normtestprachtml += 'We will first look at a histogram of the variable, ' + varname1short + '. '
normtestprachtml += 'This can be found in amongst the set of output objects and looks as follows:'
normtestprachtml += '</p>'

normtestprachtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_1_000.png'].content)  + '">'

normtestprachtml += '<p>'
normtestprachtml += 'Ideally for a normal distribution this histogram should look symmetric around the mean of the distribution, in this case ' + mean_str[0] + '. '
if skews_num[0] > 2*skewses_num[0]:
    normtestprachtml += 'This distribution appears to be significantly skewed to the right (positively skewed). '
else:
    if -skews_num[0] > 2*skewses_num[0]:
        normtestprachtml += 'This distribution appears to be significantly skewed to the left (negatively skewed). '
    else:
        normtestprachtml += 'This distribution appears to be reasonably symmetric. '
normtestprachtml += '</p>'

normtestprachtml += '<p>'
normtestprachtml += 'We will next look at a histogram of the variable, ' + varname2short + '. '
normtestprachtml += 'This can also be found in amongst the set of output objects and looks as follows:'
normtestprachtml += '</p>'

normtestprachtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_4_000.png'].content)  + '">'

normtestprachtml += '<p>'
normtestprachtml += 'Again for a normal distribution this histogram should look symmetric around the mean of the distribution, in this case ' + mean_str[1] + '. ' 
if skews_num[1] > 2*skewses_num[1]:
    normtestprachtml += 'This distribution appears to be significantly skewed to the right (positively skewed). '
else:
    if -skews_num[1] > 2*skewses_num[1]:
        normtestprachtml += 'This distribution appears to be significantly skewed to the left (negatively skewed). '
    else:
        normtestprachtml += 'This distribution appears to be reasonably symmetric. '
normtestprachtml += '</p>'

normtestprachtml += '<p>'
normtestprachtml += 'We will next look at statistical tests for the two variables to see if they back up our visual impressions from the histograms. '
normtestprachtml += '</p>'

normtestprachtml += '<p>'
normtestprachtml += 'The Kolmogorov-Smirnov test is used to test the null hypothesis that a set of data comes from a normal distribution. '
normtestprachtml += 'An alternative test derived by Shapiro and Wilks is sometimes also available in SPSS but will not be described here. The available test statistics are presented in the table below that will be amongst the outputs from the Explore command:'
normtestprachtml += '</p>'

normtestprachtml += outputs['Table_Tests of Normality.htm'].content

normtestprachtml += '<p>'
normtestprachtml += 'The Kolmogorov Smirnov tests produce test statistics that are used (along with a degrees of freedom parameter) to test for normality. '
normtestprachtml += 'Here we see that the Kolmogorov Smirnov statistic takes value ' + kmstats_str[0] + ' for ' + varname1short + ' and value ' + kmstats_str[1] + ' for ' + varname2short 
normtestprachtml += 'The test has degrees of freedom which equals the number of data points, namely ' + str(kmdfs[0]) + '. '
normtestprachtml += '</p>'

normtestprachtml += '<p>'
normtestprachtml += 'For ' + varname1short + ' we see the following: '
if kmsigs_num[0] > 0.05:
    normtestprachtml += 'The p value (quoted under Sig. for Kolmogorov Smirnov  is ' + kmsigs_str[0] + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the distribution is normal. '
else:
    if kmsigs_num[0] < 0.001:
        normtestprachtml += 'The p value (quoted under Sig. for Kolmogorov Smirnov) is ' + kmsigs_str[0] + ' (reported as p &lt; .001) which is less than 0.05. '
    else:
        normtestprachtml += 'The p value (quoted under Sig. for Kolmogorov Smirnov) is ' + kmsigs_str[0] + ' which is less than 0.05. '
    normtestprachtml += 'We therefore have significant evidence to reject the null hypothesis that the variable follows a normal distribution. '
normtestprachtml += '</p>'

normtestprachtml += '<p>'
normtestprachtml += 'For ' + varname2short + ' we see the following: '
if kmsigs_num[1] > 0.05:
    normtestprachtml += 'The p value (quoted under Sig. for Kolmogorov Smirnov  is ' + kmsigs_str[1] + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the distribution is normal. '
else:
    if kmsigs_num[1] < 0.001:
        normtestprachtml += 'The p value (quoted under Sig. for Kolmogorov Smirnov) is ' + kmsigs_str[1] + ' (reported as p &lt; .001) which is less than 0.05. '
    else:
        normtestprachtml += 'The p value (quoted under Sig. for Kolmogorov Smirnov) is ' + kmsigs_str[1] + ' which is less than 0.05. '
    normtestprachtml += 'We therefore have significant evidence to reject the null hypothesis that the variable follows a normal distribution. '
normtestprachtml += '</p>'

normtestprachtml += '<p>'
normtestprachtml += 'Although the Kolmogorov Smirnov test tells the researcher whether the distribution followed by a variable is statistically significantly different from a normal distribution one should take care in not overinterpreting such findings. '
normtestprachtml += 'Significance will be strongly effected by the number of observations and so only a small discrepancy from normality will be deemed significant for very large sample sizes whilst very large discrepancies will be required to reject the null hypothesis for small sample sizes.'
normtestprachtml += 'In addition, Pearson&apos;s correlation will be robust to non-normality in the data when samples are very large, '
if Nvalid > 500:
    normtestprachtml += ' as is the case here'
normtestprachtml += '. '
normtestprachtml += '</p>'

normtestprachtml += '<p>'
normtestprachtml += 'To complete our normality checking SPSS also produces Quantile-Quantile (or QQ) plots. '
normtestprachtml += 'We can see the one for ' + varname1short + ' below:'
normtestprachtml += '</p>'

normtestprachtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_2_000.png'].content)  + '">'

normtestprachtml += '<p>'
normtestprachtml += 'QQ plots can be used to compare the distribution of a variable with a chosen distribution (typically a normal distribution as we are doing here). '
normtestprachtml += 'The data are plotted against a theoretical normal distribution (with the same mean and variance as the sample data) in such a way that the points should form an approximate straight line. '
normtestprachtml += 'Departures from this straight line indicate departures from normality. '
if kmsigs_num[0] < 0.05:
    normtestprachtml += 'As we found a significant effect in the Kolmogorov Smirnov test for ' + varname1short + ' we should see the points diverging from the line in the plot above with either some outlying values lying away from the line or even the shape of the points forming a non-linear pattern. '
else:
    normtestprachtml += 'In this case we were not able to reject the null hypothesis of normality for ' + varname1short + ' via the Kolmogorov Smirnov test and so the points should generally lie on or close to the line given. '
normtestprachtml += '</p>'

normtestprachtml += '<p>'
normtestprachtml += 'Similarly for ' + varname2short + ' its Quantile-Quantile plot can be seen below:'
normtestprachtml += '</p>'

normtestprachtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_5_000.png'].content)  + '">'

normtestprachtml += '<p>'
if kmsigs_num[1] < 0.05:
    normtestprachtml += 'As we found a significant effect in the Kolmogorov Smirnov test for ' + varname2short + ' we should see the points diverging from the line in the plot above with either some outlying values lying away from the line or even the shape of the points forming a non-linear pattern. '
else:
    normtestprachtml += 'In this case we were not able to reject the null hypothesis of normality for ' + varname2short + ' via the Kolmogorov Smirnov test and so the points should generally lie on or close to the line given. '
normtestprachtml += '</p>'

outputs['normalitytests.html'] = HTMLOutput(normtestprachtml, "Normality test text")

normtestquesthtml = '<p>'
normtestquesthtml += 'Next use SPSS and the Explore screen to create histograms, normality tests and QQ plots of ' + varname1short + ' and ' + varname2short + ' and answer the following: '
normtestquesthtml += '</p>'
normtestquesthtml += '<ul>'
normtestquesthtml += '<li>Question: What do the plots and tests tell us about the normality of ' + varname1short + '?</li>'
normtestquesthtml += '<li>Question: What do the plots and tests tell us about the normality of ' + varname2short + '?</li>'
normtestquesthtml += '</ul>'
outputs['normalitytestquestions.html'] = HTMLOutput(normtestquesthtml, "Normality test questions text")

normtestsolhtml = '<p>'
normtestsolhtml += 'The SPSS instructions are as follows: '
normtestsolhtml += '</p>'
normtestsolhtml += normtestinstrhtml
normtestsolhtml += '<p>'
normtestsolhtml += 'Question: What do the plots and tests tell us about the normality of ' + varname1short + '?'
normtestsolhtml += '</p>'
normtestsolhtml += '<p>'
normtestsolhtml += 'Solution: The output from SPSS is as follows:'
normtestsolhtml += '</p>'
normtestsolhtml += '<p>'
normtestsolhtml += 'We will first look at a histogram of the variable, ' + varname1short + '. '
normtestsolhtml += '</p>'
normtestsolhtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_1_000.png'].content)  + '">'

normtestsolhtml += '<p>'
normtestsolhtml += 'Ideally for a normal distribution this histogram should look symmetric around the mean of the distribution, in this case ' + mean_str[0] + '. '
if skews_num[0] > 2*skewses_num[0]:
    normtestsolhtml += 'This distribution appears to be significantly skewed to the right (positively skewed). '
else:
    if -skews_num[0] > 2*skewses_num[0]:
        normtestsolhtml += 'This distribution appears to be significantly skewed to the left (negatively skewed). '
    else:
        normtestsolhtml += 'This distribution appears to be reasonably symmetric. '
normtestsolhtml += '</p>'

normtestsolhtml += '<p>'
normtestsolhtml += 'Next we look at the Normality test statistics:'
normtestsolhtml += '</p>'

normtestsolhtml += outputs['Table_Tests of Normality.htm'].content

normtestsolhtml += '<p>'
normtestsolhtml += 'The Kolmogorov Smirnov tests produce test statistics that are used (along with a degrees of freedom parameter) to test for normality. '
normtestsolhtml += 'Here we see that the Kolmogorov Smirnov statistic takes value ' + kmstats_str[0] + ' for ' + varname1short + '. ' 
normtestsolhtml += 'The test has degrees of freedom which equals the number of data points, namely ' + str(kmdfs[0]) + '. '
normtestsolhtml += '</p>'

normtestsolhtml += '<p>'
normtestsolhtml += 'For ' + varname1short + ' we see the following: '
if kmsigs_num[0] > 0.05:
    normtestsolhtml += 'The p value (quoted under Sig. for Kolmogorov Smirnov  is ' + kmsigs_str[0] + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the distribution is normal. '
else:
    if kmsigs_num[0] < 0.001:
        normtestsolhtml += 'The p value (quoted under Sig. for Kolmogorov Smirnov) is ' + kmsigs_str[0] + ' (reported as p &lt; .001) which is less than 0.05. '
    else:
        normtestsolhtml += 'The p value (quoted under Sig. for Kolmogorov Smirnov) is ' + kmsigs_str[0] + ' which is less than 0.05. '
    normtestsolhtml += 'We therefore have significant evidence to reject the null hypothesis that the variable follows a normal distribution. '
normtestsolhtml += '</p>'

normtestsolhtml += '<p>'
normtestsolhtml += 'Although the Kolmogorov Smirnov test tells the researcher whether the distribution followed by a variable is statistically significantly different from a normal distribution one should take care in not overinterpreting such findings. '
normtestsolhtml += 'Significance will be strongly effected by the number of observations and so only a small discrepancy from normality will be deemed significant for very large sample sizes whilst very large discrepancies will be required to reject the null hypothesis for small sample sizes.'
normtestsolhtml += 'In addition, Pearson&apos;s correlation will be robust to non-normality in the data when samples are very large, '
if Nvalid > 500:
    normtestsolhtml += ' as is the case here'
normtestsolhtml += '. '
normtestsolhtml += '</p>'

normtestsolhtml += '<p>'
normtestsolhtml += 'For ' + varname1short + ' its Quantile-Quantile plot can be seen below:'
normtestsolhtml += '</p>'
normtestsolhtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_2_000.png'].content)  + '">'
normtestsolhtml += '<p>'
normtestsolhtml += 'QQ plots can be used to compare the distribution of a variable with a chosen distribution (typically a normal distribution as we are doing here). '
normtestsolhtml += 'The data are plotted against a theoretical normal distribution (with the same mean and variance as the sample data) in such a way that the points should form an approximate straight line. '
normtestsolhtml += 'Departures from this straight line indicate departures from normality. '
if kmsigs_num[0] < 0.05:
    normtestsolhtml += 'As we found a significant effect in the Kolmogorov Smirnov test for ' + varname1short + ' we should see the points diverging from the line in the plot above with either some outlying values lying away from the line or even the shape of the points forming a non-linear pattern. '
else:
    normtestsolhtml += 'In this case we were not able to reject the null hypothesis of normality for ' + varname1short + ' via the Kolmogorov Smirnov test and so the points should generally lie on or close to the line given. '
normtestsolhtml += '</p>'

normtestsolhtml += '<p>'
normtestsolhtml += 'Question: What do the plots and tests tell us about the normality of ' + varname2short + '?'
normtestsolhtml += '</p>'
normtestsolhtml += '<p>'
normtestsolhtml += 'Solution: The output from SPSS is as follows:'
normtestsolhtml += '</p>'
normtestsolhtml += '<p>'
normtestsolhtml += 'We will first look at a histogram of the variable, ' + varname2short + '. '
normtestsolhtml += '</p>'
normtestsolhtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_4_000.png'].content)  + '">'

normtestsolhtml += '<p>'
normtestsolhtml += 'Again for a normal distribution this histogram should look symmetric around the mean of the distribution, in this case ' + mean_str[1] + '. ' 
if skews_num[1] > 2*skewses_num[1]:
    normtestsolhtml += 'This distribution appears to be significantly skewed to the right (positively skewed). '
else:
    if -skews_num[1] > 2*skewses_num[1]:
        normtestsolhtml += 'This distribution appears to be significantly skewed to the left (negatively skewed). '
    else:
        normtestsolhtml += 'This distribution appears to be reasonably symmetric. '
normtestsolhtml += '</p>'

normtestsolhtml += '<p>'
normtestsolhtml += 'Next we look at the Normality test statistics:'
normtestsolhtml += '</p>'

normtestsolhtml += outputs['Table_Tests of Normality.htm'].content

normtestsolhtml += '<p>'
normtestsolhtml += 'The Kolmogorov Smirnov tests produce test statistics that are used (along with a degrees of freedom parameter) to test for normality. '
normtestsolhtml += 'Here we see that the Kolmogorov Smirnov statistic takes value ' + kmstats_str[1] + ' for ' + varname2short + '. '
normtestsolhtml += 'The test has degrees of freedom which equals the number of data points, namely ' + str(kmdfs[1]) + '. '
normtestsolhtml += '</p>'

normtestsolhtml += '<p>'
normtestsolhtml += 'For ' + varname2short + ' we see the following: '
if kmsigs_num[1] > 0.05:
    normtestsolhtml += 'The p value (quoted under Sig. for Kolmogorov Smirnov  is ' + kmsigs_str[1] + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the distribution is normal. '
else:
    if kmsigs_num[1] < 0.001:
        normtestsolhtml += 'The p value (quoted under Sig. for Kolmogorov Smirnov) is ' + kmsigs_str[1] + ' (reported as p &lt; .001) which is less than 0.05. '
    else:
        normtestsolhtml += 'The p value (quoted under Sig. for Kolmogorov Smirnov) is ' + kmsigs_str[1] + ' which is less than 0.05. '
    normtestsolhtml += 'We therefore have significant evidence to reject the null hypothesis that the variable follows a normal distribution. '
normtestsolhtml += '</p>'

normtestsolhtml += '<p>'
normtestsolhtml += 'Although the Kolmogorov Smirnov test tells the researcher whether the distribution followed by a variable is statistically significantly different from a normal distribution one should take care in not overinterpreting such findings. '
normtestsolhtml += 'Significance will be strongly effected by the number of observations and so only a small discrepancy from normality will be deemed significant for very large sample sizes whilst very large discrepancies will be required to reject the null hypothesis for small sample sizes.'
normtestsolhtml += 'In addition, Pearson&apos;s correlation will be robust to non-normality in the data when samples are very large, '
if Nvalid > 500:
    normtestsolhtml += ' as is the case here'
normtestsolhtml += '. '
normtestsolhtml += '</p>'

normtestsolhtml += '<p>'
normtestsolhtml += 'For ' + varname2short + ' its Quantile-Quantile plot can be seen below:'
normtestsolhtml += '</p>'
normtestsolhtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_5_000.png'].content)  + '">'
normtestsolhtml += '<p>'
if kmsigs_num[1] < 0.05:
    normtestsolhtml += 'As we found a significant effect in the Kolmogorov Smirnov test for ' + varname2short + ' we should see the points diverging from the line in the plot above with either some outlying values lying away from the line or even the shape of the points forming a non-linear pattern. '
else:
    normtestsolhtml += 'In this case we were not able to reject the null hypothesis of normality for ' + varname2short + ' via the Kolmogorov Smirnov test and so the points should generally lie on or close to the line given. '
normtestsolhtml += '</p>'

outputs['normalitytestanswers.html'] = HTMLOutput(normtestsolhtml, "Normality test answers text")

pearsonprachtml = '<p>'
pearsonprachtml += 'We will now finally turn our attention to the main topic of this practical which is the calculation of the correlation between our two variables. '
pearsonprachtml += 'SPSS offers several correlation coefficients and we will consider these here in turn. All three are available through the Analyse->Correlate->Bivariate option in SPSS. '
pearsonprachtml += '</p>'

pearsoninstrhtml = '<ul class = "instructions">'
pearsoninstrhtml += '<li>Select <strong>Bivariate...</strong> from the <strong>Correlate</strong> option available from the <strong>Analyse</strong> menu.</li>'
pearsoninstrhtml += '<li>Copy the ' + varname1long + ' and the ' + varname2long + ' variables into the <strong>Variables</strong> box.</li>'
pearsoninstrhtml += '<li>Click on the <strong>Options</strong> button and Select the <strong>Means and Standard deviations</strong> tick box.</li>'
pearsoninstrhtml += '<li>Click on the <strong>Continue</strong> button to return to main window.</li>'
pearsoninstrhtml += '<li>Click on the <strong>OK</strong> button.</li>'
pearsoninstrhtml += '</ul>'

pearsonprachtml += pearsoninstrhtml

pearsonprachtml += '<p>'
pearsonprachtml += 'The correlation command will produce two output tables. '
pearsonprachtml += 'The first table which we show below simply gives means and standard deviations for the two variables we are comparing. '
pearsonprachtml += '</p>'

pearsonprachtml += outputs['Table_Descriptive Statistics.htm'].content

pearsonprachtml += '<p>'
pearsonprachtml += 'In the next table we see the correlation matrix for the variables we are considering:'
pearsonprachtml += '</p>'

pearsonprachtml += outputs['Table_Correlations.htm'].content

pearsonprachtml += '<p>'
pearsonprachtml += 'The Correlate option can be used for more than two variables simultaneously and will then give all correlations hence the output table is in this matrix format. '
pearsonprachtml += 'The table contains three numbers for each possible correlation (including the correlations of variables with themselves which always takes the value 1). '
pearsonprachtml += 'For each correlation there is an estimate of the correlation, an accompanying p value and a sample size on which the correlation has been calculated. '
pearsonprachtml += 'Here we are interested in the Pearson correlation between ' + varname1short + ' and ' + varname2short + ' which can be found in two places in the table - either in the row for ' + varname1short + ' and column for ' + varname2short + ' or the row for ' + varname2short + ' and column for ' + varname1short + '. '
pearsonprachtml += 'Note that the SPSS table repeats exactly the same information twice, but in the write-up of results it should only be reported once! '
pearsonprachtml += '</p>'

pearsonprachtml += '<p>'
pearsonprachtml += 'In this case the correlation (reported as the statistic <em>r</em>) takes value ' + pearcorr_str[1]+ '. '
pearsonprachtml += 'The widely-used rules specified by Cohen regard a correlation of <em>r</em>=.1 as small, <em>r</em>=.3 as moderate, and <em>r</em>=.5 as large. '
if pearcorr_num[1] > 0.5:
    pearsonprachtml += 'Here, then, we see a large positive correlation. '
elif pearcorr_num[1] > 0.3:
    pearsonprachtml += 'Here, then, we see a moderate positive correlation. '
elif pearcorr_num[1] > 0:
    pearsonprachtml += 'Here, then, we see a is small but positive. '
elif pearcorr_num[1] < -0.5:
    pearsonprachtml += 'Here, then, we see a large negative correlation. '
elif pearcorr_num[1] < -0.3:
    pearsonprachtml += 'Here, then, we see a moderate negative correlation. '
else:
    pearsonprachtml += 'This correlation is small but negative. '
pearsonprachtml += 'The correlation is given in the table, along with a significance value and a sample size which in this case is ' + pearn_str[1] + '. '
pearsonprachtml += 'This is the number of observations in which both ' + varname1short + ' and ' + varname2short + ' where observed. '
pearsonprachtml += '</p>'

pearsonprachtml += '<p>'
pearsonprachtml += 'We can test if this correlation is significantly different from zero which will depend on (i) the magnitude of the correlation and (ii) the number of observations on which the correlation is based. '
pearsonprachtml += '</p>'

pearsonprachtml += '<p>'
if sig2tail_num[1] > 0.05:
    pearsonprachtml += 'The p value (quoted under Sig. (2-tailed)) is ' + sig2tail_str[1] + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the correlation is 0. '
else:
    if sig2tail_num[1] < 0.001:
        pearsonprachtml += 'The p value (quoted under Sig. (2-tailed)) is ' + sig2tail_str[1] + ' (reported as p &lt; .001) which is less than 0.05. '
    else:
        pearsonprachtml += 'The p value (quoted under Sig. (2-tailed)) is ' + sig2tail_str[1] + ' which is less than 0.05. '
    pearsonprachtml += 'We therefore have significant evidence to reject the null hypothesis that the correlation is 0. '
pearsonprachtml += '</p>'

pearsonprachtml += '<p>'
pearsonprachtml += 'We would report the result as follows: '
pearsonprachtml += 'The variables ' + varname1short + ' and ' + varname2short + ' '
if sig2tail_num[1] > 0.05:
    pearsonprachtml += 'were not significantly correlated '
else:
    if sig2tail_num[1] < 0.001:
        pearsonprachtml += 'were significantly '
    else:
        pearsonprachtml += 'were significantly '
    if pearcorr_num[1] > 0.5:
        pearsonprachtml += 'and positively correlated. '
    elif pearcorr_num[1] > 0.3:
        pearsonprachtml += 'and moderately positively correlated '
    elif pearcorr_num[1] > 0:
        pearsonprachtml += 'and slightly positively correlated '
    elif pearcorr_num[1] < -0.5:
        pearsonprachtml += 'and negatively correlated '
    elif pearcorr_num[1] < -0.3:
        pearsonprachtml += 'and moderately negatively correlated '
    else:
        pearsonprachtml += 'and slightly negatively correlated '
pearsonprachtml += '<em>r</em> = ' + pearcorr_str[1] + ', N = ' + str(Nvalid) +  ', '
if sig2tail_num[1] > 0.05:
    pearsonprachtml += 'p = ' + sig2tail_str[1] + '. '
else:
    if sig2tail_num[1] < 0.001:
        pearsonprachtml += 'p &lt; .001' + '. '
    else:
        pearsonprachtml += 'p = ' + sig2tail_str[1] + '. '
pearsonprachtml += 'Note there is no need for a table when reporting a single correlation. '
pearsonprachtml += '</p>'

pearsonprachtml += '<p>'
pearsonprachtml += 'The Pearson correlation coefficient is appropriate to use when both variables can be assumed to follow a normal distribution or when samples are very large. '
pearsonprachtml += '</p>'
outputs['pearson.html'] = HTMLOutput(pearsonprachtml, "Pearson text")

pearsonquesthtml = '<p>'
pearsonquesthtml += 'Next use SPSS and the Correlate screen to answer the following: '
pearsonquesthtml += '</p>'

pearsonquesthtml += '<ul>'
pearsonquesthtml += '<li>Question: What is the Pearson correlation coefficient between ' + varname1short + ' and ' + varname2short + ' and is it significant?</li>'
pearsonquesthtml += '</ul>'

outputs['pearsonquestions.html'] = HTMLOutput(pearsonquesthtml , "Pearson question text")

pearsonsolhtml = '<p>'
pearsonsolhtml += 'The SPSS instructions are as follows: '
pearsonsolhtml += '</p>'
pearsonsolhtml += pearsoninstrhtml
pearsonsolhtml += '<ul>'
pearsonsolhtml += '<li>Question: What is the Pearson correlation coefficient between ' + varname1short + ' and ' + varname2short + ' and is it significant?</li>'
pearsonsolhtml += '</ul>'
pearsonsolhtml += '<p>'
pearsonsolhtml += 'Solution: The output from SPSS is as follows: '
pearsonsolhtml += '</p>'
pearsonsolhtml += outputs['Table_Descriptive Statistics.htm'].content
pearsonsolhtml += outputs['Table_Correlations.htm'].content
pearsonsolhtml += '<p>'
pearsonsolhtml += 'Here we are interested in the Pearson correlation between ' + varname1short + ' and ' + varname2short + ' which can be found in two places in the table - either in the row for ' + varname1short + ' and column for ' + varname2short + ' or the row for ' + varname2short + ' and column for ' + varname1short + '. '
pearsonsolhtml += '</p>'
pearsonsolhtml += '<p>'
pearsonsolhtml += 'In this case the correlation takes value ' + pearcorr_str[1]+ '. '   
if pearcorr_num[1] > 0.5:
    pearsonsolhtml += 'This represents a large positive correlation. '
elif pearcorr_num[1] > 0.3:
    pearsonsolhtml += 'This represents a moderate positive correlation. '
elif pearcorr_num[1] > 0:
    pearsonsolhtml += 'This correlation is small but positive. '
elif pearcorr_num[1] < -0.5:
    pearsonsolhtml += 'This represents a large negative correlation. '
elif pearcorr_num[1] < -0.3:
    pearsonsolhtml += 'This represents a moderate negative correlation. '
else:
    pearsonsolhtml += 'This correlation is small but negative. '
pearsonsolhtml += 'The correlation is given in the table, along with a significance value and a sample size which in this case is ' + pearn_str[1] + '. '
pearsonsolhtml += 'This is the number of observations in which both ' + varname1short + ' and ' + varname2short + ' where observed. '
pearsonsolhtml += '</p>'

pearsonsolhtml += '<p>'
pearsonsolhtml += 'We can test if this correlation is significantly different from zero which will depend on (i) the magnitude of the correlation and (ii) the number of observations on which the correlation is based. '
pearsonsolhtml += '</p>'

pearsonsolhtml += '<p>'
if sig2tail_num[1] > 0.05:
    pearsonsolhtml += 'The p value (quoted under Sig. (2-tailed) is ' + sig2tail_str[1] + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the correlation is 0. '
else:
    if sig2tail_num[1] < 0.001:
        pearsonsolhtml += 'The p value (quoted under Sig. (2-tailed)) is ' + sig2tail_str[1] + ' (reported as p &lt; .001) which is less than 0.05. '
    else:
        pearsonsolhtml += 'The p value (quoted under Sig. (2-tailed)) is ' + sig2tail_str[1] + ' which is less than 0.05. '
    pearsonsolhtml += 'We therefore have significant evidence to reject the null hypothesis that the correlation is 0. '
pearsonsolhtml += '</p>'

outputs['pearsonanswers.html'] = HTMLOutput(pearsonsolhtml , "Pearson answer text")

spearmanprachtml = '<p>'
spearmanprachtml += 'If this is not the case then an alternative is the Spearman rank correlation. '
spearmanprachtml += 'This correlation works in much the same way as the Pearson coefficient but is calculated on the ranks of the data points rather than the points themselves. '
spearmanprachtml += 'To calculate the Spearman correlation we need to return to the Bivariate screen and do the following:'
spearmanprachtml += '</p>'

spearmaninstrhtml = '<ul class = "instructions">'
spearmaninstrhtml += '<li>Select <strong>Bivariate...</strong> from the <strong>Correlate</strong> option available from the <strong>Analyse</strong> menu.</li>'
spearmaninstrhtml += '<li>Check that the ' + varname1long + ' and the ' + varname2long + ' variables are still in the <strong>Variables</strong> box.</li>'
spearmaninstrhtml += '<li>Deselect the <strong>Pearson</strong> tick box.</li>'
spearmaninstrhtml += '<li>Select the <strong>Spearman</strong> tick box.</li>'
spearmaninstrhtml += '<li>Click on the <strong>OK</strong> button.</li>'
spearmaninstrhtml += '</ul>'

spearmanprachtml += spearmaninstrhtml 

spearmanprachtml += '<p>'
spearmanprachtml += 'In the table produced we see the correlation matrix for the variables we are considering:'
spearmanprachtml += '</p>'

spearmanprachtml += outputs['Table_Correlations_2.htm'].content

spearmanprachtml += '<p>'
spearmanprachtml += 'For each correlation there is once again an estimate of the correlation, an accompanying p value and a sample size on which the correlation has been calculated. '
spearmanprachtml += 'Here we are interested in the Spearman correlation between ' + varname1short + ' and ' + varname2short + ' is repeated in two places in the table - either in the row for ' + varname1short + ' and column for ' + varname2short + ' or the row for ' + varname2short + ' and column for ' + varname1short + '. '
spearmanprachtml += '</p>'

spearmanprachtml += '<p>'
spearmanprachtml += 'In this case the correlation (reported as the statistics rho) takes value ' + spearrho_str[1] + '. '
if spearrho_num[1] > 0.5:
    spearmanprachtml += 'This represents a large positive correlation. '
elif spearrho_num[1] > 0.3:
    spearmanprachtml += 'This represents a moderate positive correlation. '
elif spearrho_num[1] > 0:
    spearmanprachtml += 'This correlation is small but positive. '
elif spearrho_num[1] < -0.5:
    spearmanprachtml += 'This represents a large negative correlation. '
elif spearrho_num[1] < -0.3:
    spearmanprachtml += 'This represents a moderate negative correlation. '
else:
    spearmanprachtml += 'This correlation is small but negative. '
spearmanprachtml += 'The correlation is given in the table, along with a significance value and a sample size which in this case is ' + pearn_str[1] + '. '
spearmanprachtml += 'This is the number of observations in which both ' + varname1short + ' and ' + varname2short + ' where observed. '
spearmanprachtml += '</p>'

spearmanprachtml += '<p>'
spearmanprachtml += 'We can test if this correlation is significantly different from zero which will depend on (i) the magnitude of the correlation and (ii) the number of observations on which the correlation is based.'
spearmanprachtml += '</p>'

spearmanprachtml += '<p>'
if spearrhosig2tail_num[1] > 0.05:
    spearmanprachtml += 'The p value (quoted under Sig. (2-tailed)) is ' + spearrhosig2tail_str[1] + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the correlation is 0. '
else:
    if spearrhosig2tail_num[1] < 0.001:
        spearmanprachtml += 'The p value (quoted under Sig. (2-tailed)) is ' + spearrhosig2tail_str[1] + ' (reported as p &lt; .001) which is less than 0.05. '
    else:
        spearmanprachtml += 'The p value (quoted under Sig. (2-tailed)) is ' + spearrhosig2tail_str[1] + ' which is less than 0.05. '
    spearmanprachtml += 'We therefore have significant evidence to reject the null hypothesis that the correlation is 0. '
spearmanprachtml += '</p>'

spearmanprachtml += '<p>'
spearmanprachtml += 'We would report the result as follows: '
spearmanprachtml += 'The variables ' + varname1short + ' and ' + varname2short + ' '
if spearrhosig2tail_num[1]  > 0.05:
    spearmanprachtml += 'were not significantly correlated '
else:
    if spearrhosig2tail_num[1]  < 0.001:
        spearmanprachtml += 'were significantly '
    else:
        spearmanprachtml += 'were significantly '
    if spearrho_num[1] > 0.5:
        spearmanprachtml += 'and positively correlated '
    elif spearrho_num[1] > 0.3:
        spearmanprachtml += 'and moderatly positively correlated '
    elif spearrho_num[1] > 0:
        spearmanprachtml += 'and slightly positively correlated '
    elif spearrho_num[1] < -0.5:
        spearmanprachtml += 'and negatively correlated '
    elif spearrho_num[1] < -0.3:
        spearmanprachtml += 'and moderatly negatively correlated '
    else:
        spearmanprachtml += 'and slightly negatively correlated '
spearmanprachtml += '<em>r</em> = ' + spearrho_str[1] + ', N = ' + str(Nvalid) + ', '
if spearrhosig2tail_num[1] > 0.05:
    spearmanprachtml += 'p = ' + spearrhosig2tail_str[1] + '. '
else:
    if spearrhosig2tail_num[1] < 0.001:
        spearmanprachtml += 'p &lt; .001' + '. '
    else:
        spearmanprachtml += 'p = ' + spearrhosig2tail_str[1] + '. '
spearmanprachtml += '</p>'
outputs['spearman.html'] = HTMLOutput(spearmanprachtml, "Spearman text")


spearmanquesthtml = '<p>'
spearmanquesthtml += 'Next use SPSS and the Correlate screen to answer the following: '
spearmanquesthtml += '</p>'
spearmanquesthtml += '<ul>'
spearmanquesthtml += '<li>Question: What is the Spearman correlation coefficient between ' + varname1short + ' and ' + varname2short + ' and is it significant?</li>'
spearmanquesthtml += '</ul>'

outputs['spearmanquestions.html'] = HTMLOutput(spearmanquesthtml , "Spearman question text")

spearmansolhtml = '<p>'
spearmansolhtml += 'The SPSS instructions are as follows: '
spearmansolhtml += '</p>'
spearmansolhtml += spearmaninstrhtml
spearmansolhtml += '<ul>'
spearmansolhtml += '<li>Question: What is the Spearman correlation coefficient between ' + varname1short + ' and ' + varname2short + ' and is it significant?</li>'
spearmansolhtml += '</ul>'
spearmansolhtml += '<p>'
spearmansolhtml += 'Solution: The output from SPSS is as follows: '
spearmansolhtml += '</p>'
spearmansolhtml += outputs['Table_Correlations_2.htm'].content
spearmansolhtml += '<p>'
spearmansolhtml += 'Here we are interested in the Spearman correlation between ' + varname1short + ' and ' + varname2short + ' which can be found in two places in the table - either in the row for ' + varname1short + ' and column for ' + varname2short + ' or the row for ' + varname2short + ' and column for ' + varname1short + '. '
spearmansolhtml += '</p>'

spearmansolhtml += '<p>'
spearmansolhtml += 'In this case the correlation takes value ' + spearrho_str[1] + '. '   
if spearrho_num[1] > 0.5:
    spearmansolhtml += 'This represents a large positive correlation. '
elif spearrho_num[1] > 0.3:
    spearmansolhtml += 'This represents a moderate positive correlation. '
elif spearrho_num[1] > 0:
    spearmansolhtml += 'This correlation is small but positive. '
elif spearrho_num[1] < -0.5:
    spearmansolhtml += 'This represents a large negative correlation. '
elif spearrho_num[1] < -0.3:
    spearmansolhtml += 'This represents a moderate negative correlation. '
else:
    spearmansolhtml += 'This correlation is small but negative. '
spearmansolhtml += 'The correlation is given in the table, along with a significance value and a sample size which in this case is ' + pearn_str[1] + '. '
spearmansolhtml += 'This is the number of observations in which both ' + varname1short + ' and ' + varname2short + ' where observed. '
spearmansolhtml += '</p>'

spearmansolhtml += '<p>'
spearmansolhtml += 'We can test if this correlation is significantly different from zero which will depend on (i) the magnitude of the correlation and (ii) the number of observations on which the correlation is based.'
spearmansolhtml += '</p>'

spearmansolhtml += '<p>'
if spearrhosig2tail_num[1] > 0.05:
    spearmansolhtml += 'The p value (quoted under Sig. (2-tailed)) is ' + spearrhosig2tail_str[1] + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the correlation is 0. '
else:
    if spearrhosig2tail_num[1] < 0.001:
        spearmansolhtml += 'The p value (quoted under Sig. (2-tailed)) is ' + spearrhosig2tail_str[1] + ' (reported as p &lt; .001) which is less than 0.05. '
    else:
        spearmansolhtml += 'The p value (quoted under Sig. (2-tailed)) is ' + spearrhosig2tail_str[1] + ' which is less than 0.05. '
    spearmansolhtml += 'We therefore have significant evidence to reject the null hypothesis that the correlation is 0. '
spearmansolhtml += '</p>'

outputs['spearmananswers.html'] = HTMLOutput(spearmansolhtml , "Spearman answer text")

kendallprachtml = '<p>'
kendallprachtml += 'The third possible correlation is known as Kendalls Tau-b and has desirable properties when the variables take values that are ordered categories (i.e. ordinal variables). '
kendallprachtml += 'To calculate the Kendalls Tau-b we need to return to the Bivariate screen and do the following:'
kendallprachtml += '</p>'

kendallinstrhtml = '<ul class = "instructions">'
kendallinstrhtml += '<li>Select <strong>Bivariate...</strong> from the <strong>Correlate </strong> option available from the <strong>Analyse</strong> menu.</li>'
kendallinstrhtml += '<li>Check that the ' + varname1long + ' and the ' + varname2long + ' variables are still in the <strong>Variables</strong> box.</li>'
kendallinstrhtml += '<li>Deselect the <strong>Spearman</strong> tick box.</li>'
kendallinstrhtml += '<li>Select the <strong>Kendall tau-b</strong> tick box.</li>'
kendallinstrhtml += '<li>Click on the <strong>OK</strong> button.</li>'
kendallinstrhtml += '</ul>'

kendallprachtml += kendallinstrhtml

kendallprachtml += '<p>'
kendallprachtml += 'In the table produced we see the correlation matrix for the variables we are considering:'
kendallprachtml += '</p>'

kendallprachtml += outputs['Table_Correlations_1.htm'].content

kendallprachtml += '<p>'
kendallprachtml += 'As in the previous correlation tables, for each pair of variables there is once again an estimate of the correlation, an accompanying p value and a sample size on which the correlation has been calculated, all repeated in two places in the table. '
kendallprachtml += '</p>'

kendallprachtml += '<p>'
kendallprachtml += 'In this case the correlation (reported as the statistic tau) takes value ' + kentau_b_str[1]+ '. '   
if kentau_b_num[1] > 0.5:
    kendallprachtml += 'This represents a large positive correlation. '
elif kentau_b_num[1] > 0.3:
    kendallprachtml += 'This represents a moderate positive correlation. '
elif kentau_b_num[1] > 0:
    kendallprachtml += 'This correlation is small but positive. '
elif kentau_b_num[1] < -0.5:
    kendallprachtml += 'This represents a large negative correlation. '
elif kentau_b_num[1] < -0.3:
    kendallprachtml += 'This represents a moderate negative correlation. '
else:
    kendallprachtml += 'This correlation is small but negative. '
kendallprachtml += 'As before, the correlation coefficient is accompanied by the sample size used in the calculation and the significance value will depend on (i) the magnitude of the correlation and (ii) the number of observations on which the correlation is based.'
kendallprachtml += '</p>'

kendallprachtml += '<p>'
if kentau_bsig2tail_num[1] > 0.05:
    kendallprachtml += 'The p value (quoted under Sig. (2-tailed)) is ' + kentau_bsig2tail_str[1] + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the correlation is 0. '
else:
    if kentau_bsig2tail_num[1] < 0.001:
        kendallprachtml += 'The p value (quoted under Sig. (2-tailed)) is ' + kentau_bsig2tail_str[1] + ' (reported as p &lt; .001) which is less than 0.05. '
    else:
        kendallprachtml += 'The p value (quoted under Sig. (2-tailed)) is ' + kentau_bsig2tail_str[1] + ' which is less than 0.05. '
    kendallprachtml += 'We therefore have significant evidence to reject the null hypothesis that the correlation is 0. '

kendallprachtml += '<p>'
kendallprachtml += 'We would report the result as follows: '
kendallprachtml += 'The variables ' + varname1short + ' and ' + varname2short + ' '
if kentau_bsig2tail_num[1]  > 0.05:
    kendallprachtml += 'were not significantly correlated '
else:
    if kentau_bsig2tail_num[1]  < 0.001:
        kendallprachtml += 'were significantly '
    else:
        kendallprachtml += 'were significantly '
    if kentau_b_num[1] > 0.5:
        kendallprachtml += 'and positively correlated '
    elif kentau_b_num[1] > 0.3:
        kendallprachtml += 'and moderatly positively correlated '
    elif kentau_b_num[1] > 0:
        kendallprachtml += 'and slightly positively correlated '
    elif kentau_b_num[1] < -0.5:
        kendallprachtml += 'and negatively correlated '
    elif kentau_b_num[1] < -0.3:
        kendallprachtml += 'and moderatly negatively correlated '
    else:
        kendallprachtml += 'and slightly negatively correlated '
kendallprachtml += '<em>r</em> = ' + kentau_b_str[1] + ', N = ' + str(Nvalid) + ', '
if kentau_bsig2tail_num[1] > 0.05:
    kendallprachtml += 'p = ' + kentau_bsig2tail_str[1] + '. '
else:
    if kentau_bsig2tail_num[1] < 0.001:
        kendallprachtml += 'p &lt; .001' + '. '
    else:
        kendallprachtml += 'p = ' + kentau_bsig2tail_str[1] + '. '
kendallprachtml += '</p>'
outputs['kendalltaub.html'] = HTMLOutput(kendallprachtml, "Kendall tau-b text")

kendallquesthtml = '<p>'
kendallquesthtml += 'Next use SPSS and the Correlate screen to answer the following: '
kendallquesthtml += '</p>'


kendallquesthtml += '<ul>'
kendallquesthtml += '<li>Question: What is the value of the Kendall tau-b correlation coefficient between ' + varname1short + ' and ' + varname2short + ' and is it significant?</li>'
kendallquesthtml += '</ul>'

outputs['kendallquestions.html'] = HTMLOutput(kendallquesthtml , "Kendall tau-b question text")

kendallsolhtml = '<p>'
kendallsolhtml += 'The SPSS instructions are as follows: '
kendallsolhtml += '</p>'
kendallsolhtml += kendallinstrhtml
kendallsolhtml += '<ul>'
kendallsolhtml += '<li>Question: What is the value of the Kendall tau-b correlation coefficient between ' + varname1short + ' and ' + varname2short + ' and is it significant?</li>'
kendallsolhtml += '</ul>'
kendallsolhtml += '<p>'
kendallsolhtml += 'Solution: The output from SPSS is as follows: '
kendallsolhtml += '</p>'
kendallsolhtml += outputs['Table_Correlations_1.htm'].content

kendallsolhtml += '<p>'
kendallsolhtml += 'Here we are interested in the Kendall Tau-b correlation between ' + varname1short + ' and ' + varname2short + ' which can be found in two places in the table - either in the row for ' + varname1short + ' and column for ' + varname2short + ' or the row for ' + varname2short + ' and column for ' + varname1short + '. '
kendallsolhtml += '</p>'

kendallsolhtml += '<p>'
kendallsolhtml += 'In this case the correlation takes value ' + kentau_b_str[1]+ '. '   
if kentau_b_num[1] > 0.5:
    kendallsolhtml += 'This represents a large positive correlation. '
elif kentau_b_num[1] > 0.3:
    kendallsolhtml += 'This represents a moderate positive correlation. '
elif kentau_b_num[1] > 0:
    kendallsolhtml += 'This correlation is small but positive. '
elif kentau_b_num[1] < -0.5:
    kendallsolhtml += 'This represents a large negative correlation. '
elif kentau_b_num[1] < -0.3:
    kendallsolhtml += 'This represents a moderate negative correlation. '
else:
    kendallsolhtml += 'This correlation is small but negative. '
kendallsolhtml += 'The correlation is given in the table, along with a significance value and a sample size which in this case is ' + pearn_str[1] + '. '
kendallsolhtml += 'This is the number of observations in which both ' + varname1short + ' and ' + varname2short + ' where observed. '
kendallsolhtml += '</p>'

kendallsolhtml += '<p>'
kendallsolhtml += 'We can test if this correlation is significantly different from zero which will depend on (i) the magnitude of the correlation and (ii) the number of observations on which the correlation is based.'
kendallsolhtml += '</p>'

kendallsolhtml += '<p>'
if kentau_bsig2tail_num[1] > 0.05:
    kendallsolhtml += 'The p value (quoted under Sig. (2-tailed)) is ' + kentau_bsig2tail_str[1] + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the correlation is 0. '
else:
    if kentau_bsig2tail_num[1] < 0.001:
        kendallsolhtml += 'The p value (quoted under Sig. (2-tailed)) is ' + kentau_bsig2tail_str[1] + ' (reported as p &lt; .001) which is less than 0.05. '
    else:
        kendallsolhtml += 'The p value (quoted under Sig. (2-tailed)) is ' + kentau_bsig2tail_str[1] + ' which is less than 0.05. '
    kendallsolhtml += 'We therefore have significant evidence to reject the null hypothesis that the correlation is 0. '
kendallsolhtml += '</p>'

outputs['kendallanswers.html'] = HTMLOutput(kendallsolhtml , "Kendall tau-b answer text")

conchtml = '<p>'
conchtml += 'This ends our practical on correlations.'
conchtml += '</p>'
outputs['conclusion.html'] = HTMLOutput(conchtml, "Conclusion text")


combinedprachtml = outputs['introduction.html'].content
combinedprachtml += outputs['scatterplot.html'].content
combinedprachtml += outputs['normalitytests.html'].content
combinedprachtml += outputs['pearson.html'].content
combinedprachtml += outputs['spearman.html'].content
combinedprachtml += outputs['kendalltaub.html'].content
combinedprachtml += outputs['conclusion.html'].content

outputs['allpractical.html'] = HTMLOutput(combinedprachtml, "Combined practical HTML outputs")

combinedquesthtml = outputs['introduction.html'].content
combinedquesthtml += outputs['scatterquestions.html'].content
combinedquesthtml += outputs['normalitytestquestions.html'].content
combinedquesthtml += outputs['pearsonquestions.html'].content
combinedquesthtml += outputs['spearmanquestions.html'].content
combinedquesthtml += outputs['kendallquestions.html'].content

outputs['allquestions.html'] = HTMLOutput(combinedquesthtml, "Combined questions HTML outputs")

combinedsolhtml = outputs['scatteranswers.html'].content
combinedsolhtml += outputs['normalitytestanswers.html'].content
combinedsolhtml += outputs['pearsonanswers.html'].content
combinedsolhtml += outputs['spearmananswers.html'].content
combinedsolhtml += outputs['kendallanswers.html'].content

outputs['allanswers.html'] = HTMLOutput(combinedsolhtml, "Combined solutions HTML outputs")

'''