# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class Regression2(Template):
    'Calculate More advanced descriptive Statistics in SPSS (equivalent to Analyze->Descriptive Statistics->Explore... in SPSS 23)'

    __version__ = '1.0.0'

    tags = [ 'SPSS']
    engines = ['SPSS_script']

    inputs = '''
var1 = DataVector('First categorical variable: ', help='Primary (axis) categories for charts')
var2 = DataVector('Second categorical variable: ', help='Cluster categories (bars) for charts')
'''

    spssscript = '''
FREQUENCIES VARIABLE=${var1}.

FREQUENCIES VARIABLE=${var2}.

GRAPH
 /BAR(SIMPLE)= COUNT BY ${var1}.

GRAPH
 /PIE=COUNT BY ${var1}.

GRAPH
 /BAR(SIMPLE)= COUNT BY ${var2}.

GRAPH
 /BAR(GROUPED)=COUNT BY ${var1} BY ${var2}.

GRAPH
 /BAR(GROUPED)=PCT BY ${var1} BY ${var2}.

CROSSTABS
  /TABLES=${var1} BY ${var2}
  /FORMAT=AVALUE TABLES
  /CELLS=COUNT COLUMN
  /COUNT ROUND CELL.

'''


    processoutputs = '''
import EStat
from EStat.Templating import *
import base64

from lxml import etree
from io import BytesIO

import numpy
import random

ns = {'x': 'http://www.ibm.com/software/analytics/spss/xml/oms'}

def str2float(s):
    try:
        return float(s)
    except ValueError:
        return float('nan')

# First variable information
statsout1 = etree.parse(BytesIO(outputs['Table_Statistics.xml'].content))
print(etree.tostring(statsout1, pretty_print=True))

path = "string(//x:dimension[@text='Variables']/x:category/@varName)"
varname1 = statsout1.xpath(path, namespaces=ns)
print(varname1)

path = "string(//x:dimension[@text='Variables']/x:category/@label)"
varlab1 = statsout1.xpath(path, namespaces=ns)
print(varlab1)

path = "string(//x:group[@text='N']/x:category[@text='Valid']/x:cell/@number)"
Nvalid1 = int(statsout1.xpath(path, namespaces=ns))
print(Nvalid1)

path = "string(//x:group[@text='N']/x:category[@text='Missing']/x:cell/@number)"
Nmiss1 = int(statsout1.xpath(path, namespaces=ns))
print(Nmiss1)

# Second variable information
statsout2 = etree.parse(BytesIO(outputs['Table_Statistics_1.xml'].content))
print(etree.tostring(statsout2, pretty_print=True))

path = "string(//x:dimension[@text='Variables']/x:category/@varName)"
varname2 = statsout2.xpath(path, namespaces=ns)
print(varname2)

path = "string(//x:dimension[@text='Variables']/x:category/@label)"
varlab2 = statsout2.xpath(path, namespaces=ns)
print(varlab2)

path = "string(//x:group[@text='N']/x:category[@text='Valid']/x:cell/@number)"
Nvalid2 = int(statsout2.xpath(path, namespaces=ns))
print(Nvalid2)

path = "string(//x:group[@text='N']/x:category[@text='Missing']/x:cell/@number)"
Nmiss2 = int(statsout2.xpath(path, namespaces=ns))
print(Nmiss2)

# First variable frequencies
statsout1 = etree.parse(BytesIO(outputs['Table_Frequencies.xml'].content))
print(etree.tostring(statsout1, pretty_print=True))

path = "//x:dimension[@varName='" + var1 + "']/x:group[@text='Valid']/x:group/x:group/x:category/@text"
vals1 = [str(x) for x in statsout1.xpath(path, namespaces=ns)]
print(vals1)

path = "//x:dimension[@varName='" + var1 + "']/x:group[@text='Valid']/x:group/x:group/x:category/x:dimension[@axis='column']/x:category[@text='Frequency']/x:cell/@number"
freqs1 = [int(x) for x in statsout1.xpath(path, namespaces=ns)]
print(freqs1)

path = "//x:dimension[@varName='" + var1 + "']/x:group[@text='Valid']/x:group/x:group/x:category/x:dimension[@axis='column']/x:category[@text='Percent']/x:cell/@number"
allcents1_num = [str2float(x) for x in statsout1.xpath(path, namespaces=ns)]
print(allcents1_num)

path = "//x:dimension[@varName='" + var1 + "']/x:group[@text='Valid']/x:group/x:group/x:category/x:dimension[@axis='column']/x:category[@text='Percent']/x:cell/@text"
allcents1_str = statsout1.xpath(path, namespaces=ns)
print(allcents1_str)

path = "//x:dimension[@varName='" + var1 + "']/x:group[@text='Valid']/x:group/x:group/x:category/x:dimension[@axis='column']/x:category[@text='Valid Percent']/x:cell/@number"
validcents1_num = [str2float(x) for x in statsout1.xpath(path, namespaces=ns)]
print(validcents1_num)

path = "//x:dimension[@varName='" + var1 + "']/x:group[@text='Valid']/x:group/x:group/x:category/x:dimension[@axis='column']/x:category[@text='Valid Percent']/x:cell/@text"
validcents1_str = statsout1.xpath(path, namespaces=ns)
print(validcents1_str)

path = "//x:dimension[@varName='" + var1 + "']/x:group[@text='Valid']/x:group/x:group/x:category/x:dimension[@axis='column']/x:category[@text='Cumulative Percent']/x:cell/@number"
cumcents1_num = [str2float(x) for x in statsout1.xpath(path, namespaces=ns)]
print(cumcents1_num)

path = "//x:dimension[@varName='" + var1 + "']/x:group[@text='Valid']/x:group/x:group/x:category/x:dimension[@axis='column']/x:category[@text='Cumulative Percent']/x:cell/@text"
cumcents1_str = statsout1.xpath(path, namespaces=ns)
print(cumcents1_str)

path = "string(//x:dimension[@varName='" + var1 + "']/x:group[@text='Valid']/x:category[@text='Total']/x:dimension[@axis='column']/x:category[@text='Frequency']/x:cell/@number)"
totfreq1 = int(statsout1.xpath(path, namespaces=ns))
print(totfreq1)

path = "string(//x:dimension[@varName='" + var1 + "']/x:group[@text='Valid']/x:category[@text='Total']/x:dimension[@axis='column']/x:category[@text='Percent']/x:cell/@number)"
totpercent1_num = str2float(statsout1.xpath(path, namespaces=ns))
print(totpercent1_num)

path = "string(//x:dimension[@varName='" + var1 + "']/x:group[@text='Valid']/x:category[@text='Total']/x:dimension[@axis='column']/x:category[@text='Percent']/x:cell/@text)"
totpercent1_str = statsout1.xpath(path, namespaces=ns)
print(totpercent1_str)

path = "string(//x:dimension[@varName='" + var1 + "']/x:group[@text='Valid']/x:category[@text='Total']/x:dimension[@axis='column']/x:category[@text='Valid Percent']/x:cell/@number)"
totvalidpercent1_num = str2float(statsout1.xpath(path, namespaces=ns))
print(totvalidpercent1_num)

path = "string(//x:dimension[@varName='" + var1 + "']/x:group[@text='Valid']/x:category[@text='Total']/x:dimension[@axis='column']/x:category[@text='Valid Percent']/x:cell/@text)"
totvalidpercent1_str = statsout1.xpath(path, namespaces=ns)
print(totvalidpercent1_str)

# FIXME: This only picks up missing values using system missing values
path = "string(//x:dimension[@varName='" + var1 + "']/x:group[@text='Missing']/x:category[@text='System']/x:dimension[@axis='column']/x:category[@text='Frequency']/x:cell/@number)"
totmissfreq1_num = str2float(statsout1.xpath(path, namespaces=ns))
print(totmissfreq1_num)

path = "string(//x:dimension[@varName='" + var1 + "']/x:group[@text='Missing']/x:category[@text='System']/x:dimension[@axis='column']/x:category[@text='Frequency']/x:cell/@text)"
totmissfreq1_str = statsout1.xpath(path, namespaces=ns)
print(totmissfreq1_str)

path = "string(//x:dimension[@varName='" + var1 + "']/x:group[@text='Missing']/x:category[@text='System']/x:dimension[@axis='column']/x:category[@text='Percent']/x:cell/@number)"
totmisspercent1_num = str2float(statsout1.xpath(path, namespaces=ns))
print(totmisspercent1_num)

path = "string(//x:dimension[@varName='" + var1 + "']/x:group[@text='Missing']/x:category[@text='System']/x:dimension[@axis='column']/x:category[@text='Percent']/x:cell/@text)"
totmisspercent1_str = statsout1.xpath(path, namespaces=ns)
print(totmisspercent1_str)

# Second variable frequencies
statsout2 = etree.parse(BytesIO(outputs['Table_Frequencies_1.xml'].content))
print(etree.tostring(statsout2, pretty_print=True))

path = "//x:dimension[@varName='" + var2 + "']/x:group[@text='Valid']/x:group/x:group/x:category/@text"
vals2 = [str(x) for x in statsout2.xpath(path, namespaces=ns)]
print(vals2)

path = "//x:dimension[@varName='" + var2 + "']/x:group[@text='Valid']/x:group/x:group/x:category/x:dimension[@axis='column']/x:category[@text='Frequency']/x:cell/@number"
freqs2 = [int(x) for x in statsout2.xpath(path, namespaces=ns)]
print(freqs2)

path = "//x:dimension[@varName='" + var2 + "']/x:group[@text='Valid']/x:group/x:group/x:category/x:dimension[@axis='column']/x:category[@text='Percent']/x:cell/@number"
allcents2_num = [str2float(x) for x in statsout2.xpath(path, namespaces=ns)]
print(allcents2_num)

path = "//x:dimension[@varName='" + var2 + "']/x:group[@text='Valid']/x:group/x:group/x:category/x:dimension[@axis='column']/x:category[@text='Percent']/x:cell/@text"
allcents2_str = statsout2.xpath(path, namespaces=ns)
print(allcents2_str)

path = "//x:dimension[@varName='" + var2 + "']/x:group[@text='Valid']/x:group/x:group/x:category/x:dimension[@axis='column']/x:category[@text='Valid Percent']/x:cell/@number"
validcents2_num = [str2float(x) for x in statsout2.xpath(path, namespaces=ns)]
print(validcents2_num)

path = "//x:dimension[@varName='" + var2 + "']/x:group[@text='Valid']/x:group/x:group/x:category/x:dimension[@axis='column']/x:category[@text='Valid Percent']/x:cell/@text"
validcents2_str = statsout2.xpath(path, namespaces=ns)
print(validcents2_str)

path = "//x:dimension[@varName='" + var2 + "']/x:group[@text='Valid']/x:group/x:group/x:category/x:dimension[@axis='column']/x:category[@text='Cumulative Percent']/x:cell/@number"
cumcents2_num = [str2float(x) for x in statsout2.xpath(path, namespaces=ns)]
print(cumcents2_num)

path = "//x:dimension[@varName='" + var2 + "']/x:group[@text='Valid']/x:group/x:group/x:category/x:dimension[@axis='column']/x:category[@text='Cumulative Percent']/x:cell/@text"
cumcents2_str = statsout2.xpath(path, namespaces=ns)
print(cumcents2_str)

path = "string(//x:dimension[@varName='" + var2 + "']/x:group[@text='Valid']/x:category[@text='Total']/x:dimension[@axis='column']/x:category[@text='Frequency']/x:cell/@number)"
totfreq2 = int(statsout2.xpath(path, namespaces=ns))
print(totfreq2)

path = "string(//x:dimension[@varName='" + var2 + "']/x:group[@text='Valid']/x:category[@text='Total']/x:dimension[@axis='column']/x:category[@text='Percent']/x:cell/@number)"
totpercent2_num = str2float(statsout2.xpath(path, namespaces=ns))
print(totpercent2_num)

path = "string(//x:dimension[@varName='" + var2 + "']/x:group[@text='Valid']/x:category[@text='Total']/x:dimension[@axis='column']/x:category[@text='Percent']/x:cell/@text)"
totpercent2_str = statsout2.xpath(path, namespaces=ns)
print(totpercent2_str)

path = "string(//x:dimension[@varName='" + var2 + "']/x:group[@text='Valid']/x:category[@text='Total']/x:dimension[@axis='column']/x:category[@text='Valid Percent']/x:cell/@number)"
totvalidpercent2_num = str2float(statsout2.xpath(path, namespaces=ns))
print(totvalidpercent2_num)

path = "string(//x:dimension[@varName='" + var2 + "']/x:group[@text='Valid']/x:category[@text='Total']/x:dimension[@axis='column']/x:category[@text='Valid Percent']/x:cell/@text)"
totvalidpercent2_str = statsout2.xpath(path, namespaces=ns)
print(totvalidpercent2_str)

# FIXME: This only picks up missing values using system missing values
path = "string(//x:dimension[@varName='" + var2 + "']/x:group[@text='Missing']/x:category[@text='System']/x:dimension[@axis='column']/x:category[@text='Frequency']/x:cell/@number)"
totmissfreq2_num = str2float(statsout2.xpath(path, namespaces=ns))
print(totmissfreq2_num)

path = "string(//x:dimension[@varName='" + var2 + "']/x:group[@text='Missing']/x:category[@text='System']/x:dimension[@axis='column']/x:category[@text='Frequency']/x:cell/@text)"
totmissfreq2_str = statsout2.xpath(path, namespaces=ns)
print(totmissfreq2_str)

path = "string(//x:dimension[@varName='" + var2 + "']/x:group[@text='Missing']/x:category[@text='System']/x:dimension[@axis='column']/x:category[@text='Percent']/x:cell/@number)"
totmisspercent2_num = str2float(statsout2.xpath(path, namespaces=ns))
print(totmisspercent2_num)

path = "string(//x:dimension[@varName='" + var2 + "']/x:group[@text='Missing']/x:category[@text='System']/x:dimension[@axis='column']/x:category[@text='Percent']/x:cell/@text)"
totmisspercent2_str = statsout2.xpath(path, namespaces=ns)
print(totmisspercent2_str)

crosstabout = etree.parse(BytesIO(outputs['Table_Crosstabulation.xml'].content))
print(etree.tostring(crosstabout, pretty_print=True))

path = "string(//x:dimension[@axis='row']/@text)"
rowlabel = crosstabout.xpath(path, namespaces=ns)
print(rowlabel)

path = "string(//x:dimension[@axis='row']/x:category[@text='Total']/x:dimension[@text='Statistics']/x:category[@text='Count']/x:dimension/@text)"
collabel = crosstabout.xpath(path, namespaces=ns)
print(collabel)

path = "//x:dimension[@axis='row']/x:group/x:category/@label"
rowcats = [x for x in crosstabout.xpath(path, namespaces=ns)]
if rowcats == []:
     path = "//x:dimension[@axis='row']/x:group/x:category/@text"
     rowcats = [x for x in crosstabout.xpath(path, namespaces=ns)]
print(rowcats)

path = "//x:dimension[@axis='row']/x:category[@text='Total']/x:dimension[@text='Statistics']/x:category[@text='Count']/x:dimension/x:group/x:category/@label"
colcats = [x for x in crosstabout.xpath(path, namespaces=ns)]
if colcats == []:
     path = "//x:dimension[@axis='row']/x:category[@text='Total']/x:dimension[@text='Statistics']/x:category[@text='Count']/x:dimension/x:group/x:category/@text"
     colcats = [x for x in crosstabout.xpath(path, namespaces=ns)]
print(colcats)

path = "//x:dimension[@axis='row']/x:group/x:category/x:dimension[@axis='row']/x:category[@text='Count']/x:dimension[@axis='column']/x:group/x:category/x:cell/@number"
counts = numpy.array([int(x) for x in crosstabout.xpath(path, namespaces=ns)])
counts = counts.reshape([len(rowcats), len(colcats)])
print(counts)

path = "//x:dimension[@axis='row']/x:group/x:category/x:dimension[@axis='row']/x:category[@text='Count']/x:dimension[@axis='column']/x:category[@text='Total']/x:cell/@number"
countcoltots = [int(x) for x in crosstabout.xpath(path, namespaces=ns)]
print(countcoltots)

path = "//x:dimension[@axis='row']/x:category[@text='Total']/x:dimension[@axis='row']/x:category[@text='Count']/x:dimension[@axis='column']/x:group/x:category/x:cell/@number"
countrowtots = [int(x) for x in crosstabout.xpath(path, namespaces=ns)]
print(countrowtots)

path = "//x:dimension[@axis='row']/x:group/x:category/x:dimension[@axis='row']/x:category[starts-with(@text, '% within')]/x:dimension[@axis='column']/x:group/x:category/x:cell/@number"
percents_num = numpy.array([str2float(x) for x in crosstabout.xpath(path, namespaces=ns)])
percents_num = percents_num.reshape([len(rowcats), len(colcats)])
print(percents_num)

path = "//x:dimension[@axis='row']/x:group/x:category/x:dimension[@axis='row']/x:category[starts-with(@text, '% within')]/x:dimension[@axis='column']/x:group/x:category/x:cell/@text"
percents_str = numpy.array(crosstabout.xpath(path, namespaces=ns))
percents_str = percents_str.reshape([len(rowcats), len(colcats)])
print(percents_str)

path = "//x:dimension[@axis='row']/x:group/x:category/x:dimension[@axis='row']/x:category[starts-with(@text, '% within')]/x:dimension[@axis='column']/x:category[@text='Total']/x:cell/@number"
percentcoltots_num = [str2float(x) for x in crosstabout.xpath(path, namespaces=ns)]
print(percentcoltots_num)

path = "//x:dimension[@axis='row']/x:group/x:category/x:dimension[@axis='row']/x:category[starts-with(@text, '% within')]/x:dimension[@axis='column']/x:category[@text='Total']/x:cell/@text"
percentcoltots_str = crosstabout.xpath(path, namespaces=ns)
print(percentcoltots_str)

path = "//x:dimension[@axis='row']/x:category[@text='Total']/x:dimension[@axis='row']/x:category[starts-with(@text, '% within')]/x:dimension[@axis='column']/x:group/x:category/x:cell/@number"
percentrowtots_num = [str2float(x) for x in crosstabout.xpath(path, namespaces=ns)]
print(percentrowtots_num)

path = "//x:dimension[@axis='row']/x:category[@text='Total']/x:dimension[@axis='row']/x:category[starts-with(@text, '% within')]/x:dimension[@axis='column']/x:group/x:category/x:cell/@text"
percentrowtots_str = crosstabout.xpath(path, namespaces=ns)
print(percentrowtots_str)

# Outputs:
# Table_Statistics.htm : N valid/missing for first variable
# Table_Statistics_1.htm : N valid/missing for second variable
# Table_Frequencies.htm : Frequencies of first variable
# Table_Frequencies_1.htm : Frequencies of second variable
# Chart_000.png : Simple bar chart for var1 (counts)
# Chart_1_000.png : Simple pie chart for var1 (counts)
# Chart_2_000.png : Simple bar chart for var2 (counts)
# Chart_3_000.png : Grouped bar chart for var1 by var2 (counts)
# Chart_4_000.png : Grouped bar chart for var1 by var2 (percent)
# Table_Crosstabulation.htm : Percentages for var1 within var2
# Table_Case Processing Summary.htm : Case processing information for crosstabs

# Table_Notes.htm : notes regarding first frequencies command
# Table_Notes_1.htm : notes regarding second frequencies command
# Table_Notes_2.htm : notes regarding first simple bar chart (counts)
# Table_Notes_3.htm : notes regarding second simple bar chart (counts)
# Table_Notes_4.htm : notes group bar chart (counts)
# Table_Notes_5.htm : notes group bar chart (percent)
# Table_Notes_6.htm : notes crosstabs command

varname1short = varname1
varname1long = varname1
if varlab1 != '':
    varname1long = varlab1 + '[' + varname1long + ']'

varname1short = '<strong>' + varname1short + '</strong>'
varname1long = '<strong>' + varname1long + '</strong>'

varname2short = varname2
varname2long = varname2
if varlab2 != '':
    varname2long = varlab2 + '[' + varname2long + ']'

varname2short = '<strong>' + varname2short + '</strong>'
varname2long = '<strong>' + varname2long + '</strong>'

introhtml = '<h2>Descriptive statistics for categorical variables practical</h2>'
introhtml += '<p>'
introhtml += 'Welcome to the descriptive statistics practical in which we will look at how to investigate categorical variables in SPSS. '
introhtml += 'Categorical variables can take only predefined values (or categories) and can be of two types - nominal and ordinal. '
introhtml += 'For nominal variables each category has a name but there is no natural order to the categories. '
introhtml += 'For example, a nominal variable might measure &quot;fruit choice&quot;, in which it does not make sense to describe an orange as somehow coming between an apple and a banana in an ordering. '
introhtml += 'Ordinal categories by contrast do have a natural ordering. '
introhtml += 'For example, an ordinal variable might measure opinions on an issue with the categories Strongly disagree; Disagree; Agree; and Strongly Agree forming an ordering in terms of &quot;strength of ageement&quot; from low to high. '
introhtml += '(This variable is an example of a <em>Likert scale</em>.) '
introhtml += 'In this practical we will simply look at summaries and plots of categorical variables that apply equally to both nominal and ordinal variables. '
introhtml += '</p>'

outputs['introduction.html'] = HTMLOutput(introhtml, "Introduction text")

var1prachtml = '<p>'
var1prachtml += 'The first categorical variable we will look at is ' + varname1short 
if varlab1 != '':
    var1prachtml += ', labelled ' + varlab1
var1prachtml += ' which has ' + str(len(rowcats)) + ' categories: '
for loop in range(0, len(rowcats)):
    if loop == len(rowcats)-1:
        var1prachtml += ' and '
    elif loop != 0:
        var1prachtml += ', '
    var1prachtml += '<i>' + str(rowcats[loop]) + '</i>'    
var1prachtml += '. '
var1prachtml += 'The second variable is ' + varname2short 
if varlab2 != '':
    var1prachtml += ', labelled ' + varlab2
var1prachtml += ' which has ' + str(len(colcats)) + ' categories: '
for loop in range(0, len(colcats)):
    if loop == len(colcats)-1:
        var1prachtml += ' and '
    elif loop != 0:
        var1prachtml += ', '
    var1prachtml += '<i>' + str(colcats[loop]) + '</i>'
var1prachtml += '. '
var1prachtml += '</p>'
var1prachtml += '<p>'
var1prachtml += 'We will begin by looking at how frequent the various categories for ' + varname1short + ' are by choosing the following in SPSS:'
var1prachtml += '</p>'


var1instructhtml = '<ul class = "instructions">'
var1instructhtml += '<li>Select <strong>Frequencies</strong> from the <strong>Descriptive Statistics</strong> submenu available from the <strong>Analyze</strong> menu.</li>'
var1instructhtml += '<li>Copy the ' + varname1long + ' variable into the <strong>Variable(s)</strong> box.</li>'
var1instructhtml += '<li>Click on the <strong>OK</strong> button to produce the tables as shown below.</li>'
var1instructhtml += '</ul>'

var1prachtml += var1instructhtml

var1prachtml += '<p>'
var1prachtml += 'Two tables will now appear in the output window. '
var1prachtml += 'The first is really just a count of number of observations in the dataset and how many are missing for the variable, ' + varname1short + '. '
var1prachtml += '</p>'

var1prachtml += outputs['Table_Statistics.htm'].content

var1prachtml += '<p>'
if Nmiss1 == 0:
    var1prachtml += 'In this case we see there are no missing observations for the ' + varname1short + ' variable and all ' + str(Nvalid1) + ' values are valid. '
else :
    var1prachtml += 'In this case there are ' + str(Nmiss1) + ' missing observations for variable ' + varname1short + ' with ' + str(Nvalid1) + ' valid values. '
var1prachtml += 'In the second table we can see a list of the different categories in the data and their frequencies: '
var1prachtml += '</p>'

var1prachtml += outputs['Table_Frequencies.htm'].content

var1prachtml += '<p>'
var1prachtml += 'This second table has 5 columns which we will now describe. '
var1prachtml += 'The first column simply gives the categories for the variable, ' + varname1short + ' so that we can tell what each row refers to. '
var1prachtml += 'In the second column headed Frequency we get the actual numbers of occurrences of the variable and so we see that there are '
for loop in range(0, len(rowcats)):
    if loop == len(rowcats)-1:
        var1prachtml += ' and '
    elif loop != 0:
        var1prachtml += ', '
    var1prachtml += str(freqs1[loop]) + ' occurrences of <i>' + str(vals1[loop]) + '</i>'
var1prachtml += '. '
var1prachtml += 'This is a useful summary as we can compare the counts within the dataset. '
var1prachtml += 'We might however not simply be interested in this dataset in isolation and so it is often useful to convert these counts into percentages and this is done in column 3. '
var1prachtml += 'Here we see that '
for loop in range(0, len(rowcats)):
    if loop == len(rowcats)-1:
        var1prachtml += ' and '
    elif loop != 0:
        var1prachtml += ', '
    var1prachtml += allcents1_str[loop] + ' percent of observations are in category <i>' + str(vals1[loop]) + '</i>'
var1prachtml += '. '
if Nmiss1 == 0:
    var1prachtml += 'We have no missing data in this variable and so the fourth column simply replicates the third column as all observations are valid. '
else:
    var1prachtml += 'There are ' + totmisspercent1_str + ' percent of observations that are missing and not included in the percentages above and so we often want to look at percentages out of valid observations only. '
    var1prachtml += 'These are given in the fourth column and we see that '
    for loop in range(0, len(rowcats)):
        if loop == len(rowcats)-1:
            var1prachtml += ' and '
        elif loop != 0:
            var1prachtml += ', '
        var1prachtml += validcents1_str[loop] + ' percent of valid observations are in category <i>' + str(vals1[loop]) + '</i>'
    var1prachtml += '. '
var1prachtml += 'Finally in the fifth column we look at cumulative percentages so that we see that ' + cumcents1_str[0] + ' percent are in the first category'
for loop in range(1, len(rowcats)-1):
        var1prachtml += ', ' + cumcents1_str[loop] + ' percent of valid observations in the first ' + str(loop+1) + ' categories'
var1prachtml += ' and unsurprisingly 100 percent of valid observations are in one of the ' + str(len(rowcats)) + ' categories. '
var1prachtml += 'These cumulative percentages are of most use when the variable is ordinal because, by definition, the numbering and ordering of categories of a nominal variable is arbitrary. '
var1prachtml += '</p>'
outputs['variable1.html'] = HTMLOutput(var1prachtml, "Variable one text")

rowforquestion1 = random.randint(0, len(vals1) - 1)
rowforquestion2 = random.randint(0, len(vals1) - 1)
rowforquestion3 = random.randint(0, len(vals1) - 1)
rowforquestion4 = random.randint(0, len(vals1) - 2)

var1questhtml = '<p>'
var1questhtml += 'Use the Frequencies window in SPSS to answer the following questions about the distribution of the variable ' + varname1short
if varlab1 != '':
    var1questhtml += ', ' + varlab1
var1questhtml += ':'
var1questhtml += '</p>'
var1questhtml += '<ul>'
var1questhtml += '<li>Question: How many occurrences are there of ' + str(vals1[rowforquestion1]) + '?</li>'
var1questhtml += '<li>Question: What percentage of the dataset is in the category ' + str(vals1[rowforquestion2]) + '?</li>'
var1questhtml += '<li>Question: What percentage of valid data (i.e. without missing values) is in category ' + str(vals1[rowforquestion3]) + '?</li>'
var1questhtml += '<li>Question: What percentage of values are in the first ' + str(rowforquestion4+1) + ' categories combined?</li>'
var1questhtml += '</ul>'

outputs['variable1questions.html'] = HTMLOutput(var1questhtml, "Questions for variable one text")

var1solhtml = '<p>'
var1solhtml += 'The SPSS instructions for the first question on frequencies are as follows: '
var1solhtml += '</p>'
var1solhtml += '<ul class = "instructions">'
var1solhtml += '<li>Select <strong>Frequencies</strong> from the <strong>Descriptive Statistics</strong> submenu available from the <strong>Analyze</strong> menu.</li>'
var1solhtml += '<li>Copy the ' + varname1long + ' variable into the <strong>Variable(s)</strong> box.</li>'
var1solhtml += '<li>Click on the <strong>OK</strong> button to produce the tables.</li>'
var1solhtml += '</ul>'
var1solhtml += '<ul>'
var1solhtml += '<li>Question: How many occurrences are there of ' + str(vals1[rowforquestion1]) + '?</li>'
var1solhtml += '<li>Question: What percentage of the dataset is in the category ' + str(vals1[rowforquestion2]) + '?</li>'
var1solhtml += '<li>Question: What percentage of valid data (i.e. without missing values) is in category ' + str(vals1[rowforquestion3]) + '?</li>'
var1solhtml += '<li>Question: What percentage of values are in the first ' + str(rowforquestion4+1) + ' categories combined?</li>'
var1solhtml += '</ul>'

var1solhtml += '<p>'
var1solhtml += 'Solutions: The output from SPSS is as follows:'
var1solhtml += '</p>'
var1solhtml += outputs['Table_Statistics.htm'].content
var1solhtml += outputs['Table_Frequencies.htm'].content
var1solhtml += '<p>'
var1solhtml += 'The answers are as follows:'
var1solhtml += '</p>'
var1solhtml += '<ul>'
var1solhtml += '<li>There are ' + str(freqs1[rowforquestion1]) + ' occurrences of ' + str(vals1[rowforquestion1]) + '.</li>'
var1solhtml += '<li>' + str(allcents1_str[rowforquestion2]) + ' percent of the dataset is in category ' + str(vals1[rowforquestion2]) + '.</li>'
var1solhtml += '<li>' + str(validcents1_str[rowforquestion3]) + ' percent of valid data is in category ' + str(vals1[rowforquestion3]) + '.</li>'
var1solhtml += '<li>' + str(cumcents1_str[rowforquestion4]) + ' percent of values are in the first ' + str(rowforquestion4+1) + ' categories combined.</li>'
var1solhtml += '</ul>'

outputs['variable1answers.html'] = HTMLOutput(var1solhtml, "Answers for variable one text")

var2prachtml = '<p>'
var2prachtml += 'We can now repeat this for our second variable, ' + varname2short + ' and to do this in SPSS we do the following:'
var2prachtml += '</p>'

var2instructhtml = '<ul class = "instructions">'
var2instructhtml += '<li>Select <strong>Frequencies</strong> from the <strong>Descriptive Statistics</strong> submenu available from the <strong>Analyze</strong> menu.</li>'
var2instructhtml += '<li>Remove the ' + varname1long + ' variable into the <strong>Variable(s)</strong> box.</li>'
var2instructhtml += '<li>Copy the ' + varname2long + ' variable into the <strong>Variable(s)</strong> box.</li>'
var2instructhtml += '<li>Click on the <strong>OK</strong> button to produce the table as shown below.</li>'
var2instructhtml += '</ul>'

var2prachtml += var2instructhtml

var2prachtml += '<p>'
var2prachtml += 'Two tables will again appear in the output window. '
var2prachtml += 'The first is again a count of number of observations in the dataset and how many are missing for the variable, <strong>' + varname2short + '</strong>.'
var2prachtml += '</p>'

var2prachtml += outputs['Table_Statistics_1.htm'].content

var2prachtml += '<p>'
if Nmiss2 == 0:
    var2prachtml += 'In this case we see there are no missing observations for the ' + varname2short + ' variable and all ' + str(Nvalid2) + ' values are valid. '
else :
    var2prachtml += 'In this case there are ' + str(Nmiss2) + ' missing observations for variable ' + varname2short + ' with ' +str(Nvalid2) + ' valid values. '
var2prachtml += 'In the second table we can see a list of the different categories in the data for ' + varname2short + ' and their frequencies:'
var2prachtml += '</p>'

var2prachtml += outputs['Table_Frequencies_1.htm'].content

var2prachtml += '<p>'
var2prachtml += 'Again, the first column simply gives the categories for the variable, ' + varname2short + ' so that we can tell what each row refers to, and the second column headed Frequency gives the actual numbers of occurrences of the variable. '
var2prachtml += 'Here there are '
for loop in range(0, len(colcats)):
    if loop == len(colcats)-1:
        var2prachtml += ' and '
    elif loop != 0:
        var2prachtml += ', '
    var2prachtml += str(freqs2[loop]) + ' occurrences of <i>' + str(vals2[loop]) + '</i>'
var2prachtml += '. '
var2prachtml += 'Converting these counts into percentages in column 3, we see that '
for loop in range(0, len(colcats)):
    if loop == len(colcats)-1:
        var2prachtml += ' and '
    elif loop != 0:
        var2prachtml += ', '
    var2prachtml += allcents2_str[loop] + ' percent of observations are in category <i>' + str(vals2[loop]) + '</i>'
var2prachtml += '. '
if Nmiss2 == 0:
    var2prachtml += 'We have no missing data in this variable and so the fourth column simply replicates the third column as all observations are valid. '
else:
    var2prachtml += 'There are ' + totmisspercent2_str + ' percent of observations that are missing and not included in the percentages above. '
    var2prachtml += 'To see the percentages out of valid observations only we look to the fourth column, which shows that '
    for loop in range(0, len(colcats)):
        if loop == len(colcats)-1:
            var2prachtml += ' and '
        elif loop != 0:
            var2prachtml += ', '
        var2prachtml += validcents2_str[loop] + ' percent of valid observations are in category <i>' + str(vals2[loop]) + '</i>'
    var2prachtml += '. '
var2prachtml += 'Finally, turning to the cumulative percentages in the fifth column we see that ' + cumcents2_str[0] + ' percent are in the first category'
for loop in range(1, len(colcats)-1):
        var2prachtml += ', ' + cumcents2_str[loop] + ' percent of valid observations are in the first ' + str(loop+1) + ' categories'
var2prachtml += ' and 100 percent of valid observations are in one of the ' + str(len(colcats)) + ' categories. '
var2prachtml += '</p>'
outputs['variable2.html'] = HTMLOutput(var2prachtml, "Variable two text")

rowforquestion1 = random.randint(0, len(vals2) - 1)
rowforquestion2 = random.randint(0, len(vals2) - 1)
rowforquestion3 = random.randint(0, len(vals2) - 1)
rowforquestion4 = random.randint(0, len(vals2) - 2)

var2questhtml = '<p>'
var2questhtml += 'Use the Frequencies window in SPSS to answer the following questions about the distribution of the variable ' + varname2short
if varlab2 != '':
    var2questhtml += ', ' + varlab2
var2questhtml += ':'
var2questhtml += '</p>'
var2questhtml += '<ul>'
var2questhtml += '<li>Question: How many occurrences are there of ' + str(vals2[rowforquestion1]) + '?</li>'
var2questhtml += '<li>Question: What percentage of the dataset is in the category ' + str(vals2[rowforquestion2]) + '?</li>'
var2questhtml += '<li>Question: What percentage of valid data (i.e. without missing values) is in category ' + str(vals2[rowforquestion3]) + '?</li>'
var2questhtml += '<li>Question: What percentage of values are in the first ' + str(rowforquestion4+1) + ' categories combined?</li>'
var2questhtml += '</ul>'

outputs['variable2questions.html'] = HTMLOutput(var2questhtml, "Questions for variable one text")

var2solhtml = '<p>'
var2solhtml += 'The SPSS instructions for the second question on frequencies are as follows: '
var2solhtml += '</p>'
var2solhtml += '<ul class = "instructions">'
var2solhtml += '<li>Select <strong>Frequencies</strong> from the <strong>Descriptive Statistics</strong> submenu available from the <strong>Analyze</strong> menu.</li>'
var2solhtml += '<li>Remove the ' + varname1long + ' variable into the <strong>Variable(s)</strong> box.</li>'
var2solhtml += '<li>Copy the ' + varname2long + ' variable into the <strong>Variable(s)</strong> box.</li>'
var2solhtml += '<li>Click on the <strong>OK</strong> button to produce the tables.</li>'
var2solhtml += '</ul>'
var2solhtml += '<ul>'
var2solhtml += '<li>Question: How many occurrences are there of ' + str(vals2[rowforquestion1]) + '?</li>'
var2solhtml += '<li>Question: What percentage of the dataset is in the category ' + str(vals2[rowforquestion2]) + '?</li>'
var2solhtml += '<li>Question: What percentage of valid data (i.e. without missing values) is in category ' + str(vals2[rowforquestion3]) + '?</li>'
var2solhtml += '<li>Question: What percentage of values are in the first ' + str(rowforquestion4+1) + ' categories combined?</li>'
var2solhtml += '</ul>'

var2solhtml += '<p>'
var2solhtml += 'Solutions: The output from SPSS is as follows:'
var2solhtml += '</p>'
var2solhtml += outputs['Table_Statistics_1.htm'].content
var2solhtml += outputs['Table_Frequencies_1.htm'].content
var2solhtml += '<p>'
var2solhtml += 'The answers are as follows:'
var2solhtml += '</p>'
var2solhtml += '<ul>'
var2solhtml += '<li>There are ' + str(freqs2[rowforquestion1]) + ' occurrences of ' + str(vals2[rowforquestion1]) + '.</li>'
var2solhtml += '<li>' + str(allcents2_str[rowforquestion2]) + ' percent of the dataset is in category ' + str(vals2[rowforquestion2]) + '.</li>'
var2solhtml += '<li>' + str(validcents2_str[rowforquestion3]) + ' percent of valid data is in category ' + str(vals2[rowforquestion3]) + '.</li>'
var2solhtml += '<li>' + str(cumcents2_str[rowforquestion4]) + ' percent of values are in the first ' + str(rowforquestion4+1) + ' categories combined.</li>'
var2solhtml += '</ul>'

outputs['variable2answers.html'] = HTMLOutput(var2solhtml, "Answers for variable two text")

bar1prachtml = '<p>'
bar1prachtml += 'We will next look at how we might illustrate the distribution of the variables graphically. '
bar1prachtml += 'When we have categorical data we can plot the data as a bar graph and to do this for our first variable, ' + varname1short + ' in SPSS we need to do the following:'
bar1prachtml += '</p>'

bar1instrhtml = '<ul class = "instructions">'
bar1instrhtml += '<li>Select <strong>Bar...</strong> from the <strong>Legacy Dialogs</strong> submenu available from the <strong>Graphs</strong> menu.</li>'
bar1instrhtml += '<li>We will use the defaults which are <strong>Simple</strong> and <strong>Summaries for groups of cases</strong> and then click on <strong>Define </strong>.</li>'
bar1instrhtml += '<li>Copy the ' + varname1long + ' variable into the <strong>Category Axis</strong> box.</li>'
bar1instrhtml += '<li>Click on the <strong>OK</strong> button to produce the graph as shown below.</li>'
bar1instrhtml += '</ul>'

bar1prachtml += bar1instrhtml

bar1prachtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_000.png'].content)  + '">'

bar1prachtml += '<p>'
bar1prachtml += 'You will see that the graph contains one bar for each category and each bar is labelled with the category name. '
bar1prachtml += 'The graph plots the actual counts in each category so we can see there are for example ' + str(freqs1[0]) + ' observations in category <i>' + str(vals1[0]) + '</i>. '
bar1prachtml += '</p>'
outputs['barchart1.html'] = HTMLOutput(bar1prachtml, "Bar Chart one explanation")

catforquestion1 = random.randint(0, len(vals1) - 1)

bar1questhtml =  '<p>'
bar1questhtml += 'Use SPSS to draw a bar chart and use it to answer the following questions about the variable ' + varname1short + ':'
bar1questhtml += '</p>'
bar1questhtml += '<ul>'
bar1questhtml += '<li>Question: You will see ' + str(len(vals1)) + ' categories. Approximately how many observations are in category ' + str(vals1[catforquestion1]) + '?</li>'
bar1questhtml += '</ul>'

outputs['barchart1questions.html'] = HTMLOutput(bar1questhtml, "Bar Chart one question text")

bar1solhtml = '<p>'
bar1solhtml += 'The SPSS instructions for the first bar chart are as follows: '
bar1solhtml += '</p>'
bar1solhtml += '<ul class = "instructions">'
bar1solhtml += '<li>Select <strong>Bar...</strong> from the <strong>Legacy Dialogs</strong> submenu available from the <strong>Graphs</strong> menu.</li>'
bar1solhtml += '<li>We will use the defaults which are <strong>Simple</strong> and <strong>Summaries for groups of cases</strong> and then click on <strong>Define </strong>.</li>'
bar1solhtml += '<li>Copy the ' + varname1long + ' variable into the <strong>Category Axis</strong> box.</li>'
bar1solhtml += '<li>Click on the <strong>OK</strong> button to produce the graph.</li>'
bar1solhtml += '</ul>'
bar1solhtml += '<ul>'
bar1solhtml += '<li>Question: You will see ' + str(len(vals1)) + ' categories. Approximately how many observations are in category ' + str(vals1[catforquestion1]) + '?</li>'
bar1solhtml += '</ul>'
bar1solhtml += '<p>'
bar1solhtml += 'Solution: The output from SPSS is as follows:'
bar1solhtml += '</p>'
bar1solhtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_000.png'].content)  + '">'
bar1solhtml += '<ul>'
bar1solhtml += '<li> There are ' + str(freqs1[catforquestion1]) + ' observations in category ' + str(vals1[catforquestion1]) + '.</li>'
bar1solhtml += '</ul>'

outputs['barchart1answers.html'] = HTMLOutput(bar1solhtml, "Bar Chart one answers text")

pie1prachtml = '<p>'
pie1prachtml += 'We can also look at the data in the form of a pie chart which is an alternative way of displaying the data (although pie charts are often criticised as providing less visual clarity than a bar chart). This can be done via the following:'
pie1prachtml += '</p>'

pie1instrhtml = '<ul class = "instructions">'
pie1instrhtml += '<li>Select <strong>Pie...</strong> from the <strong>Legacy Dialogs</strong> submenu available from the <strong>Graphs</strong> menu.</li>'
pie1instrhtml += '<li>Keep the choices as <strong>Summaries for groups of cases</strong> before clicking on <strong>Define</strong>.</li>'
pie1instrhtml += '<li>Add the ' + varname1long + ' variable in the <strong>Define Slices by:</strong> box.</li>'
pie1instrhtml += '<li>Click on the <strong>OK</strong> button to produce the graph as shown below.</li>'
pie1instrhtml += '</ul>'

pie1prachtml += pie1instrhtml

pie1prachtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_1_000.png'].content)  + '">'

pie1prachtml += '<p>'
pie1prachtml += 'Here the pie chart represents each category in a different colour with the proportion of the total that are in each category being proportional to the size of the slices of pie. '
pie1prachtml += 'The circle consists of 360 degrees in total, so the '
for loop in range(0, len(rowcats)):
    if loop == len(rowcats)-1:
        pie1prachtml += ' and the '
    elif loop != 0:
        pie1prachtml += ', the '
    angle = int(round((validcents1_num[loop])*3.6, 0))
    pie1prachtml += validcents1_str[loop] + ' percent of (valid) observations that are in category <i>' + str(vals1[loop]) + '</i> are represented by a slice of angle ' + str(angle) + ' degrees' 
pie1prachtml += '. '
pie1prachtml += '</p>'
outputs['piecharts.html'] = HTMLOutput(pie1prachtml, "Pie Chart explanation")

pie1questhtml =  '<p>'
pie1questhtml += 'Use SPSS to draw a pie chart and use it to answer the following question about the variable ' + varname1short + ':'
pie1questhtml += '</p>'
pie1questhtml += '<ul>'
pie1questhtml += '<li>Question: Which is the most common category?</li>'
pie1questhtml += '</ul>'

outputs['piechart1questions.html'] = HTMLOutput(pie1questhtml, "Piechart one question text")

pie1solhtml = '<p>'
pie1solhtml += 'The SPSS instructions for the pie chart are as follows: '
pie1solhtml += '</p>'
pie1solhtml += '<ul class = "instructions">'
pie1solhtml += '<li>Select <strong>Pie...</strong> from the <strong>Legacy Dialogs</strong> submenu available from the <strong>Graphs</strong> menu.</li>'
pie1solhtml += '<li>Keep the choices as <strong>Summaries for groups of cases</strong> before clicking on <strong>Define</strong>.</li>'
pie1solhtml += '<li>Add the ' + varname1long + ' variable in the <strong>Define Slices by:</strong> box.</li>'
pie1solhtml += '<li>Click on the <strong>OK</strong> button to produce the graph.</li>'
pie1solhtml += '</ul>'
pie1solhtml += '<ul>'
pie1solhtml += '<li>Question: Which is the most common category?</li>'
pie1solhtml += '</ul>'
pie1solhtml += '<p>'
pie1solhtml += 'Solution: The output from SPSS is as follows:'
pie1solhtml += '</p>'
pie1solhtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_1_000.png'].content)  + '">'

pie1solhtml += '<ul>'
pie1solhtml += '<li>' + str(vals1[freqs1.index(max(freqs1))]) + ' is the most common category. </li>'
pie1solhtml += '</ul>'

outputs['piechart1answers.html'] = HTMLOutput(pie1solhtml, "Pie one answers text")

bar2prachtml = '<p>'
bar2prachtml += 'We can next plot a bar graph for the second variable, ' + varname2short + ' by doing the following in SPSS:'
bar2prachtml += '</p>'

bar2instrhtml = '<ul class = "instructions">'
bar2instrhtml += '<li>Select again <strong>Bar...</strong> from the <strong>Legacy Dialogs</strong> submenu available from the <strong>Graphs</strong> menu.</li>'
bar2instrhtml += '<li>Use the defaults which are <strong>Simple</strong> and <strong>Summaries for groups of cases</strong> and then click on <strong>Define</strong>.</li>'
bar2instrhtml += '<li>Remove the ' + varname1long + ' variable into the <strong>Category Axis</strong> box.</li>'
bar2instrhtml += '<li>Copy the ' + varname2long + ' variable into the <strong>Category Axis</strong> box.</li>'
bar2instrhtml += '<li>Click on the <strong>OK</strong> button to produce the graph as shown below.</li>'
bar2instrhtml += '</ul>'

bar2prachtml += bar2instrhtml

bar2prachtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_2_000.png'].content)  + '">'

bar2prachtml += '<p>'
bar2prachtml += 'This graph should have a similar presentation to the first graph with this time ' + str(len(colcats)) + ' bars and with for example ' + str(freqs2[0]) + ' observations in category <i>' + str(vals2[0]) + '</i>' + '. '
bar2prachtml += 'Note that again each bar is labelled, there are labels for each axis of the graph and there are gaps between each bar. '
bar2prachtml += '</p>'
outputs['barchart2.html'] = HTMLOutput(bar2prachtml, "Bar Chart two explanation")

catforquestion1 = random.randint(0, len(vals2) - 1)

bar2questhtml = '<p>'
bar2questhtml += 'Use SPSS to draw a bar chart and use it to answer the following questions about the variable ' + varname2short + ':'
bar2questhtml += '</p>'
bar2questhtml += '<ul>'
bar2questhtml += '<li>Question: You will see ' + str(len(vals2)) + ' categories. Approximately how many observations are in category ' + str(vals2[catforquestion1]) + '?</li>'
bar2questhtml += '</ul>'

outputs['barchart2questions.html'] = HTMLOutput(bar2questhtml, "Bar Chart two question text")

bar2solhtml = '<p>'
bar2solhtml += 'The SPSS instructions for the second bar chart are as follows: '
bar2solhtml += '</p>'
bar2solhtml += '<ul class = "instructions">'
bar2solhtml += '<li>Select again <strong>Bar...</strong> from the <strong>Legacy Dialogs</strong> submenu available from the <strong>Graphs</strong> menu.</li>'
bar2solhtml += '<li>Use the defaults which are <strong>Simple</strong> and <strong>Summaries for groups of cases</strong> and then click on <strong>Define</strong>.</li>'
bar2solhtml += '<li>Remove the ' + varname1long + ' variable into the <strong>Category Axis</strong> box.</li>'
bar2solhtml += '<li>Copy the ' + varname2long + ' variable into the <strong>Category Axis</strong> box.</li>'
bar2solhtml += '<li>Click on the <strong>OK</strong> button to produce the graph as shown below.</li>'
bar2solhtml += '</ul>'
bar2solhtml += '<ul>'
bar2solhtml += '<li>Question: You will see ' + str(len(vals2)) + ' categories. Approximately how many observations are in category ' + str(vals2[catforquestion1]) + '?</li>'
bar2solhtml += '</ul>'
bar2solhtml += '<p>'
bar2solhtml += 'Solution: The output from SPSS is as follows:'
bar2solhtml += '</p>'
bar2solhtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_2_000.png'].content)  + '">'
bar2solhtml += '<ul>'
bar2solhtml += '<li> There are ' + str(freqs2[catforquestion1]) + ' observations in category ' + str(vals2[catforquestion1]) + '.</li>'
bar2solhtml += '</ul>'

outputs['barchart2answers.html'] = HTMLOutput(bar2solhtml, "Bar Chart two answers text")

compprachtml = '<p>'
compprachtml += 'We can also look at the two variables in combination. '
compprachtml += 'This is useful for looking at the distribution on one variable relative to the other and can be achieved by doing the following commands in SPSS:'
compprachtml += '</p>'

compinstrhtml = '<ul class = "instructions">'
compinstrhtml += '<li>Select <strong>Bar...</strong> from the <strong>Legacy Dialogs</strong> submenu available from the <strong>Graphs</strong> menu.</li>'
compinstrhtml += '<li>This time we will change from the defaults to <strong>Clustered</strong> and <strong>Summaries for groups of cases</strong> before clicking on <strong>Define </strong>.</li>'
compinstrhtml += '<li>Copy the ' + varname1long + ' variable into the <strong>Category Axis</strong> box.</li>'
compinstrhtml += '<li>Copy the ' + varname2long + ' variable into the <strong>Define Clusters by</strong> box.</li>'
compinstrhtml += '<li>Click on the <strong>OK</strong> button to produce the graph as shown below.</li>'
compinstrhtml += '</ul>'

compprachtml += compinstrhtml

compprachtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_3_000.png'].content)  + '">'

compprachtml += '<p>'
compprachtml += 'Here we see that each category of ' + varname2short + ' is given a bar of a different colour so that one can, by eye, compare each category of ' + varname2short + ' for different categories of ' + varname1short + '. '
compprachtml += 'For example we see when ' + str(rowlabel) + ' takes value <i>' + str(rowcats[0]) + '</i>, then there are '
for loop in range(0, len(colcats)):
    if loop == len(colcats)-1:
        compprachtml += ' and '
    elif loop != 0:
        compprachtml += ', '
    compprachtml += str(counts[0, loop]) + ' observations where ' + str(collabel) + ' takes value <i>' +  str(colcats[loop]) + '</i>'
compprachtml += '. '
compprachtml += '</p>'

outputs['comparison.html'] = HTMLOutput(compprachtml, "Comparison text")

cat1forquestion1 = random.randint(0, len(vals1) - 1)
cat2forquestion1 = random.randint(0, len(vals2) - 1)
cat1forquestion2 = cat1forquestion1
cat2forquestion2 = cat2forquestion1
while cat1forquestion2 == cat1forquestion1 and cat2forquestion2 == cat2forquestion1:
    cat1forquestion2 = random.randint(0, len(vals1) - 1)
    cat2forquestion2 = random.randint(0, len(vals2) - 1)

cat2forquestion3 = cat2forquestion2
cat1forquestion3 = cat1forquestion2
while (cat2forquestion3 == cat2forquestion2 and cat1forquestion3 == cat1forquestion2) or (cat2forquestion3 == cat2forquestion1 and cat1forquestion3 == cat1forquestion1):
    cat1forquestion3 = random.randint(0, len(vals1) - 1)
    cat2forquestion3 = random.randint(0, len(vals2) - 1)

compquesthtml =  '<p>'
compquesthtml += 'Use SPSS to draw a clustered bar chart and use it to answer the following questions: '
compquesthtml += '</p>'
compquesthtml += '<ul>'
compquesthtml += '<li>Question: Approximately how many observations take values ' + vals1[cat1forquestion1] + ' for ' + varname1short + ' and ' + vals2[cat2forquestion1] + ' for ' + varname2short + '?</li>'
compquesthtml += '<li>Question: Approximately how many observations take values ' + vals1[cat1forquestion2] + ' for ' + varname1short + ' and ' + vals2[cat2forquestion2] + ' for ' + varname2short + '?</li>'
compquesthtml += '<li>Question: Approximately how many observations take values ' + vals1[cat1forquestion3] + ' for ' + varname1short + ' and ' + vals2[cat2forquestion3] + ' for ' + varname2short + '?</li>'
compquesthtml += '</ul>'

outputs['comparisonquestions.html'] = HTMLOutput(compquesthtml, "Comparison question text")

compsolhtml = '<p>'
compsolhtml += 'The SPSS instructions for the clustered bar chart of counts are as follows: '
compsolhtml += '</p>'
compsolhtml += '<ul class = "instructions">'
compsolhtml += '<li>Select <strong>Bar...</strong> from the <strong>Legacy Dialogs</strong> submenu available from the <strong>Graphs</strong> menu.</li>'
compsolhtml += '<li>This time we will change from the defaults to <strong>Clustered</strong> and <strong>Summaries for groups of cases</strong> before clicking on <strong>Define </strong>.</li>'
compsolhtml += '<li>Copy the ' + varname1long + ' variable into the <strong>Category Axis</strong> box.</li>'
compsolhtml += '<li>Copy the ' + varname2long + ' variable into the <strong>Define Clusters by</strong> box.</li>'
compsolhtml += '<li>Click on the <strong>OK</strong> button to produce the graph.</li>'
compsolhtml += '</ul>'
compsolhtml += '<ul>'
compsolhtml += '<li>Question: Approximately how many observations take values ' + vals1[cat1forquestion1] + ' for ' + varname1short + ' and ' + vals2[cat2forquestion1] + ' for ' + varname2short + '?</li>'
compsolhtml += '<li>Question: Approximately how many observations take values ' + vals1[cat1forquestion2] + ' for ' + varname1short + ' and ' + vals2[cat2forquestion2] + ' for ' + varname2short + '?</li>'
compsolhtml += '<li>Question: Approximately how many observations take values ' + vals1[cat1forquestion3] + ' for ' + varname1short + ' and ' + vals2[cat2forquestion3] + ' for ' + varname2short + '?</li>'
compsolhtml += '</ul>'

compsolhtml += '<p>'
compsolhtml += 'Solutions: The output from SPSS is as follows:'
compsolhtml += '</p>'
compsolhtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_3_000.png'].content)  + '">'
compsolhtml += '<p>'
compsolhtml += 'The answers are as follows:'
compsolhtml += '</p>'
compsolhtml += '<ul>'
compsolhtml += '<li>There are ' + str(counts[cat1forquestion1, cat2forquestion1]) + ' observations taking values ' + vals1[cat1forquestion1] + ' for ' + varname1short + ' and ' + vals2[cat2forquestion1] + ' for ' + varname2short + '.</li>'
compsolhtml += '<li>There are ' + str(counts[cat1forquestion2, cat2forquestion2]) + ' observations taking values ' + vals1[cat1forquestion2] + ' for ' + varname1short + ' and ' + vals2[cat2forquestion2] + ' for ' + varname2short + '.</li>'
compsolhtml += '<li>There are ' + str(counts[cat1forquestion3, cat2forquestion3]) + ' observations taking values ' + vals1[cat1forquestion3] + ' for ' + varname1short + ' and ' + vals2[cat2forquestion3] + ' for ' + varname2short + '.</li>'
compsolhtml += '</ul>'
outputs['comparisonanswers.html'] = HTMLOutput(compsolhtml, "Comparison answers text")

compprachtml = '<p>'
compprachtml += 'When the number of observations in the clustering variable categories are unequal, the plotting of counts makes it difficult to compare the relative composition of the categories plotted on the horizontal axis. '
compprachtml += 'As with the frequency tables we saw earlier, conversion to percentages can aid comparisons. '
compprachtml += 'To illustrate, to explore whether the distribution of responses to ' + varname1short + ' is the same for different categories of the variable ' + varname2short + ' we can do the following in SPSS:'
compprachtml += '</p>'

compcentinstrhtml = '<ul class = "instructions">'
compcentinstrhtml += '<li>Once again select <strong>Bar...</strong> from the <strong>Legacy Dialogs</strong> submenu available from the <strong>Graphs</strong> menu.</li>'
compcentinstrhtml += '<li>Keep the choices as <strong>Clustered</strong> and <strong>Summaries for groups of cases</strong> before clicking on <strong>Define</strong>.</li>'
compcentinstrhtml += '<li>Keep the ' + varname1long + ' variable in the <strong>Category Axis</strong> box.</li>'
compcentinstrhtml += '<li>Keep the ' + varname2long + ' variable in the <strong>Define Clusters by</strong> box.</li>'
compcentinstrhtml += '<li>Select <strong>% of cases</strong> in the <strong>Bars Represent</strong> choices.</li>'
compcentinstrhtml += '<li>Click on the <strong>OK</strong> button to produce the graph as shown below.</li>'
compcentinstrhtml += '</ul>'

compprachtml += compcentinstrhtml
compprachtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_4_000.png'].content)  + '">'

compprachtml += '<p>'
compprachtml += 'Here we see that now the bars represent the percentage of each category of ' + varname1short + ' that are found in each category of ' + varname2short + '. '
compprachtml += 'So for example if we look again at the cases where ' + str(collabel) + ' takes value ' + str(colcats[0]) + ', (the blue bars) then  '
for loop in range(0, len(rowcats)):
    if loop == len(rowcats)-1:
        compprachtml += ' and '
    elif loop != 0:
        compprachtml += ', '
    compprachtml += percents_str[loop, 0] + ' percent of observations have ' + varname1short + ' taking value ' +  str(rowcats[loop])
compprachtml += '. '
compprachtml += 'The chart shows that, compared with this last number, ' + percents_str[1, -1] + ' of the group for whom ' + varname1short + ' = ' + vals1[1] + ' have ' + varname2short + ' = ' + vals2[-1] + '. '
compprachtml += '</p>'
outputs['summary.html'] = HTMLOutput(compprachtml, "Summary text")

cat1forquestion1 = random.randint(0, len(vals1) - 1)
cat2forquestion1 = random.randint(0, len(vals2) - 1)
cat1forquestion2 = cat1forquestion1
cat2forquestion2 = cat2forquestion1
while cat1forquestion2 == cat1forquestion1 and cat2forquestion2 == cat2forquestion1:
    cat1forquestion2 = random.randint(0, len(vals1) - 1)
    cat2forquestion2 = random.randint(0, len(vals2) - 1)

cat2forquestion3 = cat2forquestion2
cat1forquestion3 = cat1forquestion2
while (cat2forquestion3 == cat2forquestion2 and cat1forquestion3 == cat1forquestion2) or (cat2forquestion3 == cat2forquestion1 and cat1forquestion3 == cat1forquestion1):
    cat1forquestion3 = random.randint(0, len(vals1) - 1)
    cat2forquestion3 = random.randint(0, len(vals2) - 1)

compcentquesthtml =  '<p>'
compcentquesthtml += 'Use SPSS to draw a clustered bar chart of percentages of cases and use it to answer the following questions: '
compcentquesthtml += '</p>'
compcentquesthtml += '<ul>'
compcentquesthtml += '<li>Question: What percentage of observations in the sub-group for which ' + varname2short + ' takes value ' + vals2[cat2forquestion1] + ' take value ' + vals1[cat1forquestion1] + ' for ' + varname1short + '?</li>'
compcentquesthtml += '<li>Question: What percentage of observations in the sub-group for which ' + varname2short + ' takes value ' + vals2[cat2forquestion2] + ' take value ' + vals1[cat1forquestion2] + ' for ' + varname1short + '?</li>'
compcentquesthtml += '<li>Question: What percentage of observations in the sub-group for which ' + varname2short + ' takes value ' + vals2[cat2forquestion3] + ' take value ' + vals1[cat1forquestion3] + ' for ' + varname1short + '?</li>'
compcentquesthtml += '</ul>'

outputs['summaryquestions.html'] = HTMLOutput(compcentquesthtml, "Comparison question text")

compcentsolhtml = '<p>'
compcentsolhtml += 'The SPSS instructions for the clustered bar chart of percentages are as follows: '
compcentsolhtml += '</p>'
compcentsolhtml += '<ul class = "instructions">'
compcentsolhtml += '<li>Once again select <strong>Bar...</strong> from the <strong>Legacy Dialogs</strong> submenu available from the <strong>Graphs</strong> menu.</li>'
compcentsolhtml += '<li>Keep the choices as <strong>Clustered</strong> and <strong>Summaries for groups of cases</strong> before clicking on <strong>Define</strong>.</li>'
compcentsolhtml += '<li>Keep the ' + varname1long + ' variable in the <strong>Category Axis</strong> box.</li>'
compcentsolhtml += '<li>Keep the ' + varname2long + ' variable in the <strong>Define Clusters by</strong> box.</li>'
compcentsolhtml += '<li>Select <strong>% of cases</strong> in the <strong>Bars Represent</strong> choices.</li>'
compcentsolhtml += '<li>Click on the <strong>OK</strong> button to produce the graph.</li>'
compcentsolhtml += '</ul>'
compcentsolhtml += '<ul>'
compcentsolhtml += '<li>Question: What percentage of observations in the sub-group for which ' + varname2short + ' takes value ' + vals2[cat2forquestion1] + ' take value ' + vals1[cat1forquestion1] + ' for ' + varname1short + '?</li>'
compcentsolhtml += '<li>Question: What percentage of observations in the sub-group for which ' + varname2short + ' takes value ' + vals2[cat2forquestion2] + ' take value ' + vals1[cat1forquestion2] + ' for ' + varname1short + '?</li>'
compcentsolhtml += '<li>Question: What percentage of observations in the sub-group for which ' + varname2short + ' takes value ' + vals2[cat2forquestion3] + ' take value ' + vals1[cat1forquestion3] + ' for ' + varname1short + '?</li>'
compcentsolhtml += '</ul>'

compcentsolhtml += '<p>'
compcentsolhtml += 'Solutions: The output from SPSS is as follows:'
compcentsolhtml += '</p>'
compcentsolhtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_4_000.png'].content)  + '">'
compcentsolhtml += '<p>'
compcentsolhtml += 'The answers are as follows:'
compcentsolhtml += '</p>'
compcentsolhtml += '<ul>'
compcentsolhtml += '<li>There are ' + str(percents_str[cat1forquestion1, cat2forquestion1]) + ' percent of observations where ' + varname2short + ' takes value ' + vals2[cat2forquestion1] + ' that take value ' + vals1[cat1forquestion1] + ' for ' + varname1short + '.</li>'
compcentsolhtml += '<li>There are ' + str(percents_str[cat1forquestion2, cat2forquestion2]) + ' percent of observations where ' + varname2short + ' takes value ' + vals2[cat2forquestion2] + ' that take value ' + vals1[cat1forquestion2] + ' for ' + varname1short + '.</li>'
compcentsolhtml += '<li>There are ' + str(percents_str[cat1forquestion3, cat2forquestion3]) + ' percent of observations where ' + varname2short + ' takes value ' + vals2[cat2forquestion3] + ' that take value ' + vals1[cat1forquestion3] + ' for ' + varname1short + '.</li>'
compcentsolhtml += '</ul>'
outputs['summaryanswers.html'] = HTMLOutput(compcentsolhtml, "Comparison percent answers text")

html = '<p>'
html += 'This ends our practical. '
html += '</p>'
outputs['conclusion.html'] = HTMLOutput(html, "Conclustion text")

combinedprachtml = outputs['introduction.html'].content
combinedprachtml += outputs['variable1.html'].content
combinedprachtml += outputs['variable2.html'].content
combinedprachtml += outputs['barchart1.html'].content
combinedprachtml += outputs['piecharts.html'].content
combinedprachtml += outputs['barchart2.html'].content
combinedprachtml += outputs['comparison.html'].content
combinedprachtml += outputs['summary.html'].content
combinedprachtml += outputs['conclusion.html'].content

outputs['allpractical.html'] = HTMLOutput(combinedprachtml, "Combined practical HTML outputs")

combinedquesthtml = outputs['introduction.html'].content
combinedquesthtml += outputs['variable1questions.html'].content
combinedquesthtml += outputs['variable2questions.html'].content
combinedquesthtml += outputs['barchart1questions.html'].content
combinedquesthtml += outputs['piechart1questions.html'].content
combinedquesthtml += outputs['barchart2questions.html'].content
combinedquesthtml += outputs['comparisonquestions.html'].content
combinedquesthtml += outputs['summaryquestions.html'].content

outputs['allquestions.html'] = HTMLOutput(combinedquesthtml, "Combined questions HTML outputs")

combinedsolhtml = outputs['variable1answers.html'].content
combinedsolhtml += outputs['variable2answers.html'].content
combinedsolhtml += outputs['barchart1answers.html'].content
combinedsolhtml += outputs['piechart1answers.html'].content
combinedsolhtml += outputs['barchart2answers.html'].content
combinedsolhtml += outputs['comparisonanswers.html'].content
combinedsolhtml += outputs['summaryanswers.html'].content

outputs['allanswers.html'] = HTMLOutput(combinedsolhtml, "Combined solutions HTML outputs")

'''