# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class Regression2(Template):
    'Calculate More advanced descriptive Statistics in SPSS (equivalent to Analyze->Descriptive Statistics->Explore... in SPSS 23)'

    __version__ = '1.0.0'

    tags = [ 'SPSS']
    engines = ['SPSS_script']

    inputs = '''
var = DataVector('Dependent Variable: ')
'''

    spssscript = '''
EXAMINE VARIABLES=${var}
  /PLOT HISTOGRAM NPPLOT
  /COMPARE GROUPS
  /NOTOTAL.

GGRAPH
  /GRAPHDATASET NAME="graphdataset" VARIABLES=${var} MISSING=LISTWISE REPORTMISSING=NO
  /GRAPHSPEC SOURCE=INLINE.
BEGIN GPL
  SOURCE: s=userSource(id("graphdataset"))
  DATA: ${var}=col(source(s), name("${var}"))
  DATA: id=col(source(s), name("$CASENUM"), unit.category())
  GUIDE: text.footnote(label("${datafile.variables[var]['varlabel']}"))
  ELEMENT: schema(position(bin.quantile.letter(1*${var})), color.interior(color."D3CE97"), label(id))
END GPL.
'''


    processoutputs = '''
import EStat
from EStat.Templating import *
import base64

from lxml import etree
from io import BytesIO

ns = {'x': 'http://www.ibm.com/software/analytics/spss/xml/oms'}

def str2float(s):
    try:
        return float(s)
    except ValueError:
        return float('nan')

caseout =  etree.parse(BytesIO(outputs['Table_Case Processing Summary.xml'].content))
print(etree.tostring(caseout, pretty_print=True))

path = "string(//x:dimension[@axis='row']/x:category/@text)"
casetext = caseout.xpath(path, namespaces=ns)
print(casetext)

path = "string(//x:dimension[@axis='row']/x:category/x:dimension[@text='Cases']/x:category[@text='Valid']/x:dimension[@text='Statistics']/x:category[@text='N']/x:cell/@number)"
Nvalid = int(caseout.xpath(path, namespaces=ns))
print(Nvalid)

path = "string(//x:dimension[@axis='row']/x:category/x:dimension[@text='Cases']/x:category[@text='Missing']/x:dimension[@text='Statistics']/x:category[@text='N']/x:cell/@number)"
Nmissing = int(caseout.xpath(path, namespaces=ns))
print(Nmissing)

path = "string(//x:dimension[@axis='row']/x:category/x:dimension[@text='Cases']/x:category[@text='Total']/x:dimension[@text='Statistics']/x:category[@text='N']/x:cell/@number)"
Ntotal = int(caseout.xpath(path, namespaces=ns))
print(Ntotal)

descsout = etree.parse(BytesIO(outputs['Table_Descriptives.xml'].content))
print(etree.tostring(descsout, pretty_print=True))

path = "string(//x:dimension[@text='Dependent Variables']/x:category/@label)"
varlab = descsout.xpath(path, namespaces=ns)
print(varlab)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
mean_num = str2float(descsout.xpath(path, namespaces=ns))
print(mean_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
mean_str = descsout.xpath(path, namespaces=ns)
print(mean_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@number)"
ses_num = str2float(descsout.xpath(path, namespaces=ns))
print(ses_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@text)"
ses_str = descsout.xpath(path, namespaces=ns)
print(ses_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval for Mean']/x:category[@text='Lower Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
lci_num = str2float(descsout.xpath(path, namespaces=ns))
print(lci_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval for Mean']/x:category[@text='Lower Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
lci_str = descsout.xpath(path, namespaces=ns)
print(lci_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval for Mean']/x:category[@text='Upper Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
uci_num = str2float(descsout.xpath(path, namespaces=ns))
print(uci_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval for Mean']/x:category[@text='Upper Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
uci_str = descsout.xpath(path, namespaces=ns)
print(uci_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='5% Trimmed Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
tms_num = str2float(descsout.xpath(path, namespaces=ns))
print(tms_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='5% Trimmed Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
tms_str = descsout.xpath(path, namespaces=ns)
print(tms_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Median']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
meds_num = str2float(descsout.xpath(path, namespaces=ns))
print(meds_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Median']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
meds_str = descsout.xpath(path, namespaces=ns)
print(meds_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Variance']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
varis_num = str2float(descsout.xpath(path, namespaces=ns))
print(varis_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Variance']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
varis_str = descsout.xpath(path, namespaces=ns)
print(varis_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Std. Deviation']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
sds_num = str2float(descsout.xpath(path, namespaces=ns))
print(sds_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Std. Deviation']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
sds_str = descsout.xpath(path, namespaces=ns)
print(sds_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Minimum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
mins_num = str2float(descsout.xpath(path, namespaces=ns))
print(mins_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Minimum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
mins_str = descsout.xpath(path, namespaces=ns)
print(mins_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Maximum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
maxs_num = str2float(descsout.xpath(path, namespaces=ns))
print(maxs_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Maximum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
maxs_str = descsout.xpath(path, namespaces=ns)
print(maxs_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
ranges_num = str2float(descsout.xpath(path, namespaces=ns))
print(ranges_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
ranges_str = descsout.xpath(path, namespaces=ns)
print(ranges_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Interquartile Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
iqrs_num = str2float(descsout.xpath(path, namespaces=ns))
print(iqrs_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Interquartile Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
iqrs_str = descsout.xpath(path, namespaces=ns)
print(iqrs_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
skews_num = str2float(descsout.xpath(path, namespaces=ns))
print(skews_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
skews_str = descsout.xpath(path, namespaces=ns)
print(skews_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@number)"
skewses_num = str2float(descsout.xpath(path, namespaces=ns))
print(skewses_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@text)"
skewses_str = descsout.xpath(path, namespaces=ns)
print(skewses_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
kurts_num = str2float(descsout.xpath(path, namespaces=ns))
print(kurts_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
kurts_str = descsout.xpath(path, namespaces=ns)
print(kurts_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@number)"
kurtses_num = str2float(descsout.xpath(path, namespaces=ns))
print(kurtses_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@text)"
kurtses_str = descsout.xpath(path, namespaces=ns)
print(kurtses_str)

statsout = etree.parse(BytesIO(outputs['Table_Tests of Normality.xml'].content))
print(etree.tostring(statsout, pretty_print=True))

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='Statistic']/x:cell/@number)"
kmstats_num = str2float(statsout.xpath(path, namespaces=ns))
print(kmstats_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='Statistic']/x:cell/@text)"
kmstats_str = statsout.xpath(path, namespaces=ns)
print(kmstats_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='df']/x:cell/@number)"
kmdfs = int(statsout.xpath(path, namespaces=ns))
print(kmdfs)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@number)"
kmsigs_num = str2float(statsout.xpath(path, namespaces=ns))
print(kmsigs_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@text)"
kmsigs_str = statsout.xpath(path, namespaces=ns)
print(kmsigs_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='Statistic']/x:cell/@text)"
swstats_str = statsout.xpath(path, namespaces=ns)
print(swstats_str)

if swstats_str != '': # For non-weighted data Shapiro-Wilk is only calculated for a sample size less than 5000 (see https://www.ibm.com/support/knowledgecenter/en/SSLVMB_24.0.0/spss/base/syn_examine_plot.html)
    path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='Statistic']/x:cell/@number)"
    swstats_num = str2float(statsout.xpath(path, namespaces=ns))
    print(swstats_num)

    path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='df']/x:cell/@text)"
    swdfs = int(statsout.xpath(path, namespaces=ns))
    print(swdfs)

    path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@number)"
    swsigs_num = str2float(statsout.xpath(path, namespaces=ns))
    print(swsigs_num)

    path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@text)"
    swsigs_str = statsout.xpath(path, namespaces=ns)
    print(swsigs_str)

varnameshort = str(var)
varnamelong = str(var)
if varlab != '':
    varnamelong = varlab + '[' + varnamelong + ']'

varnameshort = '<strong>' + varnameshort + '</strong>'
varnamelong = '<strong>' + varnamelong + '</strong>'

introhtml = '<h2>Checking for Normality</h2>'
introhtml += '<p>'
introhtml += 'In this practical we look at how we can use SPSS to investigate whether a variable can be assumed to be normally distributed. '
introhtml += 'This is an important decision as most of the parameteric statistical tests that we consider rely on the assumption that variables are normally distributed, unless sample sizes are very large. '
introhtml += 'We will look at this both graphically and through a statistical test known as the Kolmogorov-Smirnov test. '
introhtml += '</p>'
outputs['introduction.html'] = HTMLOutput(introhtml, "Introduction text")

histprachtml = '<p>'
histprachtml += 'We start by giving instructions on how to get the required graphs and the test statistics in SPSS which are accessed via the <strong>Explore</strong> option as detailed here:'
histprachtml += '</p>'

histinstrhtml = '<ul class = "instructions">'
histinstrhtml += '<li>Select <strong>Descriptive Statistics</strong> from the <strong>Analyze</strong> menu.</li>'
histinstrhtml += '<li>Select <strong>Explore</strong> from the <strong>Descriptive Statistics</strong> sub-menu.</li>'
histinstrhtml += '<li>Click on the <strong>Reset</strong> button.</li>'
histinstrhtml += '<li>Copy the ' + varnamelong + ' variables into the <strong>Dependent List:</strong> box.</li>'
histinstrhtml += '<li>Click on the <strong>Plots...</strong> button.</li>'
histinstrhtml += '<li>On the screen that appears select the <strong>Histogram</strong> tick box.</li>'
histinstrhtml += '<li>Unselect the <strong>Stem and leaf</strong> button.</li>'
histinstrhtml += '<li>Select the <strong>Normality plots with tests</strong> button.</li>'
histinstrhtml += '<li>Click on the <strong>Continue</strong> button.</li>'
histinstrhtml += '<li>Click on the <strong>OK</strong> button.</li>'
histinstrhtml += '</ul>'

histprachtml +=  histinstrhtml
histprachtml += '<p>'
histprachtml += 'We will first look at a histogram of the variable, ' + varnameshort + '. '
histprachtml += 'This can be found in amongst the set of output objects and looks as follows:'
histprachtml += '</p>'

histprachtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_000.png'].content)  + '">'

histprachtml += '<p>'
histprachtml += 'Ideally for a normal distribution this histogram should look symmetric around the mean of the distribution, in this case ' + mean_str + '. '
if skews_num > 2*skewses_num:
    histprachtml += 'This distribution appears to be skewed to the right (positively skewed). '
else:
    if -skews_num > 2*skewses_num:
        histprachtml += 'This distribution appears to be skewed to the left (negatively skewed). '
    else:
        histprachtml += 'This distribution appears to be reasonably symmetric. '
histprachtml += '</p>'

outputs['histogram.html'] = HTMLOutput(histprachtml, "Histogram text")

histquesthtml = '<p>'
histquesthtml += 'Use the Explore window to construct plots and tests for normality to answer the following questions: '
histquesthtml += '</p>'
histquesthtml += '<ul>'
histquesthtml += '<li>Question: Firstly considering the histogram for the variable, ' + varnameshort + '. Does this histogram suggest normality?</li>'
histquesthtml += '</ul>'
outputs['histogramquestions.html'] = HTMLOutput(histquesthtml, "Histogram question text")

histsolhtml = '<p>'
histsolhtml += 'The SPSS instructions are as follows: '
histsolhtml = '</p>'
histsolhtml += histinstrhtml
histsolhtml += '<ul>'
histsolhtml += '<li>Question: Firstly considering the histogram for the variable, ' + varnameshort + '. Does this histogram suggest normality?</li>'
histsolhtml += '</ul>'

histsolhtml += '<p>'
histsolhtml += 'Solution: The output histogram from SPSS looks as follows:'
histsolhtml += '</p>'
histsolhtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_000.png'].content)  + '">'
histsolhtml += '<p>'
histsolhtml += 'Ideally for a normal distribution this histogram should look symmetric around the mean of the distribution, in this case ' + mean_str + '. '
if skews_num > 2*skewses_num:
    histsolhtml += 'This distribution appears to be significantly skewed to the right (positively skewed) making us possibly question normality. '
else:
    if -skews_num > 2*skewses_num:
        histsolhtml += 'This distribution appears to be significantly skewed to the left (negatively skewed) making us possibly question normality. '
    else:
        histsolhtml += 'This distribution appears to be reasonably symmetric so there is no obvious skewness to make us question normality. '
histsolhtml += '</p>'
outputs['histogramanswers.html'] = HTMLOutput(histsolhtml, "Histogram answer text")

testprachtml = '<p>'
testprachtml += 'We will next look at a statistical test to see if this backs up our visual impressions from the histogram. '
testprachtml += '</p>'

testprachtml += '<p>'
testprachtml += 'The Kolmogorov-Smirnov test is used to test the null hypothesis that a set of data comes from a Normal distribution. '
if swstats_str != '':
    testprachtml += 'An alternative test derived by Shapiro and Wilk is also available in SPSS and both are presented in the table below that will be amongst the outputs from the Explore command:'
testprachtml += '</p>'

testprachtml += outputs['Table_Tests of Normality.htm'].content

testprachtml += '<p>'
if swstats_str != '':
    testprachtml += 'Both the Kolmogorov Smirnov and Shapiro Wilk tests produce test statistics that are used (along with a degrees of freedom parameter) to test for normality. '
else:
    testprachtml += 'The Kolmogorov Smirnov test produces test statistics that are used (along with a degrees of freedom parameter) to test for normality. '
testprachtml += 'Here we see that the Kolmogorov Smirnov statistic takes value ' + kmstats_str
if swstats_str != '':
    testprachtml += ' whilst the Shapiro-Wilks statistic takes value ' + swstats_str
testprachtml += '. '
if swstats_str != '':
    testprachtml += 'Both tests have the same degrees of freedom which equals the number of data points, namely ' + str(kmdfs) + '. '
else:
    testprachtml += 'This has degrees of freedom which equals the number of data points, namely ' + str(kmdfs) + '. '
testprachtml += '</p>'

testprachtml += '<p>'
if kmsigs_num > 0.05:
    testprachtml += 'Here we see that the p value (quoted under Sig. for Kolmogorov Smirnov  is ' + kmsigs_str + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the distribution is normal. '
else:
    if kmsigs_num < 0.001:
        testprachtml += 'Here we see the p-value provided by SPSS (quoted under Sig. for Kolmogorov-Smirnov) is ' + kmsigs_str + ' (reported as p &lt; .001). '
    else:
        testprachtml += 'Here we see that the p value (quoted under Sig. for Kolmogorov Smirnov) is ' + kmsigs_str + ' which is less than 0.05. '
    testprachtml += 'We therefore have significant evidence to reject the null hypothesis that the variable follows a normal distribution. '
if swstats_str != '':
    if kmsigs_num > 0.05:
        if swsigs_num > 0.05:
            testprachtml += 'The Shapiro-Wilks p value agrees with the Kolmogorov-Smirnov p value that the null hypothesis cannot be rejected. '
        else:
            testprachtml += 'Unusually the Shapiro-Wilks p value disagrees with the Kolmogorov-Smirnov p value and suggests that the null hypothesis can be rejected. '
            testprachtml += 'In this case we would use the histogram and QQ plot to inform our decision. '
    if kmsigs_num < 0.05:
        if swsigs_num  < 0.05:
            testprachtml += 'The Shapiro-Wilks p value agrees with the Kolmogorov-Smirnov p value that the null hypothesis can be rejected. '
        else:
            testprachtml += 'Unusually the Shapiro-Wilks p value disagrees with the Kolmogorov-Smirnov p value and suggests that the null hypothesis cannot be rejected. '
            testprachtml += 'In this case we would use the histogram and QQ plot to inform our decision. '
testprachtml += '</p>'

testprachtml += '<p>'
testprachtml += 'Although both these statistics tell the researcher whether the distribution followed by a variable is statistically significantly different from a normal distribution one should take care in not overinterpreting such findings. '
testprachtml += 'Significance will be strongly affected by the number of observations and so only a small discrepancy from normality will be deemed significant for very large sample sizes '
if Nvalid > 500 and kmsigs_num < 0.05:
    testprachtml += 'as may be the case here '
testprachtml += 'whilst very large discrepancies will be required to reject the null hypothesis for small sample sizes. '
testprachtml += 'It should also be remembered that many parametric statistics are robust to non-normality when sample sizes are very large (employing the Central Limit Theorem), so the implications of non-normality are primarily of interest is designs with smaller sample sizes. '
testprachtml += '</p>'
outputs['tests.html'] = HTMLOutput(testprachtml, "Normality test text")

testquesthtml = '<ul>'
testquesthtml += '<li> Question: Next looking at the two Normality test statistics do they suggest normality?</li>'
testquesthtml += '</ul>'
outputs['testquestions.html'] = HTMLOutput(testquesthtml, "Normality test question text")

testsolhtml = '<ul>'
testsolhtml += '<li> Question: Next looking at the two Normality test statistics do they suggest normality?</li>'
testsolhtml += '</ul>'
testsolhtml += '<p>'
testsolhtml += 'Solution: The output of the test statistics from SPSS is as follows'
testsolhtml += '</p>'
testsolhtml += outputs['Table_Tests of Normality.htm'].content
testsolhtml += 'Here we see that the Kolmogorov Smirnov statistic takes value ' + kmstats_str
if swstats_str != '':
    testsolhtml += ' whilst the Shapiro-Wilks statistic takes value ' + swstats_str
testsolhtml += '. '
if swstats_str != '':
    testsolhtml += 'Both tests have the same degrees of freedom which equals the number of data points, namely ' + str(kmdfs) + '. '
else:
    testsolhtml += 'This has degrees of freedom which equals the number of data points, namely ' + str(kmdfs) + '. '
testsolhtml += '</p>'

testsolhtml += '<p>'
if kmsigs_num > 0.05:
    testsolhtml += 'Here we see that the p value (quoted under Sig. for Kolmogorov Smirnov  is ' + kmsigs_str + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the distribution is normal. '
else:
    if kmsigs_num < 0.001:
        testsolhtml += 'Here we see the p-value provided by SPSS (quoted under Sig. for Kolmogorov-Smirnov) is ' + kmsigs_str + ' (reported as p &lt; .001). '
    else:
        testsolhtml += 'Here we see that the p value (quoted under Sig. for Kolmogorov Smirnov) is ' + kmsigs_str + ' which is less than 0.05. '
    testsolhtml += 'We therefore have significant evidence to reject the null hypothesis that the variable follows a normal distribution. '
if swstats_str != '':
    if kmsigs_num > 0.05:
        if swsigs_num > 0.05:
            testsolhtml += 'The Shapiro-Wilks p value agrees with the Kolmogorov-Smirnov p value that the null hypothesis cannot be rejected. '
        else:
            testsolhtml += 'Unusually the Shapiro-Wilks p value disagrees with the Kolmogorov-Smirnov p value and suggests that the null hypothesis can be rejected. '
            testsolhtml += 'In this case we would use the histogram and QQ plot to inform our decision. '
    if kmsigs_num < 0.05:
        if swsigs_num  < 0.05:
            testsolhtml += 'The Shapiro-Wilks p value agrees with the Kolmogorov-Smirnov p value that the null hypothesis can be rejected. '
        else:
            testsolhtml += 'Unusually the Shapiro-Wilks p value disagrees with the Kolmogorov-Smirnov p value and suggests that the null hypothesis cannot be rejected. '
            testsolhtml += 'In this case we would use the histogram and QQ plot to inform our decision. '
testsolhtml += '</p>'

testsolhtml += '<p>'
testsolhtml += 'As explained in the practical, formal tests of normality are only one piece of evidence that should be used in judgements of whether the distributional assumptions of parametric tests are likely to be violated. '
testsolhtml += 'Visual inspection and consideration of sample sizes are equally, if not more, important. '
testsolhtml += '</p>'
outputs['testanswers.html'] = HTMLOutput(testsolhtml , "Normality test answer text")

qqprachtml = '<p>'
qqprachtml += 'To complete our practical on checking for normality SPSS also produces a Quantile-Quantile (or QQ) plot that can be seen below:'
qqprachtml += '</p>'

qqprachtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_1_000.png'].content)  + '">'

qqprachtml += '<p>'
qqprachtml += 'QQ plots can be used to compare the distribution of a variable with a chosen distribution (typically a normal distribution as we are doing here). '
qqprachtml += 'The data are plotted against a theoretical normal distribution (with the same mean and variance as the sample data) in such a way that the points should form an approximate straight line. '
qqprachtml += 'Departures from this straight line indicate departures from normality. '
if kmsigs_num < 0.05:
    qqprachtml += 'As we found a significant effect in the Kolmogorov Smirnov test we should see points diverging from the line in the plot above with either some outlying values lying away from the line or even the shape of the points forming a non-linear pattern. '
else:
    qqprachtml += 'In this case we were not able to reject the null hypothesis of normality via the Kolmogorov Smirnov test and so the lines should generally lie on or close to the line given. '
qqprachtml += '</p>'
outputs['qqplot.html'] = HTMLOutput(qqprachtml, "QQ plot test")

qqquesthtml = '<ul>'
qqquesthtml += '<li>Question: Finally does the QQ plot produced by SPSS suggest normality?</li>'
qqquesthtml += '</ul>'
outputs['qqplotquestions.html'] = HTMLOutput(qqquesthtml, "QQ plot test question text")

qqsolhtml = '<ul>'
qqsolhtml += '<li>Question: Finally does the QQ plot produced by SPSS suggest normality?</li>'
qqsolhtml += '</ul>'
qqsolhtml += '<p>'
qqsolhtml += 'Solution: The output from SPSS is as follows'
qqsolhtml += '</p>'
qqsolhtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_1_000.png'].content)  + '">'
qqsolhtml += '<p>'
qqsolhtml += 'In a QQ plot of normally distributed data all the points will lie on the diagonal black line. '
if kmsigs_num < 0.05:
    qqsolhtml += 'As we found a significant effect in the Kolmogorov Smirnov test we should see points diverging from the line in the plot above with either some outlying values lying away from the line or even the shape of the points forming a non-linear pattern. '
else:
    qqsolhtml += 'In this case we were not able to reject the null hypothesis of normality via the Kolmogorov Smirnov test and so the lines should generally lie on or close to the line given. '
qqsolhtml += '</p>'

outputs['qqplotanswers.html'] = HTMLOutput(qqsolhtml , "QQ plot test answer text")

combinedprachtml = outputs['introduction.html'].content
combinedprachtml += outputs['histogram.html'].content
combinedprachtml += outputs['tests.html'].content
combinedprachtml += outputs['qqplot.html'].content

outputs['allpractical.html'] = HTMLOutput(combinedprachtml, "Combined practical HTML outputs")

combinedquesthtml = outputs['introduction.html'].content
combinedquesthtml += outputs['histogramquestions.html'].content
combinedquesthtml += outputs['testquestions.html'].content
combinedquesthtml += outputs['qqplotquestions.html'].content

outputs['allquestions.html'] = HTMLOutput(combinedquesthtml, "Combined questions HTML outputs")

combinedsolhtml = outputs['histogramanswers.html'].content
combinedsolhtml += outputs['testanswers.html'].content
combinedsolhtml += outputs['qqplotanswers.html'].content

outputs['allanswers.html'] = HTMLOutput(combinedsolhtml, "Combined solutions HTML outputs")

'''



    spssgui = r'''
\begin{aligned}
&\text{Select }\textbf{Descriptive Statistics}\text{ from the }\textbf{Analyze}\text{ menu.} \\ 
&\text{Select }\textbf{Explore}\text{ from the }\textbf{Descriptive Statistics}\text{ sub-menu.} \\ 
&\text{Click on the } \textbf{Reset}\text{ button } \\ 
&\text{Copy the } \textbf{${datafile.variables[var]['varlabel'] + ' ['+var+']' }}\text{ variables into the } \textbf{Dependent List:}\text{ box. } \\  
&\text{Click on the } \textbf{Plots...}\text{ button } \\ 
&\text{On the screen that appears select the } \textbf{Histogram}\text{ tick box } \\ 
&\text{Unselect the } \textbf{Stem and leaf}\text{ button } \\ 
&\text{Select the } \textbf{Normality plots with tests}\text{ button } \\ 
&\text{Click on the } \textbf{Continue}\text{ button } \\ 
&\text{Click on the } \textbf{OK}\text{ button } \\  
\end{aligned}
'''
