# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class Regression2(Template):
    'Analyze->Compare Means->Independent-Sample T Test...'

    __version__ = '1.0.0'

    tags = [ 'SPSS']
    engines = ['SPSS_script']

    inputs = '''
testvar = DataVector('Test Variable: ', help='Select a continuous variable')
groupvar = DataVector('Grouping Variable: ', help='Here specify the category code numbers of the two groups to be tested (values taken by the Grouping Variable ' + str(testvar) + ')')
group1 = Integer('Group 1: ', help='Here specify the first category code number of the two groups to be tested (values taken by the Grouping Variable ' + str(groupvar) + ')')
group2 = Integer('Group 2: ', help='Here specify the second category code number of the two groups to be tested (values taken by the Grouping Variable ' + str(groupvar) + ')')
options = Boolean('Options?', default = False)
if options:
    percentvals = Integer('Confidence Interval Percentile: ', default = 95)
    missingvals = Text('Missing Values', ['Exclude cases analysis by analysis', 'Exclude cases listwise'])
else:
    percentvals = Integer(value=95)
'''

    spssscript = '''

EXAMINE VARIABLES=${testvar} BY ${groupvar}
  /PLOT HISTOGRAM NPPLOT
  /COMPARE GROUPS
  /CINTERVAL ${percentvals}
  /NOTOTAL.

GRAPH
  /ERRORBAR(CI ${percentvals})=${testvar} BY ${groupvar}.

T-TEST GROUPS=${groupvar}(${group1} ${group2})
 /VARIABLES=${testvar}
% if options:
 /CRITERIA=CI(0.${percentvals})
% if missingvals == 'Exclude cases analysis by analysis':
 /MISSING=ANALYSIS
% endif
% if missingvals == 'Exclude cases listwise':
 /MISSING=LISTWISE
% endif
% endif
.
'''


    processoutputs = '''
import EStat
from EStat.Templating import *

import base64

import numpy

from lxml import etree
from io import BytesIO

ns = {'x': 'http://www.ibm.com/software/analytics/spss/xml/oms'}

def str2float(s):
    try:
        return float(s)
    except ValueError:
        return float('nan')

group1path = "/x:category[@number='" + str(group1) + "']"
group2path = "/x:category[@number='" + str(group2) + "']"

descsout = etree.parse(BytesIO(outputs['Table_Descriptives.xml'].content))
print(etree.tostring(descsout, pretty_print=True))

path = "string(//x:dimension[@text='Dependent Variables']/x:category/@label)"
varlab = descsout.xpath(path, namespaces=ns)
print(varlab)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
mean1_num = str2float(descsout.xpath(path, namespaces=ns))
print(mean1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
mean1_str = descsout.xpath(path, namespaces=ns)
print(mean1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@number)"
ses1_num = str2float(descsout.xpath(path, namespaces=ns))
print(ses1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@text)"
ses1_str = descsout.xpath(path, namespaces=ns)
print(ses1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Statistics']/x:group[@text='" + str(percentvals) + "% Confidence Interval for Mean']/x:category[@text='Lower Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
lci1_num = str2float(descsout.xpath(path, namespaces=ns))
print(lci1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Statistics']/x:group[@text='" + str(percentvals) + "% Confidence Interval for Mean']/x:category[@text='Lower Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
lci1_str = descsout.xpath(path, namespaces=ns)
print(lci1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Statistics']/x:group[@text='" + str(percentvals) + "% Confidence Interval for Mean']/x:category[@text='Upper Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
uci1_num = str2float(descsout.xpath(path, namespaces=ns))
print(uci1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Statistics']/x:group[@text='" + str(percentvals) + "% Confidence Interval for Mean']/x:category[@text='Upper Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
uci1_str = descsout.xpath(path, namespaces=ns)
print(uci1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Statistics']/x:category[@text='5% Trimmed Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
tms1_num = str2float(descsout.xpath(path, namespaces=ns))
print(tms1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Statistics']/x:category[@text='5% Trimmed Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
tms1_str = descsout.xpath(path, namespaces=ns)
print(tms1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Statistics']/x:category[@text='Median']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
meds1_num = str2float(descsout.xpath(path, namespaces=ns))
print(meds1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Statistics']/x:category[@text='Median']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
meds1_str = descsout.xpath(path, namespaces=ns)
print(meds1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Statistics']/x:category[@text='Variance']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
varis1_num = str2float(descsout.xpath(path, namespaces=ns))
print(varis1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Statistics']/x:category[@text='Variance']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
varis1_str = descsout.xpath(path, namespaces=ns)
print(varis1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Statistics']/x:category[@text='Std. Deviation']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
sds1_num = str2float(descsout.xpath(path, namespaces=ns))
print(sds1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Statistics']/x:category[@text='Std. Deviation']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
sds1_str = descsout.xpath(path, namespaces=ns)
print(sds1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Statistics']/x:category[@text='Minimum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
mins1_num = str2float(descsout.xpath(path, namespaces=ns))
print(mins1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Statistics']/x:category[@text='Minimum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
mins1_str = descsout.xpath(path, namespaces=ns)
print(mins1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Statistics']/x:category[@text='Maximum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
maxs1_num = str2float(descsout.xpath(path, namespaces=ns))
print(maxs1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Statistics']/x:category[@text='Maximum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
maxs1_str = descsout.xpath(path, namespaces=ns)
print(maxs1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Statistics']/x:category[@text='Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
ranges1_num = str2float(descsout.xpath(path, namespaces=ns))
print(ranges1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Statistics']/x:category[@text='Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
ranges1_str = descsout.xpath(path, namespaces=ns)
print(ranges1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Statistics']/x:category[@text='Interquartile Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
iqrs1_num = str2float(descsout.xpath(path, namespaces=ns))
print(iqrs1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Statistics']/x:category[@text='Interquartile Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
iqrs1_str = descsout.xpath(path, namespaces=ns)
print(iqrs1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
skews1_num = str2float(descsout.xpath(path, namespaces=ns))
print(skews1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
skews1_str = descsout.xpath(path, namespaces=ns)
print(skews1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@number)"
skewses1_num = str2float(descsout.xpath(path, namespaces=ns))
print(skewses1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@text)"
skewses1_str = descsout.xpath(path, namespaces=ns)
print(skewses1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
kurts1_num = str2float(descsout.xpath(path, namespaces=ns))
print(kurts1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
kurts1_str = descsout.xpath(path, namespaces=ns)
print(kurts1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@number)"
kurtses1_num = str2float(descsout.xpath(path, namespaces=ns))
print(kurtses1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@text)"
kurtses1_str = descsout.xpath(path, namespaces=ns)
print(kurtses1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
mean2_num = str2float(descsout.xpath(path, namespaces=ns))
print(mean2_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
mean2_str = descsout.xpath(path, namespaces=ns)
print(mean2_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@number)"
ses2_num = str2float(descsout.xpath(path, namespaces=ns))
print(ses2_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@text)"
ses2_str = descsout.xpath(path, namespaces=ns)
print(ses2_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Statistics']/x:group[@text='" + str(percentvals) + "% Confidence Interval for Mean']/x:category[@text='Lower Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
lci2_num = str2float(descsout.xpath(path, namespaces=ns))
print(lci2_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Statistics']/x:group[@text='" + str(percentvals) + "% Confidence Interval for Mean']/x:category[@text='Lower Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
lci2_str = descsout.xpath(path, namespaces=ns)
print(lci2_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Statistics']/x:group[@text='" + str(percentvals) + "% Confidence Interval for Mean']/x:category[@text='Upper Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
uci2_num = str2float(descsout.xpath(path, namespaces=ns))
print(uci2_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Statistics']/x:group[@text='" + str(percentvals) + "% Confidence Interval for Mean']/x:category[@text='Upper Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
uci2_str = descsout.xpath(path, namespaces=ns)
print(uci2_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Statistics']/x:category[@text='5% Trimmed Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
tms2_num = str2float(descsout.xpath(path, namespaces=ns))
print(tms2_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Statistics']/x:category[@text='5% Trimmed Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
tms2_str = descsout.xpath(path, namespaces=ns)
print(tms2_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Statistics']/x:category[@text='Median']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
meds1_num = str2float(descsout.xpath(path, namespaces=ns))
print(meds1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Statistics']/x:category[@text='Median']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
meds2_str = descsout.xpath(path, namespaces=ns)
print(meds2_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Statistics']/x:category[@text='Variance']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
varis2_num = str2float(descsout.xpath(path, namespaces=ns))
print(varis2_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Statistics']/x:category[@text='Variance']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
varis2_str = descsout.xpath(path, namespaces=ns)
print(varis2_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Statistics']/x:category[@text='Std. Deviation']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
sds2_num = str2float(descsout.xpath(path, namespaces=ns))
print(sds2_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Statistics']/x:category[@text='Std. Deviation']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
sds2_str = descsout.xpath(path, namespaces=ns)
print(sds2_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Statistics']/x:category[@text='Minimum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
mins2_num = str2float(descsout.xpath(path, namespaces=ns))
print(mins2_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Statistics']/x:category[@text='Minimum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
mins2_str = descsout.xpath(path, namespaces=ns)
print(mins2_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Statistics']/x:category[@text='Maximum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
maxs2_num = str2float(descsout.xpath(path, namespaces=ns))
print(maxs2_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Statistics']/x:category[@text='Maximum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
maxs2_str = descsout.xpath(path, namespaces=ns)
print(maxs2_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Statistics']/x:category[@text='Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
ranges2_num = str2float(descsout.xpath(path, namespaces=ns))
print(ranges2_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Statistics']/x:category[@text='Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
ranges1_str = descsout.xpath(path, namespaces=ns)
print(ranges1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Statistics']/x:category[@text='Interquartile Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
iqrs1_num = str2float(descsout.xpath(path, namespaces=ns))
print(iqrs1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Statistics']/x:category[@text='Interquartile Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
iqrs2_str = descsout.xpath(path, namespaces=ns)
print(iqrs2_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
skews2_num = str2float(descsout.xpath(path, namespaces=ns))
print(skews2_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
skews2_str = descsout.xpath(path, namespaces=ns)
print(skews2_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@number)"
skewses2_num = str2float(descsout.xpath(path, namespaces=ns))
print(skewses2_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@text)"
skewses2_str = descsout.xpath(path, namespaces=ns)
print(skewses2_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
kurts2_num = str2float(descsout.xpath(path, namespaces=ns))
print(kurts2_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
kurts2_str = descsout.xpath(path, namespaces=ns)
print(kurts2_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@number)"
kurtses2_num = str2float(descsout.xpath(path, namespaces=ns))
print(kurtses2_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@text)"
kurtses2_str = descsout.xpath(path, namespaces=ns)
print(kurtses2_str)

statsout = etree.parse(BytesIO(outputs['Table_Tests of Normality.xml'].content))
print(etree.tostring(statsout, pretty_print=True))

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='Statistic']/x:cell/@number)"
kmstats1_num = str2float(statsout.xpath(path, namespaces=ns))
print(kmstats1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='Statistic']/x:cell/@text)"
kmstats1_str = statsout.xpath(path, namespaces=ns)
print(kmstats1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='df']/x:cell/@number)"
kmdfs1 = int(statsout.xpath(path, namespaces=ns))
print(kmdfs1)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@number)"
kmsigs1_num = str2float(statsout.xpath(path, namespaces=ns))
print(kmsigs1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@text)"
kmsigs1_str = statsout.xpath(path, namespaces=ns)
print(kmsigs1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='Statistic']/x:cell/@text)"
swstats1_str = statsout.xpath(path, namespaces=ns)
print(swstats1_str)

if swstats1_str != '': # For non-weighted data Shapiro-Wilk is only calculated for a sample size less than 5000 (see https://www.ibm.com/support/knowledgecenter/en/SSLVMB_24.0.0/spss/base/syn_examine_plot.html)
    path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='Statistic']/x:cell/@number)"
    swstats1_num = str2float(statsout.xpath(path, namespaces=ns))
    print(swstats1_num)

    path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='df']/x:cell/@text)"
    swdfs1 = int(statsout.xpath(path, namespaces=ns))
    print(swdfs1)

    path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@number)"
    swsigs1_num = str2float(statsout.xpath(path, namespaces=ns))
    print(swsigs1_num)

    path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group1) + "]/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@text)"
    swsigs1_str = statsout.xpath(path, namespaces=ns)
    print(swsigs1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='Statistic']/x:cell/@number)"
kmstats2_num = str2float(statsout.xpath(path, namespaces=ns))
print(kmstats2_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='Statistic']/x:cell/@text)"
kmstats2_str = statsout.xpath(path, namespaces=ns)
print(kmstats2_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='df']/x:cell/@number)"
kmdfs2 = int(statsout.xpath(path, namespaces=ns))
print(kmdfs2)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@number)"
kmsigs2_num = str2float(statsout.xpath(path, namespaces=ns))
print(kmsigs2_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@text)"
kmsigs2_str = statsout.xpath(path, namespaces=ns)
print(kmsigs2_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='Statistic']/x:cell/@text)"
swstats2_str = statsout.xpath(path, namespaces=ns)
print(swstats2_str)

if swstats2_str != '': # For non-weighted data Shapiro-Wilk is only calculated for a sample size less than 5000 (see https://www.ibm.com/support/knowledgecenter/en/SSLVMB_24.0.0/spss/base/syn_examine_plot.html)
    path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='Statistic']/x:cell/@number)"
    swstats2_num = str2float(statsout.xpath(path, namespaces=ns))
    print(swstats2_num)

    path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='df']/x:cell/@text)"
    swdfs2 = int(statsout.xpath(path, namespaces=ns))
    print(swdfs2)

    path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@number)"
    swsigs2_num = str2float(statsout.xpath(path, namespaces=ns))
    print(swsigs2_num)

    path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[@number=" + str(group2) + "]/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@text)"
    swsigs2_str = statsout.xpath(path, namespaces=ns)
    print(swsigs2_str)

statsout = etree.parse(BytesIO(outputs['Table_Group Statistics.xml'].content))
print(etree.tostring(statsout, pretty_print=True))

path = "string(//x:category/x:dimension" + group1path  + "/x:dimension[@text='Statistics']/x:category[@text='N']/x:cell/@number)"
N1 = int(statsout.xpath(path, namespaces=ns))
print(N1)

path = "string(//x:category/x:dimension" + group2path  + "/x:dimension[@text='Statistics']/x:category[@text='N']/x:cell/@number)"
N2 = int(statsout.xpath(path, namespaces=ns))
print(N2)

path = "string(//x:category/x:dimension" + group1path  + "/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:cell/@number)"
mean1_num = str2float(statsout.xpath(path, namespaces=ns))
print(mean1_num)

path = "string(//x:category/x:dimension" + group1path  + "/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:cell/@text)"
mean1_str = statsout.xpath(path, namespaces=ns)
print(mean1_str)

path = "string(//x:category/x:dimension" + group2path  + "/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:cell/@number)"
mean2_num = str2float(statsout.xpath(path, namespaces=ns))
print(mean2_num)

path = "string(//x:category/x:dimension" + group2path  + "/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:cell/@text)"
mean2_str = statsout.xpath(path, namespaces=ns)
print(mean2_str)

path = "string(//x:category/x:dimension" + group1path  + "/x:dimension[@text='Statistics']/x:category[@text='Std. Deviation']/x:cell/@number)"
sd1_num = str2float(statsout.xpath(path, namespaces=ns))
print(sd1_num)

path = "string(//x:category/x:dimension" + group1path  + "/x:dimension[@text='Statistics']/x:category[@text='Std. Deviation']/x:cell/@text)"
sd1_str = statsout.xpath(path, namespaces=ns)
print(sd1_str)

path = "string(//x:category/x:dimension" + group2path  + "/x:dimension[@text='Statistics']/x:category[@text='Std. Deviation']/x:cell/@number)"
sd2_num = str2float(statsout.xpath(path, namespaces=ns))
print(sd2_num)

path = "string(//x:category/x:dimension" + group2path  + "/x:dimension[@text='Statistics']/x:category[@text='Std. Deviation']/x:cell/@text)"
sd2_str = statsout.xpath(path, namespaces=ns)
print(sd2_str)

path = "string(//x:category/x:dimension" + group1path  + "/x:dimension[@text='Statistics']/x:category[@text='Std. Error Mean']/x:cell/@number)"
semean1_num = str2float(statsout.xpath(path, namespaces=ns))
print(semean1_num)

path = "string(//x:category/x:dimension" + group1path  + "/x:dimension[@text='Statistics']/x:category[@text='Std. Error Mean']/x:cell/@text)"
semean1_str = statsout.xpath(path, namespaces=ns)
print(semean1_str)

path = "string(//x:category/x:dimension" + group2path  + "/x:dimension[@text='Statistics']/x:category[@text='Std. Error Mean']/x:cell/@number)"
semean2_num = str2float(statsout.xpath(path, namespaces=ns))
print(semean2_num)

path = "string(//x:category/x:dimension" + group2path  + "/x:dimension[@text='Statistics']/x:category[@text='Std. Error Mean']/x:cell/@text)"
semean2_str = statsout.xpath(path, namespaces=ns)
print(semean2_str)

path = "string(//x:category[@variable='true']/@text)"
varlab = statsout.xpath(path, namespaces=ns)
print(varlab)

path = "string(//x:category/x:dimension/@label)"
grouplab = statsout.xpath(path, namespaces=ns)
print(grouplab)

path = "string(//x:category/x:dimension" + group1path  + "/@text)"
group1lab = statsout.xpath(path, namespaces=ns)
print(group1lab)

path = "string(//x:category/x:dimension" + group2path  + "/@text)"
group2lab = statsout.xpath(path, namespaces=ns)
print(group2lab)

testout =  etree.parse(BytesIO(outputs['Table_Independent Samples Test.xml'].content))
print(etree.tostring(testout, pretty_print=True))

# NOTE: lxml does not like the apostrophe in 'Levene's Test for Equality of Variances'
path = "string(//x:category/x:dimension[@text='Assumptions']/x:category[@text='Equal variances assumed']/x:dimension[@text='Statistics']/x:group[contains(@text, 'Levene')]/x:category[@text='F']/x:cell/@number)"
levenef_num = str2float(testout.xpath(path, namespaces=ns))
print(levenef_num)

path = "string(//x:category/x:dimension[@text='Assumptions']/x:category[@text='Equal variances assumed']/x:dimension[@text='Statistics']/x:group[contains(@text, 'Levene')]/x:category[@text='F']/x:cell/@text)"
levenef_str = testout.xpath(path, namespaces=ns)
print(levenef_str)

path = "string(//x:category/x:dimension[@text='Assumptions']/x:category[@text='Equal variances assumed']/x:dimension[@text='Statistics']/x:group[contains(@text, 'Levene')]/x:category[@text='Sig.']/x:cell/@number)"
levenesig_num = str2float(testout.xpath(path, namespaces=ns))
print(levenesig_num)

path = "string(//x:category/x:dimension[@text='Assumptions']/x:category[@text='Equal variances assumed']/x:dimension[@text='Statistics']/x:group[contains(@text, 'Levene')]/x:category[@text='Sig.']/x:cell/@text)"
levenesig_str = testout.xpath(path, namespaces=ns)
print(levenesig_str)

path = "string(//x:category/x:dimension[@text='Assumptions']/x:category[@text='Equal variances assumed']/x:dimension[@text='Statistics']/x:group[@text='t-test for Equality of Means']/x:category[@text='t']/x:cell/@number)"
teq_num = str2float(testout.xpath(path, namespaces=ns))
print(teq_num)

path = "string(//x:category/x:dimension[@text='Assumptions']/x:category[@text='Equal variances assumed']/x:dimension[@text='Statistics']/x:group[@text='t-test for Equality of Means']/x:category[@text='t']/x:cell/@text)"
teq_str = testout.xpath(path, namespaces=ns)
print(teq_str)

path = "string(//x:category/x:dimension[@text='Assumptions']/x:category[@text='Equal variances not assumed']/x:dimension[@text='Statistics']/x:group[@text='t-test for Equality of Means']/x:category[@text='t']/x:cell/@number)"
tneq_num = str2float(testout.xpath(path, namespaces=ns))
print(tneq_num)

path = "string(//x:category/x:dimension[@text='Assumptions']/x:category[@text='Equal variances not assumed']/x:dimension[@text='Statistics']/x:group[@text='t-test for Equality of Means']/x:category[@text='t']/x:cell/@text)"
tneq_str = testout.xpath(path, namespaces=ns)
print(tneq_str)

path = "string(//x:category/x:dimension[@text='Assumptions']/x:category[@text='Equal variances assumed']/x:dimension[@text='Statistics']/x:group[@text='t-test for Equality of Means']/x:category[@text='df']/x:cell/@number)"
dfeq_num = str2float(testout.xpath(path, namespaces=ns))
print(dfeq_num)

path = "string(//x:category/x:dimension[@text='Assumptions']/x:category[@text='Equal variances assumed']/x:dimension[@text='Statistics']/x:group[@text='t-test for Equality of Means']/x:category[@text='df']/x:cell/@text)"
dfeq_str = testout.xpath(path, namespaces=ns)
print(dfeq_str)

path = "string(//x:category/x:dimension[@text='Assumptions']/x:category[@text='Equal variances not assumed']/x:dimension[@text='Statistics']/x:group[@text='t-test for Equality of Means']/x:category[@text='df']/x:cell/@number)"
dfneq_num = str2float(testout.xpath(path, namespaces=ns))
print(dfneq_num)

path = "string(//x:category/x:dimension[@text='Assumptions']/x:category[@text='Equal variances not assumed']/x:dimension[@text='Statistics']/x:group[@text='t-test for Equality of Means']/x:category[@text='df']/x:cell/@text)"
dfneq_str = testout.xpath(path, namespaces=ns)
print(dfneq_str)

path = "string(//x:category/x:dimension[@text='Assumptions']/x:category[@text='Equal variances assumed']/x:dimension[@text='Statistics']/x:group[@text='t-test for Equality of Means']/x:category[@text='Sig. (2-tailed)']/x:cell/@number)"
sigeq_num = str2float(testout.xpath(path, namespaces=ns))
print(sigeq_num)

path = "string(//x:category/x:dimension[@text='Assumptions']/x:category[@text='Equal variances assumed']/x:dimension[@text='Statistics']/x:group[@text='t-test for Equality of Means']/x:category[@text='Sig. (2-tailed)']/x:cell/@text)"
sigeq_str = testout.xpath(path, namespaces=ns)
print(sigeq_str)

path = "string(//x:category/x:dimension[@text='Assumptions']/x:category[@text='Equal variances not assumed']/x:dimension[@text='Statistics']/x:group[@text='t-test for Equality of Means']/x:category[@text='Sig. (2-tailed)']/x:cell/@number)"
signeq_num = str2float(testout.xpath(path, namespaces=ns))
print(signeq_num)

path = "string(//x:category/x:dimension[@text='Assumptions']/x:category[@text='Equal variances not assumed']/x:dimension[@text='Statistics']/x:group[@text='t-test for Equality of Means']/x:category[@text='Sig. (2-tailed)']/x:cell/@text)"
signeq_str = testout.xpath(path, namespaces=ns)
print(signeq_str)

path = "string(//x:category/x:dimension[@text='Assumptions']/x:category[@text='Equal variances assumed']/x:dimension[@text='Statistics']/x:group[@text='t-test for Equality of Means']/x:category[@text='Mean Difference']/x:cell/@number)"
meandiffeq_num = str2float(testout.xpath(path, namespaces=ns))
print(meandiffeq_num)

path = "string(//x:category/x:dimension[@text='Assumptions']/x:category[@text='Equal variances assumed']/x:dimension[@text='Statistics']/x:group[@text='t-test for Equality of Means']/x:category[@text='Mean Difference']/x:cell/@text)"
meandiffeq_str = testout.xpath(path, namespaces=ns)
print(meandiffeq_str)

path = "string(//x:category/x:dimension[@text='Assumptions']/x:category[@text='Equal variances not assumed']/x:dimension[@text='Statistics']/x:group[@text='t-test for Equality of Means']/x:category[@text='Mean Difference']/x:cell/@number)"
meandiffneq_num = str2float(testout.xpath(path, namespaces=ns))
print(meandiffneq_num)

path = "string(//x:category/x:dimension[@text='Assumptions']/x:category[@text='Equal variances not assumed']/x:dimension[@text='Statistics']/x:group[@text='t-test for Equality of Means']/x:category[@text='Mean Difference']/x:cell/@text)"
meandiffneq_str = testout.xpath(path, namespaces=ns)
print(meandiffneq_str)

path = "string(//x:category/x:dimension[@text='Assumptions']/x:category[@text='Equal variances assumed']/x:dimension[@text='Statistics']/x:group[@text='t-test for Equality of Means']/x:category[@text='Std. Error Difference']/x:cell/@number)"
seeq_num = str2float(testout.xpath(path, namespaces=ns))
print(seeq_num)

path = "string(//x:category/x:dimension[@text='Assumptions']/x:category[@text='Equal variances assumed']/x:dimension[@text='Statistics']/x:group[@text='t-test for Equality of Means']/x:category[@text='Std. Error Difference']/x:cell/@text)"
seeq_str = testout.xpath(path, namespaces=ns)
print(seeq_str)

path = "string(//x:category/x:dimension[@text='Assumptions']/x:category[@text='Equal variances not assumed']/x:dimension[@text='Statistics']/x:group[@text='t-test for Equality of Means']/x:category[@text='Std. Error Difference']/x:cell/@number)"
seneq_num = str2float(testout.xpath(path, namespaces=ns))
print(seneq_num)

path = "string(//x:category/x:dimension[@text='Assumptions']/x:category[@text='Equal variances not assumed']/x:dimension[@text='Statistics']/x:group[@text='t-test for Equality of Means']/x:category[@text='Std. Error Difference']/x:cell/@text)"
seneq_str = testout.xpath(path, namespaces=ns)
print(seneq_str)

path = "string(//x:category/x:dimension[@text='Assumptions']/x:category[@text='Equal variances assumed']/x:dimension[@text='Statistics']/x:group[@text='t-test for Equality of Means']/x:group[@text='" + str(percentvals) + "% Confidence Interval of the Difference']/x:category[@text='Lower']/x:cell/@number)"
lowereq_num = str2float(testout.xpath(path, namespaces=ns))
print(lowereq_num)

path = "string(//x:category/x:dimension[@text='Assumptions']/x:category[@text='Equal variances assumed']/x:dimension[@text='Statistics']/x:group[@text='t-test for Equality of Means']/x:group[@text='" + str(percentvals) + "% Confidence Interval of the Difference']/x:category[@text='Lower']/x:cell/@text)"
lowereq_str = testout.xpath(path, namespaces=ns)
print(lowereq_str)

path = "string(//x:category/x:dimension[@text='Assumptions']/x:category[@text='Equal variances not assumed']/x:dimension[@text='Statistics']/x:group[@text='t-test for Equality of Means']/x:group[@text='" + str(percentvals) + "% Confidence Interval of the Difference']/x:category[@text='Lower']/x:cell/@number)"
lowerneq_num = str2float(testout.xpath(path, namespaces=ns))
print(lowerneq_num)

path = "string(//x:category/x:dimension[@text='Assumptions']/x:category[@text='Equal variances not assumed']/x:dimension[@text='Statistics']/x:group[@text='t-test for Equality of Means']/x:group[@text='" + str(percentvals) + "% Confidence Interval of the Difference']/x:category[@text='Lower']/x:cell/@text)"
lowerneq_str = testout.xpath(path, namespaces=ns)
print(lowerneq_str)

path = "string(//x:category/x:dimension[@text='Assumptions']/x:category[@text='Equal variances assumed']/x:dimension[@text='Statistics']/x:group[@text='t-test for Equality of Means']/x:group[@text='" + str(percentvals) + "% Confidence Interval of the Difference']/x:category[@text='Upper']/x:cell/@number)"
uppereq_num = str2float(testout.xpath(path, namespaces=ns))
print(uppereq_num)

path = "string(//x:category/x:dimension[@text='Assumptions']/x:category[@text='Equal variances assumed']/x:dimension[@text='Statistics']/x:group[@text='t-test for Equality of Means']/x:group[@text='" + str(percentvals) + "% Confidence Interval of the Difference']/x:category[@text='Upper']/x:cell/@text)"
uppereq_str = testout.xpath(path, namespaces=ns)
print(uppereq_str)

path = "string(//x:category/x:dimension[@text='Assumptions']/x:category[@text='Equal variances not assumed']/x:dimension[@text='Statistics']/x:group[@text='t-test for Equality of Means']/x:group[@text='" + str(percentvals) + "% Confidence Interval of the Difference']/x:category[@text='Upper']/x:cell/@number)"
upperneq_num = str2float(testout.xpath(path, namespaces=ns))
print(upperneq_num)

path = "string(//x:category/x:dimension[@text='Assumptions']/x:category[@text='Equal variances not assumed']/x:dimension[@text='Statistics']/x:group[@text='t-test for Equality of Means']/x:group[@text='" + str(percentvals) + "% Confidence Interval of the Difference']/x:category[@text='Upper']/x:cell/@text)"
upperneq_str = testout.xpath(path, namespaces=ns)
print(upperneq_str)

shortname = testvar
longname = testvar
if varlab != '':
    longname = varlab + '[' + longname + ']'
varnameshort = '<strong>' + shortname + '</strong>'
varnamelong  = '<strong>' + longname + '</strong>'

groupnameshort = groupvar
groupnamelong = groupvar
if grouplab != '':
    groupnamelong = grouplab + '[' + groupnamelong + ']'
groupnameshort = '<strong>' + groupnameshort + '</strong>'
groupnamelong = '<strong>' + groupnamelong + '</strong>'

introhtml = '<h2>Independent Samples t test</h2>'
introhtml += '<p>'
introhtml += 'In this practical we are going to investigate how to perform an independent samples t-test. '
introhtml += 'This test is used when we have an interval or ratio level variable measured for two separate, or independent, groups of cases and we want to test if the mean of this dependent variable is different in the two groups. '
introhtml += 'The test assumes that the dependent variable is normally distributed in both groups. '
introhtml += 'To run a single test in SPSS requires that your dataset has one column containing the dependent variable to be tested and another column of the same length containing an indicator for which group the case belongs to. '
introhtml += '</p>'

outputs['introduction.html'] = HTMLOutput(introhtml, "Introduction text")

normalitytesthtml = '<p>'
normalitytesthtml += 'In this case the groups are identified by specific group codes although it is possible to also use a second continuous variable for groups and specify ranges of this variable that correspond to groups. '
normalitytesthtml += '</p>'

normalitytesthtml += '<p>'
normalitytesthtml += 'Here we are considering the variable, ' + varnameshort + '. '
normalitytesthtml += '</p>'

normalitytesthtml += '<p>'
normalitytesthtml += 'As a first step we should generally test for the normality of the variable ' + varnameshort + ' in each of the two groups that are indicated by ' + groupnameshort + '. '
normalitytesthtml += 'We first test for normality in each group. '
normalitytesthtml += '</p>'

instructhtml = '<ul class = "instructions">'
instructhtml += '<li>Select <strong>Descriptive Statistics</strong> from the <strong>Analyze</strong> menu.</li>'
instructhtml += '<li>Select <strong>Explore</strong> from the <strong>Descriptive Statistics</strong> sub-menu.</li>'
instructhtml += '<li>Click on the <strong>Reset</strong> button.</li>'
instructhtml += '<li>Copy the ' + varnamelong + ' variables into the <strong>Dependent List:</strong> box.</li>'
instructhtml += '<li>Copy the ' + groupnamelong + ' variable into the <strong>Factor List:</strong> box.</li>'
instructhtml += '<li>Click on the <strong>Plots...</strong> button.</li>'
instructhtml += '<li>On the screen that appears select the <strong>Histogram</strong> tick box.</li>'
instructhtml += '<li>Unselect the <strong>Stem and leaf</strong> button.</li>'
instructhtml += '<li>Select the <strong>Normality plots with tests</strong> button.</li>'
instructhtml += '<li>Click on the <strong>Continue</strong> button.</li>'
instructhtml += '<li>Click on the <strong>OK</strong> button.</li>'
instructhtml += '</ul>'

normalitytesthtml += instructhtml

normalitytesthtml += '<p>'
normalitytesthtml += 'We will first look at a histogram of the ' + varnameshort + ' variable, for the group defined by ' + groupnameshort + ' = <strong>' + str(group1lab) + '</strong>. '
normalitytesthtml += 'This can be found in amongst the set of output objects and looks as follows:'
normalitytesthtml += '</p>'

if int(group2) > int(group1):
    normalitytesthtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_000.png'].content)  + '">'
else:
    normalitytesthtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_1_000.png'].content)  + '">'

# FIXME: Currently this assumes only one variable is chosen
normalitytesthtml += '<p>'
normalitytesthtml += 'Ideally for a normal distribution this histogram should look symmetric around the mean of the distribution, in this case ' + mean1_str + '. '

if skews1_num > 2*skewses1_num:
    normalitytesthtml += 'This distribution appears to be positively skewed. '
else:
    if -skews1_num > 2*skewses1_num:
        normalitytesthtml += 'This distribution appears to be negatively skewed. '
    else:
        normalitytesthtml += 'This distribution appears to be reasonably symmetric. '
normalitytesthtml += '</p>'

normalitytesthtml += '<p>'
normalitytesthtml += 'Next we will look at a histogram of the ' + varnameshort + ' variable, for the group defined by ' + groupnameshort + ' = <strong>' + str(group2lab) + '</strong>. '
normalitytesthtml += 'This can be found in amongst the set of output objects and looks as follows:'
normalitytesthtml += '</p>'

if int(group2) > int(group1):
    normalitytesthtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_1_000.png'].content)  + '">'
else:
    normalitytesthtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_000.png'].content)  + '">'

normalitytesthtml += '<p>'
normalitytesthtml += 'Ideally for a normal distribution this histogram should look symmetric around the mean of the distribution, in this case ' + mean2_str + '. '
if skews2_num > 2*skewses2_num:
    normalitytesthtml += 'This distribution appears to be positively skewed. '
else:
    if -skews2_num > 2*skewses2_num:
        normalitytesthtml += 'This distribution appears to be negatively skewed. '
    else:
        normalitytesthtml += 'This distribution appears to be reasonably symmetric. '
normalitytesthtml += '</p>'

normalitytesthtml += '<p>'
normalitytesthtml += 'We will next look at a statistical test to see if this backs up our visual impressions from the histogram. '
normalitytesthtml += '</p>'

normalitytesthtml += '<p>'
normalitytesthtml += 'The Kolmogorov-Smirnov test is used to test the null hypothesis that a set of data comes from a Normal distribution. '
normalitytesthtml += 'The Kolmogorov Smirnov test produces test statistics that are used (along with a degrees of freedom parameter) to test for normality. '
normalitytesthtml += '</p>'

normalitytesthtml += outputs['Table_Tests of Normality.htm'].content

normalitytesthtml += '<p>'
normalitytesthtml += 'For the group ' + groupnameshort + ' = <strong>' + str(group1lab) + '</strong>: '
normalitytesthtml += 'This has degrees of freedom which equals the number of data points, namely ' + str(kmdfs1) + '. '
normalitytesthtml += '</p>'

normalitytesthtml += '<p>'
if kmsigs1_num > 0.05:
    normalitytesthtml += 'Here we see that the p value (quoted under Sig. for Kolmogorov Smirnov  is ' + kmsigs1_str + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the distribution is normal. '
else:
    if kmsigs1_num < 0.001:
        normalitytesthtml += 'The p-value provided by SPSS (quoted under Sig. for Kolmogorov-Smirnov) is ' + kmsigs1_str + ' (reported as p &lt; .001). '
    else:
        normalitytesthtml += 'Here we see that the p value (quoted under Sig. for Kolmogorov Smirnov) is ' + kmsigs1_str + ' which is less than 0.05. '
    normalitytesthtml += 'We therefore have significant evidence to reject the null hypothesis that the variable follows a normal distribution. '
normalitytesthtml += '</p>'

normalitytesthtml += '<p>'
normalitytesthtml += 'For the group ' + groupnameshort + ' = <strong>' + str(group2lab) + '</strong>: '
normalitytesthtml += 'This has degrees of freedom which equals the number of data points, namely ' + str(kmdfs2) + '. '
normalitytesthtml += '</p>'

normalitytesthtml += '<p>'
if kmsigs2_num > 0.05:
    normalitytesthtml += 'Here we see that the p value (quoted under Sig. for Kolmogorov Smirnov  is ' + kmsigs2_str + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the distribution is normal. '
else:
    if kmsigs2_num < 0.001:
        normalitytesthtml += 'The p-value provided by SPSS (quoted under Sig. for Kolmogorov-Smirnov) is ' + kmsigs2_str + ' (reported as p &lt; .001). '
    else:
        normalitytesthtml += 'Here we see that the p value (quoted under Sig. for Kolmogorov Smirnov) is ' + kmsigs2_str + ' which is less than 0.05. '
    normalitytesthtml += 'We therefore have significant evidence to reject the null hypothesis that the variable follows a normal distribution. '
normalitytesthtml += '</p>'

normalitytesthtml += '<p>'
normalitytesthtml += 'Although these statistics tell the researcher whether the distribution followed by a variable is statistically significantly different from a normal distribution one should take care in not overinterpreting such findings. '
normalitytesthtml += 'Significance will be strongly affected by the number of observations and so only a small discrepancy from normality will be deemed significant for very large sample sizes whilst very large discrepancies will be required to reject the null hypothesis for small sample sizes. '
normalitytesthtml += '</p>'

normalitytesthtml += '<p>'
normalitytesthtml += 'SPSS also supplies QQ plots to assist in looking at normality but for brevity we do not show them here. '
normalitytesthtml += '</p>'

outputs['normalitytests.html'] = HTMLOutput(normalitytesthtml, "Normality tests text")

normalitytestquesthtml = '<p>'
normalitytestquesthtml += 'The first step is generally to test for the normality of the variable ' + varnameshort + ' in each of the two groups that are indicated by ' + groupnameshort + ', so do this now. '
normalitytestquesthtml += '</p>'
normalitytestquesthtml += '<ul>'
normalitytestquesthtml += '<li>Question: Looking at the histogram and the normality tests does this variable look normal for ' + groupnameshort + ' = <strong>' + str(group1lab) + '?</strong></li>'
normalitytestquesthtml += '<li>Question: How about for ' + groupnameshort + ' = <strong>' + str(group2lab) + '</strong>?</li>'
normalitytestquesthtml += '</ul>'

outputs['normalitytestquestions.html'] = HTMLOutput(normalitytestquesthtml, "Normality tests questions")

normalitytestsolhtml = '<p>'
normalitytestsolhtml += 'To test the dependent variable for normality in each group: '
normalitytestsolhtml += '</p>'

normalitytestsolhtml += instructhtml
normalitytestsolhtml += '<p>'
normalitytestsolhtml += 'Question: Looking at the histogram and the normality tests does this variable look normal for ' + groupnameshort + ' = <strong>' + str(group1lab) + '</strong>?'
normalitytestsolhtml += '</p>'

normalitytestsolhtml += '<p>'
normalitytestsolhtml += 'Solution: The output from SPSS is as follows: '
normalitytestsolhtml += '</p>'

normalitytestsolhtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_000.png'].content)  + '">'

normalitytestsolhtml += '<p>'
normalitytestsolhtml += 'Ideally for a normal distribution this histogram should look symmetric around the mean of the distribution, in this case ' + mean1_str + '. '

if skews1_num > 2*skewses1_num:
    normalitytestsolhtml += 'This distribution appears to be positively skewed. '
else:
    if -skews1_num > 2*skewses1_num:
        normalitytestsolhtml += 'This distribution appears to be negatively skewed. '
    else:
        normalitytestsolhtml += 'This distribution appears to be reasonably symmetric. '
normalitytestsolhtml += '</p>'

normalitytestsolhtml += outputs['Table_Tests of Normality.htm'].content

normalitytestsolhtml += '<p>'
normalitytestsolhtml += 'The relevant Kolmogorov-Smirnov statistic from the table has degrees of freedom equal to the sample size for which ' + groupnameshort + ' = <strong>' + str(group1lab) + '</strong>, i.e. ' + str(kmdfs1) + '. '
if kmsigs1_num > 0.05:
    normalitytestsolhtml += 'The p-value provided by SPSS (quoted under Sig. for Kolmogorov Smirnov) is ' + kmsigs1_str + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the distribution is normal. '
else:
    if kmsigs1_num < 0.001:
        normalitytestsolhtml += 'The p-value provided by SPSS (quoted under Sig. for Kolmogorov-Smirnov) is ' + kmsigs1_str + ' (reported as p &lt; .001). '
    else:
        normalitytestsolhtml += 'Here we see that the p value (quoted under Sig. for Kolmogorov Smirnov) is ' + kmsigs1_str + ' which is less than 0.05. '
    normalitytestsolhtml += 'We therefore have significant evidence to reject the null hypothesis that the variable follows a normal distribution. '
    normalitytestsolhtml += 'As noted in the Practical, however, formal tests of normality must be considered in the light both of sample sizes and of whether the data seem &quot;sufficiently&quot; normal on the basis of visual inspection. '
normalitytestsolhtml += '</p>'

normalitytestsolhtml += '<p>'
normalitytestsolhtml += 'Question: How about for ' + groupnameshort + ' = <strong>' + str(group2lab) + '</strong>?'
normalitytestsolhtml += '</p>'
normalitytestsolhtml += '<p>'
normalitytestsolhtml += 'Solution: The output from SPSS is as follows: '
normalitytestsolhtml += '</p>'
normalitytestsolhtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_1_000.png'].content)  + '">'

normalitytestsolhtml += '<p>'
normalitytestsolhtml += 'Ideally for a normal distribution this histogram should look symmetric around the mean of the distribution, in this case ' + mean2_str + '. '
if skews2_num > 2*skewses2_num:
    normalitytestsolhtml += 'This distribution appears to be positively skewed. '
else:
    if -skews2_num > 2*skewses2_num:
        normalitytestsolhtml += 'This distribution appears to be negatively skewed. '
    else:
        normalitytestsolhtml += 'This distribution appears to be reasonably symmetric. '
normalitytestsolhtml += '</p>'

normalitytestsolhtml += outputs['Table_Tests of Normality.htm'].content

normalitytestsolhtml += '<p>'
normalitytestsolhtml += 'The relevant Kolmogorov-Smirnov statistic from the table has degrees of freedom equal to the sample size for which ' + groupnameshort + ' = <strong>' + str(group2lab) + '</strong>, i.e. ' + str(kmdfs2) + '. '
if kmsigs2_num > 0.05:
    normalitytestsolhtml += 'The p-value (quoted under Sig. for Kolmogorov Smirnov is ' + kmsigs2_str + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the distribution is normal. '
else:
    if kmsigs2_num < 0.001:
        normalitytestsolhtml += 'The p-value provided by SPSS (quoted under Sig. for Kolmogorov-Smirnov) is ' + kmsigs2_str + ' (reported as p &lt; .001). '
    else:
        normalitytestsolhtml += 'Here we see that the p value (quoted under Sig. for Kolmogorov Smirnov) is ' + kmsigs2_str + ' which is less than 0.05. '
    normalitytestsolhtml += 'We therefore have significant evidence to reject the null hypothesis that the variable follows a normal distribution, though with the caveats noted above. '
normalitytestsolhtml += '</p>'

outputs['normalitytestanswers.html'] = HTMLOutput(normalitytestsolhtml, "Normality tests questions")

errorbarhtml = '<p>'
errorbarhtml += 'If both groups can be assumed to follow normal distributions then a first descriptive step before performing the t test is to look at an error bar plot to see if the intervals of the two groups overlap. '
errorbarhtml += '</p>'

errorbarhtml += '<p>'
errorbarhtml += 'To do this do the following in SPSS: '
errorbarhtml += '</p>'

errorbarinstructhtml = '<ul class = "instructions">'
errorbarinstructhtml += '<li>Select <strong>Error Bar</strong> from the <strong>Legacy Dialogs</strong> submenu of the <strong>Graphs</strong> menu.</li>'
errorbarinstructhtml += '<li>Select <strong>Simple</strong> and <strong>Summaries for groups of cases</strong> as for the boxplot and click on the <strong>Define</strong> button.</li>'
errorbarinstructhtml += '<li>Transfer the ' + longname + ' variable to the <strong>Variable</strong> box.</li>'
errorbarinstructhtml += '<li>Transfer the ' + groupnamelong + ' variable to the <strong>Category Axis</strong> box.</li>'
errorbarinstructhtml += '<li>Click on the <strong>OK</strong> button.</li>'
errorbarinstructhtml += '</ul>'

errorbarhtml += errorbarinstructhtml

errorbarhtml += '<p>'
errorbarhtml += 'The output will look as follow: '
errorbarhtml += '</p>'

errorbarhtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_6_000.png'].content)  + '">'

overcats = []
overlaps = False

cilow1 = mean1_num - 1.96*semean1_num
cihigh1 = mean1_num + 1.96*semean1_num

cilow2 = mean2_num - 1.96*semean2_num
cihigh2 = mean2_num + 1.96*semean2_num

# NOTE: This is checking "Equal variances assumed, is this right?"
# Check if confidence intervals contain zero
if (cihigh1 > cihigh2 and cilow1 < cihigh2) or (cilow1 < cilow2 and cihigh1 > cilow2):
    overlaps = True

errorbarhtml += '<p>'
errorbarhtml += 'Here we see that the mean of ' + varnameshort + ' in the group ' + groupnameshort + ' = <strong>' + str(group1lab) + '</strong> is '
if mean1_num > mean2_num:
    errorbarhtml += 'higher '
else:
    errorbarhtml += 'lower '
errorbarhtml += ' than the mean for ' + groupnameshort + ' = <strong>' + str(group2lab) + '</strong>. '

if overlaps == False:
    errorbarhtml += 'The bars do not overlap so we will expect the t test to return a significant result. '

if overlaps == True:
    errorbarhtml += 'The bars overlap and so we are not sure whether there is a significant difference. '

errorbarhtml += '</p>'
outputs['errorbars.html'] = HTMLOutput(errorbarhtml, "Introduction text")

errorbarquesthtml = '<p>'
errorbarquesthtml += 'If both groups can be assumed to follow normal distributions then a first descriptive step before performing the test is to look at an error bar plot to see if the intervals of the two groups overlap, so do this now. '
errorbarquesthtml += '</p>'
errorbarquesthtml += '<ul>'
errorbarquesthtml += '<li>Question: Does the plot give evidence of a significant difference?</li>'
errorbarquesthtml += '</ul>'

outputs['errorbarsquestions.html'] = HTMLOutput(errorbarquesthtml, "Introduction text")

errorbarsolhtml = '<p>'
errorbarsolhtml += 'To plot the error bar chart in SPSS: '
errorbarsolhtml += '</p>'
errorbarsolhtml += errorbarinstructhtml
errorbarsolhtml += '<p>'
errorbarsolhtml += 'Question: Does the plot give evidence of a significant difference?'
errorbarsolhtml += '</p>'
errorbarsolhtml += '<p>'
errorbarsolhtml += 'Solution: The output from SPSS is as follows:'
errorbarsolhtml += '</p>'

errorbarsolhtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_6_000.png'].content)  + '">'

errorbarsolhtml += '<p>'
if overlaps == False:
    errorbarsolhtml += 'Here we see that the bars do not overlap so we will expect the t test to return a significant result. '

if overlaps == True:
    errorbarsolhtml += 'Here the bars overlap and so we are not sure whether there is a significant difference. '

errorbarsolhtml += '</p>'

outputs['errorbarsanswers.html'] = HTMLOutput(errorbarsolhtml, "Introduction text")

summstathtml = '<p>'
summstathtml += 'We will now do the t test itself. '
summstathtml += '</p>'

summstathtml += '<p>'
summstathtml += 'Below you will see instructions on how to perform the t test in SPSS. '
summstathtml += 'If you follow the instructions you will see the two tabular outputs that are embedded in the explanations below. '
summstathtml += '</p>'

summstatinstrhtml = '<ul class = "instructions">'
summstatinstrhtml += '<li>Select <strong>Compare Means</strong> from the <strong>Analyze</strong> menu.</li>'
summstatinstrhtml += '<li>Select <strong>Independent-Sample T Test...</strong> from the <strong>Compare Means</strong> sub-menu.</li>'
summstatinstrhtml += '<li>Click on the <strong>reset</strong> button.</li>'
summstatinstrhtml += '<li>Copy the ' + varnamelong + ' variable into the <strong>Test Variable(s):</strong> box.</li>'
summstatinstrhtml += '<li>Copy the ' + groupnamelong + ' variable into the <strong>Grouping Variable:</strong> box.</li>'
summstatinstrhtml += '<li>Click on the <strong>Define Groups...</strong> button.</li>'
summstatinstrhtml += '<li>Click on the <strong>Use specified values</strong> button.</li>'
summstatinstrhtml += '<li>Type <strong>' + str(group1) + '</strong> into the <strong>Group 1</strong> box.</li>'
summstatinstrhtml += '<li>Type <strong>' + str(group2) + '</strong> into the <strong>Group 2</strong> box.</li>'
summstatinstrhtml += '<li>Click on the <strong>Continue</strong> button.</li>'
if options:
    summstatinstrhtml += '<li>Click on the <strong>Options...</strong> button.</li>'
    summstatinstrhtml += '<li>Type <strong>' + str(percentvals) + '</strong> into the <strong>Confidence Interval Percentage</strong> box.</li>'
    if missingvals == 'Exclude cases analysis by analysis':
        summstatinstrhtml += '<li>Click on the <strong>Exclude cases analysis by analysis</strong> button.</li>'
    if missingvals == 'Exclude cases analysis by analysis':
        summstatinstrhtml += '<li>Click on the <strong>Exclude cases analysis by analysis</strong> button.</li>'
    if missingvals == 'Exclude cases listwise':
        summstatinstrhtml += '<li>Click on the <strong>Exclude cases listwise</strong> button.</li>'
    summstatinstrhtml += '<li>Click on the <strong>Continue</strong> button.</li>'
summstatinstrhtml += '<li>Click on the <strong>OK</strong> button.</li>'
summstatinstrhtml += '</ul>'

summstathtml += summstatinstrhtml

summstathtml += '<p>'
summstathtml += 'The first SPSS output table contains summary statistics for all the variables considered split by group and can be seen below:'
summstathtml += '</p>'

summstathtml += outputs['Table_Group Statistics.htm'].content

summstathtml += '<p>'
summstathtml += 'The summary statistics table contains 5 columns and 1 row for each group. '

summstathtml += 'After the first column which contains the name of each dependent variable and group categories we next see the number of valid observations in each group, i.e. cases with a valid value of ' + varnameshort + '. '
summstathtml += 'Here for the group indexed by ' + groupnameshort + '<strong> = ' + str(group1lab) + '</strong>, we have ' + str(N1) + ' observations and for ' + groupnameshort + '<strong> = ' + group2lab + '</strong>, there are ' + str(N2) + ' observations. ' 
summstathtml += 'Next we see that the mean of the variable ' + varnameshort + ' for the group with ' + groupnameshort + '<strong> = ' + group1lab + '</strong> is ' + mean1_str + ' whilst for the group with ' + groupnameshort + '<strong> = ' + group2lab + '</strong> it is ' + mean2_str + '. '
if mean1_num > mean2_num:
    summstathtml += 'Hence, as we saw from the error bar chart the group with ' + groupnameshort + '<strong> = ' + group1lab + '</strong> has the bigger mean and the test will now establish if this difference is statistically significant. '
if mean2_num > mean1_num:
    summstathtml += 'Hence, as we saw from the error bar chart the group with ' + groupnameshort + '<strong> = ' + group2lab + '</strong> has the bigger mean and the test will now establish if this difference is statistically significant. '
if mean1_num == mean2_num:
    summstathtml += 'Very unusually the means are equal.'
summstathtml += '</p>'

summstathtml += '<p>'
summstathtml += 'In the next column we see the standard deviations for ' + varnameshort + ' variable in the two groups. '
summstathtml += 'As we will see in the next table there are two versions of the independent samples t-test depending on whether the variability (and therefore the standard deviations) of the dependent variable in the two groups can be assumed equal or not. '
summstathtml += 'In this case the standard deviation of ' + varnameshort + ' when ' + groupnameshort + '<strong> = ' + group1lab + '</strong> is ' + sd1_str + ' whilst for ' + groupnameshort + '<strong> = ' + group2lab + '</strong> it is ' + sd2_str + '. '
if sd1_num > sd2_num:
    summstathtml += 'So there is slightly more variability among ' + groupnameshort + '<strong> = ' + group1lab + '</strong> than ' + groupnameshort + '<strong> = ' + group2lab + '</strong>. '
    summstathtml += 'But is the difference big enough to violate the assumption of equal variances? '
if sd2_num > sd1_num:
    summstathtml += 'So there is slightly more variability among ' + groupnameshort + '<strong> = ' + group2lab + '</strong> than ' + groupnameshort + '<strong> = ' + group1lab + '</strong>. '
    summstathtml += 'But is the difference big enough to violate the assumption of equal variances? '
if sd1_num == sd2_num:
    summstathtml += 'So the variability among ' + groupnameshort + '<strong> = ' + group1lab + '</strong> and ' + groupnameshort + '<strong> = ' + group2lab + '</strong> is the same. '
summstathtml += 'In the final column are the standard errors of the means for each group. '
summstathtml += 'Whilst the standard deviations measure the variability in the data the standard errors of the means measures how confident we are in the estimates of the means. '
summstathtml += 'As we collect more data the standard error of the mean gets smaller as we get more confident in the mean estimate and in fact the formula for the standard error of the mean = standard deviation / square root of N. '
summstathtml += 'In this case the standard error of the mean for ' + varnameshort + ' when ' + groupnameshort + '<strong> = ' + group1lab + '</strong> is ' + semean1_str + ' whilst for ' + groupnameshort + '<strong> = ' + group2lab + '</strong> it is ' + semean2_str + '. '

summstathtml += '</p>'
outputs['summarystatistics.html'] = HTMLOutput(summstathtml, "Summary statistics text")

summstatquesthtml = '<p>'
summstatquesthtml += 'Next conduct the independent samples t-test. '
summstatquesthtml += '</p>'
summstatquesthtml += '<ul>'
summstatquesthtml += '<li>Question: From the summary stats block which group has the larger mean?</li>'
summstatquesthtml += '<li>Question: Which group has the largest variability?</li>'
summstatquesthtml += '</ul>'

outputs['summarystatisticsquestions.html'] = HTMLOutput(summstatquesthtml, "Summary statistics question text")

summstatsolhtml = '<p>'
summstatsolhtml += 'Here are the instuctions to perform the t test in SPSS. '
summstatsolhtml += 'They will produce the two tabular outputs embedded in the answers below. '
summstatsolhtml += '<p>'
summstatsolhtml += summstatinstrhtml
summstatsolhtml += '<ul>'
summstatsolhtml += '<li>Question: From the summary stats block which group has the larger mean?</li>'
summstatsolhtml += '<li>Question: Which group has the largest variability?</li>'
summstatsolhtml += '</ul>'

summstatsolhtml += '<p>'
summstatsolhtml += 'Solutions: The output from SPSS is as follows:'
summstatsolhtml += outputs['Table_Group Statistics.htm'].content
summstatsolhtml += '</p>'
summstatsolhtml += '<p>'
summstatsolhtml += 'The answers are as follows:'
summstatsolhtml += '<ul>'
if mean1_num > mean2_num:
    summstatsolhtml += '<li>The group ' + groupnameshort + '<strong> = ' + group1lab + '</strong> has the larger mean.</li>'
if mean2_num > mean1_num:
    summstatsolhtml += '<li>The group ' + groupnameshort + '<strong> = ' + group2lab + '</strong> has the larger mean.</li>'
if mean1_num == mean2_num:
    summstatsolhtml += '<li>Very unusually the means are equal.</li>'
if sd1_num > sd2_num:
    summstatsolhtml += '<li>The group ' + groupnameshort + '<strong> = ' + group1lab + '</strong> has the larger variability.</li>'
if sd2_num > sd1_num:
    summstatsolhtml += '<li>The group ' + groupnameshort + '<strong> = ' + group2lab + '</strong> has the larger variability.</li>'
if sd1_num == sd2_num:
    summstatsolhtml += '<li>They are the same</li>'
summstatsolhtml += '</ul>'
summstatsolhtml += '</p>'

outputs['summarystatisticsanswers.html'] = HTMLOutput(summstatsolhtml, "Summary statistics answer text")

testhtml = '<p>'
testhtml += 'The second SPSS output table contains details of the test itself and can be seen below:'
testhtml += '</p>'

testhtml += outputs['Table_Independent Samples Test.htm'].content

testhtml += '<p>'
testhtml += 'The above table in fact captures two tests.'
testhtml += '</p>'

testhtml += '<p>'
testhtml += 'The first test is known as Levene&apos;s test and tests the assumption that the variability in the two groups is equal and if this is not the case then a slightly different t-test is performed. '
testhtml += 'The two rows of numbers in the right of the table correspond to the two different versions of the test. '
testhtml += 'We decide which one to report depending on the p-value (labelled Sig in SPSS) of Levene&apos;s test. '
testhtml += 'The null hypothesis of Levene&apos;s test is that the variances (and SDs) in the two groups are equal in the population. '
testhtml += 'When this is true, the Levene&apos;s test statistic follows a standard statistical distribution called an F distribution. '
testhtml += 'Higher values of the F-statistic are associated with a lower likelihood that the sample did indeed come from a population in which the null hypothesis is true. '
if levenef_num < 0.001:
    testhtml += 'In this case the F statistic has value ' + levenef_str + ' and SPSS calculates the corresponding p value for this statistic which is ' + levenesig_str + ' (reported as p &lt; .001). '
else:
    testhtml += 'In this case the F statistic has value ' + levenef_str + ' and SPSS calculates the corresponding p value for this statistic which is ' + levenesig_str + '. '
if levenesig_num > 0.05:
    testhtml += 'This p value being greater than 0.05 does not give us strong enough evidence to reject the hypothesis of equal variances so we use the top row of numbers to the right of the table. '
    testhtml += '</p>'

    testhtml += '<p>'
    testhtml += 'So if we now look at the top row of numbers we will start with the column headed Mean difference. '
    testhtml += 'Here we see the value ' + meandiffeq_str +'. '
    testhtml += 'If you look back at the summary statistics table this value is calculated by subtracting one mean from the other. '
    testhtml += 'Next to the mean difference is the standard error of the difference. '
    testhtml += 'This here has the value ' + seeq_str + ' and is calculated via a formula from the standard errors of each group and their respective sample sizes. '
    testhtml += 'Working back to the start of the row, the column entitled t is the statistic used in the t test and t like F is a standard statistical distribution. '
    testhtml += 'The t statistic is calculated simply by dividing the mean difference by its standard error so ' + meandiffeq_str + ' / ' + seeq_str + ' = ' + teq_str + '. '
    testhtml += 'Next to t is a column labelled df which stands for degrees of freedom and is a parameter used to choose the correct t distribution for the statistic. '
    testhtml += 'When we can assume equal variances then the degrees of freedom equal the number of observations - 2 (' + dfeq_str + ') as we have used 2 degrees of freedom in estimating 2 means. '
    testhtml += '</p>'

    testhtml += '<p>'
    testhtml += 'The column labelled &quot;Sig (2-tailed)&quot; contains a test of the null hypothesis that the means of the ' + varnameshort + ' variable in the two groups are the same. '
    testhtml += 'By default, the two-tailed test reported uses a non-directional alternative hypothesis. '
    testhtml += 'It gives the probability that the data in the sample came from a population in which the group means are truly equal, when either a positive or a negative difference between sample group means is evidence against that null hypothesis. '
    testhtml += 'To conduct a one-tailed test, in which the alternative hypothesis specifies a particular direction to the difference, we would simply halve the p-value provided by SPSS.'
    testhtml += '</p>'

    testhtml += '<p>'
    testhtml += 'We can reject the null if there is sufficient evidence that the mean of Group 1 is either higher or lower than the mean of Group 2. '
    testhtml += 'Looking up the t statistic in the appropriate table gives the associated p value, in this case ' + sigeq_str + '. '
    if sigeq_num > 0.05:
        testhtml += 'Here we see that the p value is greater than 0.05 and therefore we cannot reject the null hypothesis that the two groups have the same means. '
        testhtml += 'Finally we can see the ' + str(percentvals) + '% confidence interval for the difference which runs from ' + lowereq_str + ' to ' + uppereq_str + '. '
        testhtml += 'Here we see it contains the value 0 backing up our failure to reject the null hypothesis. '
    else:
        testhtml += 'Here we see that the p value is less than 0.05 and therefore we can reject the null hypothesis that the two groups have the same means. '
        testhtml += 'Finally we can see the ' + str(percentvals) + '% confidence interval for the difference which runs from ' + lowereq_str + ' to ' + uppereq_str + '. '
        testhtml += 'Here we see it does not contain the value 0 backing up our rejection of the null hypothesis. '
else:
    if levenesig_num < 0.001:
         testhtml += 'The corresponding p-value for this statistic which is ' + levenesig_str + ' (reported as p &lt; .001) '
    testhtml += 'This p value being less than 0.05 gives us enough evidence to reject the null hypothesis and to assume that the two groups have different variances. '
    testhtml += 'This leads us to using the bottom row of numbers to the right in the table. '
    testhtml += '</p>'

    testhtml += '<p>'
    testhtml += 'So if we now look at the bottom row of numbers we will start with the column headed Mean Difference. '
    testhtml += 'Here we see the value ' + meandiffneq_str +'. '
    testhtml += 'If you look back at the summary statistics table this value is calculated by subtracting one mean from the other. '
    if meandiffneq_num < 0:
        testhtml += 'Here it is negative because earlier we defined Group 1 as corresponding to the group with ' + groupnameshort + '<strong> =' + group1lab + '</strong> (with a lower mean) and Group 2 as corresponding to the group with ' + groupnameshort + '<strong> =' + str(group2lab) + '</strong> (with a higher mean). '
    if meandiffneq_num > 0:
        testhtml += 'Here it is positive because earlier we defined Group 1 as corresponding to the group with ' + groupnameshort + '<strong> =' + group1lab + '</strong> (with a higher mean) and Group 2 as corresponding to the group with ' + groupnameshort + '<strong> =' + str(group2lab) + '</strong> (with a lower mean). '
    testhtml += 'The SPSS t-test calculates the mean differences as [Mean of Group 1] - [Mean of Group 2]. '
    if meandiffneq_num < 0:
        testhtml += 'If we reversed the definition of the groups in the initial dialog box, in this case the mean difference would have been positive ' + str(-float(meandiffneq_str)) + '. '
    if meandiffneq_num > 0:
        testhtml += 'If we reversed the definition of the groups in the initial dialog box, in this case the mean difference would have been negative ' + str(-float(meandiffneq_str)) + '. '
    testhtml += 'Next to the mean difference is the standard error of the difference. '
    testhtml += 'This here has the value ' + seneq_str + ' and is calculated via a formula from the standard errors of the means of each group and their respective sample sizes. '
    testhtml += 'You will notice that this value is slightly different from the value in the row above for results with Equal variances assumed, which results from the adjustment made for unequal variances. '
    testhtml += 'Working back to the start of the row, the column entitled t is the statistic used in the t test and t like F is a standard statistical distribution. '
    testhtml += 'The t statistic is calculated simply by dividing the mean difference by its standard error so ' + meandiffneq_str + ' / ' + seneq_str + ' = ' + tneq_str + '. '
    testhtml += 'Next to t is a column labelled df which stands for degrees of freedom and is a parameter used to choose the correct t distribution for the sampling distribution of the statistic. '
    testhtml += 'When we can assume equal variances then the degrees of freedom equal two less than the number of observations (N - 2, here ' + dfeq_str + ') as we have used 2 degrees of freedom in estimating 2 means. '
    testhtml += 'If we cannot assume equal variances then the value is lower as seen here (' + dfneq_str + '). '
    testhtml += '</p>'

    testhtml += '<p>'
    testhtml += 'The column labelled &quot;Sig (2-tailed)&quot; contains a test of the null hypothesis that the means of the ' + varnameshort + ' variable in the two groups are the same. '
    testhtml += 'By default, the two-tailed test reported uses a non-directional alternative hypothesis. '
    testhtml += 'It gives the probability that the data in the sample came from a population in which the group means are truly equal, when either a positive or a negative difference between sample group means is evidence against that null hypothesis. '
    testhtml += 'To conduct a one-tailed test, in which the alternative hypothesis specifies a particular direction to the difference, we would simply halve the p-value provided by SPSS.'
    testhtml += '</p>'

    testhtml += '<p>'
    testhtml += 'We can reject the null if there is sufficient evidence that the mean of Group 1 is either higher or lower than the mean of Group 2. '
    testhtml += 'SPSS looks up the t statistic in the appropriate table gives the associated p value associated with the calculated t-statistic and degrees of freedom. '
    if signeq_num < 0.001:
        testhtml += 'In this case SPSS quotes it as p = ' + signeq_str + ' (reported as p &lt; 0.001). '
    else:
        testhtml += 'In this case SPSS reports it as p = ' + signeq_str + '. '
    if signeq_num == 0:
        testhtml += 'As the p-value is never exactly zero there is always a chance, however small, that the null hypothesis is true, so here we should report p&lt;.0.01. '
    if signeq_num > 0.05:
        testhtml += 'Here we see that the p value is greater than 0.05 and therefore we cannot reject the null hypothesis that the two groups have the same means. '
        testhtml += 'Finally we can see the ' + str(percentvals) + '% confidence interval for the difference which runs from ' + lowerneq_str + ' to ' + upperneq_str + '. '
        testhtml += 'Here we see it contains the value 0 backing up our failure to reject the null hypothesis. '
    else:
        testhtml += 'Here we see that the p value is less than 0.05 and therefore we can reject the null hypothesis that the two groups have the same means. '
        testhtml += 'Finally we can see the ' + str(percentvals) + '% confidence interval for the difference which runs from ' + lowerneq_str + ' to ' + upperneq_str + '. '
        testhtml += 'Another way to decide whether we can reject the null hypothesis is to check whether this interval contains the value 0. '
        testhtml += 'Here we see it does not, a finding that would lead us to reject the null hypothesis. '

testhtml += '</p>'

outputs['testdetails.html'] = HTMLOutput(testhtml, "Test details text")

testquesthtml = '<ul>'
testquesthtml += '<li>Question: Using Levene&apos;s test can we assume equal variances?</li>'
testquesthtml += '<li>Question: Is there a significant difference between the groups?</li>'
testquesthtml += '</ul>'
outputs['testdetailsquestions.html'] = HTMLOutput(testquesthtml, "Test details question text")

testsolhtml = '<ul>'
testsolhtml += '<li>Question: Using Levene&apos;s test can we assume equal variances?</li>'
testsolhtml += '<li>Question: Is there a significant difference between the groups?</li>'
testsolhtml += '</ul>'
testsolhtml += '<p>'
testsolhtml += 'Solutions: The output from SPSS is as follows:'
testsolhtml += '</p>'
testsolhtml += outputs['Table_Independent Samples Test.htm'].content
testsolhtml += '<ul>'
if levenesig_num < 0.05:
    testsolhtml += '<li>No, we cannot assume equal variances.</li>'
else:
    testsolhtml += '<li>Yes, we can assume equal variances</li>'
if sigeq_num > 0.05:
    testsolhtml += '<li>No, there are no significant differences between the two groups.</li>'
else:
    testsolhtml += '<li>Yes, there are significant differences between the two groups.</li>'
testsolhtml += '</ul>'

outputs['testdetailsanswers.html'] = HTMLOutput(testsolhtml, "Test details answer text")

conchtml = '<p>'
conchtml += 'In conclusion, we could report this to a reader as follows: '

conchtml += 'Mean test scores were higher among '
if mean1_num > mean2_num:
    conchtml += groupnameshort + '<strong> = ' + group1lab + '</strong> (N=' + str(N1) + ', M=' + mean1_str + ', SD=' + sd1_str + ') than ' + groupnameshort + '<strong> = ' + group2lab + '</strong> (N=' + str(N2) + ', M=' + mean2_str + ', SD=' + sd2_str + '). '
if mean2_num > mean1_num:
    conchtml += groupnameshort + '<strong> = ' + group2lab + '</strong> (N=' + str(N2) + ', M=' + mean2_str + ', SD=' + sd2_str + ') than ' + groupnameshort + '<strong> = ' + group1lab + '</strong> (N=' + str(N1) + ', M=' + mean1_str + ', SD=' + sd1_str + '). '
if levenesig_num > 0.05:
    conchtml += 'Levene&apos;s test was not able to reject the null hypothesis of equal variances between the two groups (F=' + levenef_str + ', p=' + levenesig_str + ') so an unadjusted version of the independent samples t-test was chosen. '
    if sigeq_num > 0.05:
        conchtml += 'The difference in means (difference = ' + meandiffeq_str + ') was not statistically significant, t(' + dfeq_str + ') = ' + teq_str + ', p=' + sigeq_str + '. '
    else:
        conchtml += 'The difference in means (difference = ' + meandiffeq_str + ') was statistically significant, t(' + dfeq_str + ') = ' + teq_str + ', '
        if sigeq_num == 0:
            conchtml += 'p&lt;.001.'
        else:
            conchtml += 'p=' + sigeq_str + '. '
else:
    if levenesig_str < 0.001:
        conchtml += 'Levene&apos;s test rejected the null hypothesis of equal variances between the two groups (F=' + levenef_str + ', p=' + levenesig_str + ') so an adjusted version of the independent samples t-test that relaxes this assumption was chosen. '
    else:
        conchtml += 'Levene&apos;s test rejected the null hypothesis of equal variances between the two groups (F=' + levenef_str + ', p &lt; .001) so an adjusted version of the independent samples t-test that relaxes this assumption was chosen. '
    if signeq_num > 0.05:
        conchtml += 'The difference in means (difference = ' + meandiffneq_str + ') was not statistically significant, t(' + dfneq_str + ') = ' + tneq_str + ', p=' + sigeq_str + '. '
    else:
        conchtml += 'The difference in means (difference = ' + meandiffneq_str + ') was statistically significant, t(' + dfneq_str + ') = ' + tneq_str + ', '
        if signeq_num < 0.001:
            conchtml += 'p&lt;.001.'
        else:
            conchtml += 'p=' + signeq_str + '. '

conchtml += '</p>'

outputs['conclusion.html'] = HTMLOutput(conchtml, "Conclusion text")

concquesthtml = '<ul>'
concquesthtml += '<li>Question: How would you write up these findings?</li>'
concquesthtml += '</ul>'
outputs['conclusionquestions.html'] = HTMLOutput(concquesthtml, "Conclusion question text")

concsolhtml = concquesthtml
concsolhtml += '<p>Solution: </p>'
concsolhtml += conchtml
outputs['conclusionsolutions.html'] = HTMLOutput(concsolhtml, "Conclusion solutions text")


combinedprachtml = outputs['introduction.html'].content
combinedprachtml += outputs['normalitytests.html'].content
combinedprachtml += outputs['errorbars.html'].content
combinedprachtml += outputs['summarystatistics.html'].content
combinedprachtml += outputs['testdetails.html'].content
combinedprachtml += outputs['conclusion.html'].content

outputs['allpractical.html'] = HTMLOutput(combinedprachtml, "Combined practical HTML outputs")

combinedquesthtml = outputs['introduction.html'].content
combinedquesthtml += outputs['normalitytestquestions.html'].content
combinedquesthtml += outputs['errorbarsquestions.html'].content
combinedquesthtml += outputs['summarystatisticsquestions.html'].content
combinedquesthtml += outputs['testdetailsquestions.html'].content
combinedquesthtml += outputs['conclusionquestions.html'].content

outputs['allquestions.html'] = HTMLOutput(combinedquesthtml, "Combined questions HTML outputs")

combinedsolhtml = outputs['normalitytestanswers.html'].content
combinedsolhtml += outputs['errorbarsanswers.html'].content
combinedsolhtml += outputs['summarystatisticsanswers.html'].content
combinedsolhtml += outputs['testdetailsanswers.html'].content
combinedsolhtml += outputs['conclusionsolutions.html'].content

outputs['allanswers.html'] = HTMLOutput(combinedsolhtml, "Combined solutions HTML outputs")

'''