# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class Regression2(Template):
    'Performs non-parametric tests in SPSS (equivalent to Analyze->Nonparametric Tests->Legacy Dialogs->2 Independent Samples... in SPSS 23)'

    __version__ = '1.0.0'

    tags = [ 'SPSS']
    engines = ['SPSS_script']

    inputs = '''
var = DataVector('Variable: ')
group = DataVector('Grouping variable (two values): ')
val1 = Integer('Code for first group: ')
val2 = Integer('Code for second group: ')
exactopts = True
method = 'Exact'
options = False
'''

    spssscript = '''
EXAMINE VARIABLES=${var} BY ${group}
  /PLOT HISTOGRAM NPPLOT
  /COMPARE GROUPS
  /NOTOTAL.

NPAR TESTS
  /M-W=${var} BY ${group}(${val1}, ${val2})
  /METHOD= EXACT.
'''



    processoutputs = '''
import EStat
from EStat.Templating import *

import base64

from lxml import etree
from io import BytesIO

ns = {'x': 'http://www.ibm.com/software/analytics/spss/xml/oms'}

def str2float(s):
    try:
        return float(s)
    except ValueError:
        return float('nan')

descsout = etree.parse(BytesIO(outputs['Table_Descriptives.xml'].content))
print(etree.tostring(descsout, pretty_print=True))

path = "string(//x:dimension[@text='Dependent Variables']/x:category/@label)"
varlab = descsout.xpath(path, namespaces=ns)
print(varlab)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
mean1_num = str2float(descsout.xpath(path, namespaces=ns))
print(mean1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
mean1_str = descsout.xpath(path, namespaces=ns)
print(mean1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@number)"
ses1_num = str2float(descsout.xpath(path, namespaces=ns))
print(ses1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@text)"
ses1_str = descsout.xpath(path, namespaces=ns)
print(ses1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval for Mean']/x:category[@text='Lower Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
lci1_num = str2float(descsout.xpath(path, namespaces=ns))
print(lci1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval for Mean']/x:category[@text='Lower Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
lci1_str = descsout.xpath(path, namespaces=ns)
print(lci1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval for Mean']/x:category[@text='Upper Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
uci1_num = str2float(descsout.xpath(path, namespaces=ns))
print(uci1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval for Mean']/x:category[@text='Upper Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
uci1_str = descsout.xpath(path, namespaces=ns)
print(uci1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Statistics']/x:category[@text='5% Trimmed Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
tms1_num = str2float(descsout.xpath(path, namespaces=ns))
print(tms1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Statistics']/x:category[@text='5% Trimmed Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
tms1_str = descsout.xpath(path, namespaces=ns)
print(tms1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Statistics']/x:category[@text='Median']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
meds1_num = str2float(descsout.xpath(path, namespaces=ns))
print(meds1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Statistics']/x:category[@text='Median']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
meds1_str = descsout.xpath(path, namespaces=ns)
print(meds1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Statistics']/x:category[@text='Variance']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
varis1_num = str2float(descsout.xpath(path, namespaces=ns))
print(varis1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Statistics']/x:category[@text='Variance']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
varis1_str = descsout.xpath(path, namespaces=ns)
print(varis1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Statistics']/x:category[@text='Std. Deviation']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
sds1_num = str2float(descsout.xpath(path, namespaces=ns))
print(sds1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Statistics']/x:category[@text='Std. Deviation']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
sds1_str = descsout.xpath(path, namespaces=ns)
print(sds1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Statistics']/x:category[@text='Minimum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
mins1_num = str2float(descsout.xpath(path, namespaces=ns))
print(mins1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Statistics']/x:category[@text='Minimum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
mins1_str = descsout.xpath(path, namespaces=ns)
print(mins1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Statistics']/x:category[@text='Maximum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
maxs1_num = str2float(descsout.xpath(path, namespaces=ns))
print(maxs1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Statistics']/x:category[@text='Maximum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
maxs1_str = descsout.xpath(path, namespaces=ns)
print(maxs1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Statistics']/x:category[@text='Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
ranges1_num = str2float(descsout.xpath(path, namespaces=ns))
print(ranges1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Statistics']/x:category[@text='Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
ranges1_str = descsout.xpath(path, namespaces=ns)
print(ranges1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Statistics']/x:category[@text='Interquartile Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
iqrs1_num = str2float(descsout.xpath(path, namespaces=ns))
print(iqrs1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Statistics']/x:category[@text='Interquartile Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
iqrs1_str = descsout.xpath(path, namespaces=ns)
print(iqrs1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
skews1_num = str2float(descsout.xpath(path, namespaces=ns))
print(skews1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
skews1_str = descsout.xpath(path, namespaces=ns)
print(skews1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@number)"
skewses1_num = str2float(descsout.xpath(path, namespaces=ns))
print(skewses1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@text)"
skewses1_str = descsout.xpath(path, namespaces=ns)
print(skewses1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
kurts1_num = str2float(descsout.xpath(path, namespaces=ns))
print(kurts1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
kurts1_str = descsout.xpath(path, namespaces=ns)
print(kurts1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@number)"
kurtses1_num = str2float(descsout.xpath(path, namespaces=ns))
print(kurtses1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@text)"
kurtses1_str = descsout.xpath(path, namespaces=ns)
print(kurtses1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
mean2_num = str2float(descsout.xpath(path, namespaces=ns))
print(mean2_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
mean2_str = descsout.xpath(path, namespaces=ns)
print(mean2_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@number)"
ses2_num = str2float(descsout.xpath(path, namespaces=ns))
print(ses2_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@text)"
ses2_str = descsout.xpath(path, namespaces=ns)
print(ses2_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval for Mean']/x:category[@text='Lower Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
lci2_num = str2float(descsout.xpath(path, namespaces=ns))
print(lci2_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval for Mean']/x:category[@text='Lower Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
lci2_str = descsout.xpath(path, namespaces=ns)
print(lci2_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval for Mean']/x:category[@text='Upper Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
uci2_num = str2float(descsout.xpath(path, namespaces=ns))
print(uci2_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval for Mean']/x:category[@text='Upper Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
uci2_str = descsout.xpath(path, namespaces=ns)
print(uci2_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Statistics']/x:category[@text='5% Trimmed Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
tms2_num = str2float(descsout.xpath(path, namespaces=ns))
print(tms2_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Statistics']/x:category[@text='5% Trimmed Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
tms2_str = descsout.xpath(path, namespaces=ns)
print(tms2_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Statistics']/x:category[@text='Median']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
meds1_num = str2float(descsout.xpath(path, namespaces=ns))
print(meds1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Statistics']/x:category[@text='Median']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
meds2_str = descsout.xpath(path, namespaces=ns)
print(meds2_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Statistics']/x:category[@text='Variance']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
varis2_num = str2float(descsout.xpath(path, namespaces=ns))
print(varis2_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Statistics']/x:category[@text='Variance']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
varis2_str = descsout.xpath(path, namespaces=ns)
print(varis2_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Statistics']/x:category[@text='Std. Deviation']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
sds2_num = str2float(descsout.xpath(path, namespaces=ns))
print(sds2_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Statistics']/x:category[@text='Std. Deviation']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
sds2_str = descsout.xpath(path, namespaces=ns)
print(sds2_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Statistics']/x:category[@text='Minimum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
mins2_num = str2float(descsout.xpath(path, namespaces=ns))
print(mins2_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Statistics']/x:category[@text='Minimum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
mins2_str = descsout.xpath(path, namespaces=ns)
print(mins2_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Statistics']/x:category[@text='Maximum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
maxs2_num = str2float(descsout.xpath(path, namespaces=ns))
print(maxs2_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Statistics']/x:category[@text='Maximum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
maxs2_str = descsout.xpath(path, namespaces=ns)
print(maxs2_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Statistics']/x:category[@text='Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
ranges2_num = str2float(descsout.xpath(path, namespaces=ns))
print(ranges2_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Statistics']/x:category[@text='Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
ranges1_str = descsout.xpath(path, namespaces=ns)
print(ranges1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Statistics']/x:category[@text='Interquartile Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
iqrs1_num = str2float(descsout.xpath(path, namespaces=ns))
print(iqrs1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Statistics']/x:category[@text='Interquartile Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
iqrs2_str = descsout.xpath(path, namespaces=ns)
print(iqrs2_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
skews2_num = str2float(descsout.xpath(path, namespaces=ns))
print(skews2_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
skews2_str = descsout.xpath(path, namespaces=ns)
print(skews2_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@number)"
skewses2_num = str2float(descsout.xpath(path, namespaces=ns))
print(skewses2_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@text)"
skewses2_str = descsout.xpath(path, namespaces=ns)
print(skewses2_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
kurts2_num = str2float(descsout.xpath(path, namespaces=ns))
print(kurts2_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
kurts2_str = descsout.xpath(path, namespaces=ns)
print(kurts2_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@number)"
kurtses2_num = str2float(descsout.xpath(path, namespaces=ns))
print(kurtses2_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@text)"
kurtses2_str = descsout.xpath(path, namespaces=ns)
print(kurtses2_str)

statsout = etree.parse(BytesIO(outputs['Table_Tests of Normality.xml'].content))
print(etree.tostring(statsout, pretty_print=True))

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='Statistic']/x:cell/@number)"
kmstats1_num = str2float(statsout.xpath(path, namespaces=ns))
print(kmstats1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='Statistic']/x:cell/@text)"
kmstats1_str = statsout.xpath(path, namespaces=ns)
print(kmstats1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='df']/x:cell/@number)"
kmdfs1 = int(statsout.xpath(path, namespaces=ns))
print(kmdfs1)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@number)"
kmsigs1_num = str2float(statsout.xpath(path, namespaces=ns))
print(kmsigs1_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@text)"
kmsigs1_str = statsout.xpath(path, namespaces=ns)
print(kmsigs1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='Statistic']/x:cell/@text)"
swstats1_str = statsout.xpath(path, namespaces=ns)
print(swstats1_str)

if swstats1_str != '': # For non-weighted data Shapiro-Wilk is only calculated for a sample size less than 5000 (see https://www.ibm.com/support/knowledgecenter/en/SSLVMB_24.0.0/spss/base/syn_examine_plot.html)
    path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='Statistic']/x:cell/@number)"
    swstats1_num = str2float(statsout.xpath(path, namespaces=ns))
    print(swstats1_num)

    path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='df']/x:cell/@text)"
    swdfs1 = int(statsout.xpath(path, namespaces=ns))
    print(swdfs1)

    path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@number)"
    swsigs1_num = str2float(statsout.xpath(path, namespaces=ns))
    print(swsigs1_num)

    path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[1]/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@text)"
    swsigs1_str = statsout.xpath(path, namespaces=ns)
    print(swsigs1_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='Statistic']/x:cell/@number)"
kmstats2_num = str2float(statsout.xpath(path, namespaces=ns))
print(kmstats2_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='Statistic']/x:cell/@text)"
kmstats2_str = statsout.xpath(path, namespaces=ns)
print(kmstats2_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='df']/x:cell/@number)"
kmdfs2 = int(statsout.xpath(path, namespaces=ns))
print(kmdfs2)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@number)"
kmsigs2_num = str2float(statsout.xpath(path, namespaces=ns))
print(kmsigs2_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@text)"
kmsigs2_str = statsout.xpath(path, namespaces=ns)
print(kmsigs2_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='Statistic']/x:cell/@text)"
swstats2_str = statsout.xpath(path, namespaces=ns)
print(swstats2_str)

if swstats2_str != '': # For non-weighted data Shapiro-Wilk is only calculated for a sample size less than 5000 (see https://www.ibm.com/support/knowledgecenter/en/SSLVMB_24.0.0/spss/base/syn_examine_plot.html)
    path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='Statistic']/x:cell/@number)"
    swstats2_num = str2float(statsout.xpath(path, namespaces=ns))
    print(swstats2_num)

    path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='df']/x:cell/@text)"
    swdfs2 = int(statsout.xpath(path, namespaces=ns))
    print(swdfs2)

    path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@number)"
    swsigs2_num = str2float(statsout.xpath(path, namespaces=ns))
    print(swsigs2_num)

    path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category[2]/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@text)"
    swsigs2_str = statsout.xpath(path, namespaces=ns)
    print(swsigs2_str)

ranksout = etree.parse(BytesIO(outputs['Table_Mann Whitney Ranks.xml'].content))
print(etree.tostring(ranksout, pretty_print=True))

statsout = etree.parse(BytesIO(outputs['Table_Mann Whitney Test Statistics.xml'].content))
print(etree.tostring(statsout, pretty_print=True))

path = "string(//x:dimension[@axis='row']/x:category[@varName='" + str(var) + "']/@text)"
varlab = ranksout.xpath(path, namespaces=ns)
print(varlab)

path = "string(//x:dimension[@axis='row']/x:category/x:dimension/@text)"
grouplab = ranksout.xpath(path, namespaces=ns)
print(grouplab)

path = "//x:dimension[@axis='row']/x:category[@varName='" + var + "']/x:dimension[@varName='" + str(group) + "']/x:category[@varName='" + str(group) + "']/@text"
cats = ranksout.xpath(path, namespaces=ns)
print(cats)

path = "//x:dimension[@axis='row']/x:category[@varName='" + var + "']/x:dimension[@varName='" + str(group) + "']/x:category[@varName='" + str(group) + "']/x:dimension/x:category[@text='N']/x:cell/@number"
N = [int(x) for x in ranksout.xpath(path, namespaces=ns)]
print(N)

path = "//x:dimension[@axis='row']/x:category[@varName='" + var + "']/x:dimension[@varName='" + str(group) + "']/x:category[@varName='" + str(group) + "']/x:dimension/x:category[@text='Mean Rank']/x:cell/@number"
means_num = [str2float(x) for x in ranksout.xpath(path, namespaces=ns)]
print(means_num)

path = "//x:dimension[@axis='row']/x:category[@varName='" + var + "']/x:dimension[@varName='" + str(group) + "']/x:category[@varName='" + str(group) + "']/x:dimension/x:category[@text='Mean Rank']/x:cell/@text"
means_str = ranksout.xpath(path, namespaces=ns)
print(means_str)

path = "//x:dimension[@axis='row']/x:category[@varName='" + var + "']/x:dimension[@varName='" + str(group) + "']/x:category[@varName='" + str(group) + "']/x:dimension/x:category[@text='Sum of Ranks']/x:cell/@number"
sums_num = [str2float(x) for x in ranksout.xpath(path, namespaces=ns)]
print(sums_num)

path = "//x:dimension[@axis='row']/x:category[@varName='" + var + "']/x:dimension[@varName='" + str(group) + "']/x:category[@varName='" + str(group) + "']/x:dimension/x:category[@text='Sum of Ranks']/x:cell/@text"
sums_str = ranksout.xpath(path, namespaces=ns)
print(sums_str)

path = "//x:dimension[@axis='row']/x:category[@varName='" + var + "']/x:dimension[@varName='" + str(group) + "']/x:category[@text='Total']/x:dimension/x:category[@text='N']/x:cell/@number"
totN = [int(x) for x in ranksout.xpath(path, namespaces=ns)]
print(totN)

path = "string(//x:dimension[@axis='row']/x:group[@text='A']/x:category[@text='Mann-Whitney U']/x:dimension[@axis='column']/x:category[@varName='" + var + "']/x:cell/@number)"
MannWhitneyU_num = str2float(statsout.xpath(path, namespaces=ns))
print(MannWhitneyU_num)

path = "string(//x:dimension[@axis='row']/x:group[@text='A']/x:category[@text='Mann-Whitney U']/x:dimension[@axis='column']/x:category[@varName='" + var + "']/x:cell/@text)"
MannWhitneyU_str = statsout.xpath(path, namespaces=ns)
print(MannWhitneyU_str)

path = "string(//x:dimension[@axis='row']/x:group[@text='A']/x:category[@text='Wilcoxon W']/x:dimension[@axis='column']/x:category[@varName='" + var + "']/x:cell/@number)"
WilcoxonW_num = str2float(statsout.xpath(path, namespaces=ns))
print(WilcoxonW_num)

path = "string(//x:dimension[@axis='row']/x:group[@text='A']/x:category[@text='Wilcoxon W']/x:dimension[@axis='column']/x:category[@varName='" + var + "']/x:cell/@text)"
WilcoxonW_str = statsout.xpath(path, namespaces=ns)
print(WilcoxonW_str)

path = "string(//x:dimension[@axis='row']/x:group[@text='A']/x:category[@text='Z']/x:dimension[@axis='column']/x:category[@varName='" + var + "']/x:cell/@number)"
Z_num = str2float(statsout.xpath(path, namespaces=ns))
print(Z_num)

path = "string(//x:dimension[@axis='row']/x:group[@text='A']/x:category[@text='Z']/x:dimension[@axis='column']/x:category[@varName='" + var + "']/x:cell/@text)"
Z_str = statsout.xpath(path, namespaces=ns)
print(Z_str)

path = "string(//x:dimension[@axis='row']/x:group[@text='A']/x:category[@text='Asymp. Sig. (2-tailed)']/x:dimension[@axis='column']/x:category[@varName='" + var + "']/x:cell/@number)"
asympsig2_num = str2float(statsout.xpath(path, namespaces=ns))
print(asympsig2_num)

path = "string(//x:dimension[@axis='row']/x:group[@text='A']/x:category[@text='Asymp. Sig. (2-tailed)']/x:dimension[@axis='column']/x:category[@varName='" + var + "']/x:cell/@text)"
asympsig2_str = statsout.xpath(path, namespaces=ns)
print(asympsig2_str)

path = "string(//x:dimension[@axis='row']/x:category[@text='Exact Sig. (2-tailed)']/x:dimension[@axis='column']/x:category[@varName='" + var + "']/x:cell/@text)"
exactsig2_str = statsout.xpath(path, namespaces=ns)
print(exactsig2_str)
if exactsig2_str != '':
    path = "string(//x:dimension[@axis='row']/x:category[@text='Exact Sig. (2-tailed)']/x:dimension[@axis='column']/x:category[@varName='" + var + "']/x:cell/@number)"
    exactsig2_num = str2float(statsout.xpath(path, namespaces=ns))
    print(exactsig2_num)

    path = "string(//x:dimension[@axis='row']/x:category[@text='Exact Sig. (1-tailed)']/x:dimension[@axis='column']/x:category[@varName='" + var + "']/x:cell/@number)"
    exactsig1_num = str2float(statsout.xpath(path, namespaces=ns))
    print(exactsig1_num)

    path = "string(//x:dimension[@axis='row']/x:category[@text='Exact Sig. (1-tailed)']/x:dimension[@axis='column']/x:category[@varName='" + var + "']/x:cell/@text)"
    exactsig1_str = statsout.xpath(path, namespaces=ns)
    print(exactsig1_str)

    path = "string(//x:dimension[@axis='row']/x:category[@text='Point Probability']/x:dimension[@axis='column']/x:category[@varName='" + var + "']/x:cell/@number)"
    pointprob_num = str2float(statsout.xpath(path, namespaces=ns))
    print(pointprob_num)

    path = "string(//x:dimension[@axis='row']/x:category[@text='Point Probability']/x:dimension[@axis='column']/x:category[@varName='" + var + "']/x:cell/@text)"
    pointprob_str = statsout.xpath(path, namespaces=ns)
    print(pointprob_str)

varname1short = str(var)
varname1long = str(var)
if varlab != '':
    varname1long = varlab + '[' + varname1long + ']'

varname1short = '<strong>' + varname1short + '</strong>'
varname1long = '<strong>' + varname1long + '</strong>'

varname2short = str(group)
varname2long = str(group)
if grouplab != '':
    varname2long = grouplab + '[' + varname2long + ']'

varname2short = '<strong>' + varname2short + '</strong>'
varname2long = '<strong>' + varname2long + '</strong>'

groupnameshort = group
groupnamelong = group
if grouplab != '':
    groupnamelong = grouplab + '[' + groupnamelong + ']'
groupnameshort = '<strong>' + groupnameshort + '</strong>'
groupnamelong = '<strong>' + groupnamelong + '</strong>'

introhtml = '<h2>The Mann Whitney test</h2>'
introhtml += '<p>'
introhtml += 'In this practical we are going to investigate how to perform a Mann Whitney test using SPSS. '
introhtml += 'A Mann-Whitney test is used when we have a continuous variable measured for all observations in two groups and we want to test if the distribution of this variable is different in the two groups but we are unable to assume normality in both groups. ' 
introhtml += 'It can also be used to compare an ordered categorical variable measured on two groups. '
introhtml += 'It is the non-parametric equivalent of the independent samples t-test but unlike the t-test it tests for differences in the overall distribution across groups rather than for differences in the mean. '
introhtml += 'The test does not assume any particular distribution for the variable in either of the groups. '
introhtml += 'To run a single test in SPSS requires that your dataset has one column containing the variable to be tested and another column of the same length with the groups identified. '
introhtml += '</p>'
outputs['introduction.html'] = HTMLOutput(introhtml, "Introduction text")

normprachtml = '<p>'
normprachtml += 'As a first step we should generally test for the normality of the variable of interest, ' + varname1short + ' in each of the two groups that are indicated by the grouping variable, ' + varname2short + '. '
normprachtml += '(In practice you might skip this step if your dependent variable is ordinal rather than continuous, but let&apos;s work through it here for completeness.)'
normprachtml += '</p>'

norminstrhtml = '<ul class = "instructions">'
norminstrhtml += '<li>Select <strong>Descriptive Statistics</strong> from the <strong>Analyze</strong> menu.</li>'
norminstrhtml += '<li>Select <strong>Explore</strong> from the <strong>Descriptive Statistics</strong> sub-menu.</li>'
norminstrhtml += '<li>Click on the <strong>Reset</strong> button.</li>'
norminstrhtml += '<li>Copy the ' + varname1long + ' variables into the <strong>Dependent List:</strong> box.</li>'
norminstrhtml += '<li>Copy the ' + varname2long + ' variable into the <strong>Factor List:</strong> box.</li>'
norminstrhtml += '<li>Click on the <strong>Plots...</strong> button.</li>'
norminstrhtml += '<li>On the screen that appears select the <strong>Histogram</strong> tick box.</li>'
norminstrhtml += '<li>Unselect the <strong>Stem and leaf</strong> button.</li>'
norminstrhtml += '<li>Select the <strong>Normality plots with tests</strong> button.</li>'
norminstrhtml += '<li>Click on the <strong>Continue</strong> button.</li>'
norminstrhtml += '<li>Click on the <strong>OK</strong> button.</li>'
norminstrhtml += '</ul>'

normprachtml += norminstrhtml

normprachtml += '<p>'
normprachtml += 'We will first look at a histogram of the variable, ' + varname2short + ' = <strong>' + str(cats[0]) + '</strong>. '
normprachtml += 'This can be found in amongst the set of output objects and looks as follows:'
normprachtml += '</p>'

normprachtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_000.png'].content)  + '">'

normprachtml += '<p>'
normprachtml += 'Ideally for a normal distribution this histogram should look symmetric around the mean of the distribution, in this case ' + mean1_str + '. '

if skews1_num > 2*skewses1_num:
    normprachtml += 'This distribution appears to be significantly skewed to the right. '
else:
    if -skews1_num > 2*skewses1_num:
        normprachtml += 'This distribution appears to be significantly skewed to the left. '
    else:
        normprachtml += 'This distribution appears to be reasonably symmetric. '
normprachtml += '</p>'

normprachtml += '<p>'
normprachtml += 'Next we will look at a histogram of the variable, ' + varname2short + ' = <strong>' + str(cats[1]) + '</strong>. '
normprachtml += 'This can be found in amongst the set of output objects and looks as follows:'
normprachtml += '</p>'

normprachtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_1_000.png'].content)  + '">'

normprachtml += '<p>'
normprachtml += 'Ideally for a normal distribution this histogram should look symmetric around the mean of the distribution, in this case ' + mean2_str + '. '
if skews2_num > 2*skewses2_num:
    normprachtml += 'This distribution appears to be significantly skewed to the right (positively skewed). '
else:
    if -skews2_num > 2*skewses2_num:
        normprachtml += 'This distribution appears to be significantly skewed to the left (negatively skewed). '
    else:
        normprachtml += 'This distribution appears to be reasonably symmetric. '
normprachtml += '</p>'

normprachtml += '<p>'
normprachtml += 'We will next look at a statistical test to see if this backs up our visual impressions from the histogram. '
normprachtml += '</p>'

normprachtml += '<p>'
normprachtml += 'The Kolmogorov-Smirnov test is used to test the null hypothesis that a set of data comes from a Normal distribution. '
normprachtml += 'The Kolmogorov Smirnov test produces test statistics that are used (along with a degrees of freedom parameter) to test for normality. '
normprachtml += '</p>'

normprachtml += outputs['Table_Tests of Normality.htm'].content

normprachtml += '<p>'
normprachtml += 'For the first group: '
normprachtml += 'The Kolmogorov-Smirnov test has degrees of freedom which equals the number of data points, namely ' + str(kmdfs1) + '. '
normprachtml += '</p>'

normprachtml += '<p>'
if kmsigs1_num > 0.05:
    normprachtml += 'Here we see that the p value is ' + kmsigs1_str + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the distribution is normal. '
else:
    if kmsigs1_num < 0.001:
        normprachtml += 'Here we see that the p value is ' + kmsigs1_str + ' (reported as p &lt; .001) which is less than 0.05. '
    else:
        normprachtml += 'Here we see that the p value is ' + kmsigs1_str + ' which is less than 0.05. '
    normprachtml += 'We therefore have significant evidence to reject the null hypothesis that the variable follows a normal distribution. '
normprachtml += '</p>'

normprachtml += '<p>'
normprachtml += 'For the second group: '
normprachtml += 'The Kolmogorov-Smirnov test has degrees of freedom which equals the number of data points, namely ' + str(kmdfs2) + '. '
normprachtml += '</p>'

normprachtml += '<p>'
if kmsigs2_num > 0.05:
    normprachtml += 'Here we see that the p value (quoted under Sig. for Kolmogorov Smirnov)  is ' + kmsigs2_str + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the distribution is normal. '
else:
    if kmsigs2_num < 0.001:
        normprachtml += 'Here we see that the p value (quoted under Sig. for Kolmogorov Smirnov) is ' + kmsigs2_str + ' (reported as p &lt; .001) which is less than 0.05. '  
    else:
        normprachtml += 'Here we see that the p value (quoted under Sig. for Kolmogorov Smirnov) is ' + kmsigs2_str + ' which is less than 0.05. '
    normprachtml += 'We therefore have significant evidence to reject the null hypothesis that the variable follows a normal distribution. '
normprachtml += '</p>'

normprachtml += '<p>'
normprachtml += 'Although the Kolmogorov Smirnov statistic tells the researcher whether the distribution followed by a variable is statistically significantly different from a normal distribution one should take care in not overinterpreting such findings. '
normprachtml += 'Significance will be strongly effected by the number of observations and so only a small discrepancy from normality will be deemed significant for very large sample sizes whilst very large discrepancies will be required to reject the null hypothesis for small sample sizes. '
normprachtml += 'Many parametric statistics are robust to non-normality when sample sizes are very large, so non-parametric statistics like the Mann-Whitney are generally used for small samples of non-normal data or ordinal dependent variables. '
normprachtml += '</p>'

normprachtml += '<p>'
normprachtml += 'SPSS also supplies QQ plots to assist in looking at normality but for brevity we do not show them here. '
normprachtml += '</p>'

outputs['normality.html'] = HTMLOutput(normprachtml, "Normality tests text")

normalitytestquesthtml = '<p>'
normalitytestquesthtml += 'The first step is generally to test for the normality of the variable ' + varname1short + ' in each of the two groups that are indicated by ' + groupnameshort + ', so do this now. '
normalitytestquesthtml += '(In practice you might skip this step if your dependent variable is ordinal rather than continuous, but let&apos;s work through it here for completeness.)'
normalitytestquesthtml += '</p>'
normalitytestquesthtml += '<ul>'
normalitytestquesthtml += '<li>Question: Looking at the histogram and the normality tests does this variable look normal for ' + varname2short + ' = <strong>' + str(cats[0]) + '</strong>?</li>'
normalitytestquesthtml += '<li>Question: How about for ' + varname2short + ' = <strong>' + str(cats[1]) + '</strong>?</li>'
normalitytestquesthtml += '</ul>'


outputs['normalityquestions.html'] = HTMLOutput(normalitytestquesthtml, "Normality tests questions")

normalitytestsolhtml = '<p>'
normalitytestsolhtml += 'To test the dependent variable for normality in each group: '
normalitytestsolhtml += '</p>'
normalitytestsolhtml += norminstrhtml
normalitytestsolhtml += '<p>'
normalitytestsolhtml += 'Question: Looking at the histogram and the normality tests does this variable look normal for ' + varname2short + ' = <strong>' + str(cats[0]) + '</strong>?'
normalitytestsolhtml += '</p>'

normalitytestsolhtml += '<p>'
normalitytestsolhtml += 'Solution: The output from SPSS is as follows: '
normalitytestsolhtml += '</p>'

normalitytestsolhtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_000.png'].content)  + '">'

normalitytestsolhtml += '<p>'
normalitytestsolhtml += 'Ideally for a normal distribution this histogram should look symmetric around the mean of the distribution, in this case ' + mean1_str + '. '

if skews1_num > 2*skewses1_num:
    normalitytestsolhtml += 'This distribution appears to be significantly skewed to the right (positively skewed). '
else:
    if -skews1_num > 2*skewses1_num:
        normalitytestsolhtml += 'This distribution appears to be significantly skewed to the left (negatively skewed). '
    else:
        normalitytestsolhtml += 'This distribution appears to be reasonably symmetric. '
normalitytestsolhtml += '</p>'

normalitytestsolhtml += outputs['Table_Tests of Normality.htm'].content

normalitytestsolhtml += '<p>'
normalitytestsolhtml += 'For the first group: '
normalitytestsolhtml += 'The Kolmogorov-Smirnov test has degrees of freedom which equals the number of data points, namely ' + str(kmdfs1) + '. '
normalitytestsolhtml += '</p>'

normalitytestsolhtml += '<p>'
if kmsigs1_num > 0.05:
    normalitytestsolhtml += 'Here we see that the p value is ' + kmsigs1_str + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the distribution is normal. '
else:
    if kmsigs1_num < 0.001:
        normalitytestsolhtml += 'Here we see that the p value is ' + kmsigs1_str + ' (reported as p &lt; .001) which is less than 0.05. '
    else:
        normalitytestsolhtml += 'Here we see that the p value is ' + kmsigs1_str + ' which is less than 0.05. '
    normalitytestsolhtml += 'We therefore have significant evidence to reject the null hypothesis that the variable follows a normal distribution. '
normalitytestsolhtml += '</p>'

normalitytestsolhtml += '<p>'
normalitytestsolhtml += 'Question: How about for ' + varname2short + ' = <strong>' + str(cats[1]) + '</strong>?'
normalitytestsolhtml += '</p>'
normalitytestsolhtml += '<p>'
normalitytestsolhtml += 'Solution: The output from SPSS is as follows: '
normalitytestsolhtml += '</p>'
normalitytestsolhtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_1_000.png'].content)  + '">'

normalitytestsolhtml += '<p>'
normalitytestsolhtml += 'Ideally for a normal distribution this histogram should look symmetric around the mean of the distribution, in this case ' + mean2_str + '. '
if skews2_num > 2*skewses2_num:
    normalitytestsolhtml += 'This distribution appears to be significantly skewed to the right. '
else:
    if -skews2_num > 2*skewses2_num:
        normalitytestsolhtml += 'This distribution appears to be significantly skewed to the left. '
    else:
        normalitytestsolhtml += 'This distribution appears to be reasonably symmetric. '
normalitytestsolhtml += '</p>'

normalitytestsolhtml += outputs['Table_Tests of Normality.htm'].content

normalitytestsolhtml += '<p>'
normalitytestsolhtml += 'For the second group: '
normalitytestsolhtml += 'The Kolmogorov-Smirnov test has degrees of freedom which equals the number of data points, namely ' + str(kmdfs2) + '. '
normalitytestsolhtml += '</p>'

normalitytestsolhtml += '<p>'
if kmsigs2_num > 0.05:
    normalitytestsolhtml += 'Here we see that the p value (quoted under Sig. for Kolmogorov Smirnov)  is ' + kmsigs2_str + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the distribution is normal. '
else:
    if kmsigs2_num < 0.001:
        normalitytestsolhtml += 'Here we see that the p value (quoted under Sig. for Kolmogorov Smirnov) is ' + kmsigs2_str + ' (reported as p &lt; .001) which is less than 0.05. '
    else:
        normalitytestsolhtml += 'Here we see that the p value (quoted under Sig. for Kolmogorov Smirnov) is ' + kmsigs2_str + ' which is less than 0.05. '
    normalitytestsolhtml += 'We therefore have significant evidence to reject the null hypothesis that the variable follows a normal distribution. '
normalitytestsolhtml += '</p>'

outputs['normalityanswers.html'] = HTMLOutput(normalitytestsolhtml, "Normality tests answer text")

rankprachtml = '<p>'
rankprachtml += 'We will now move on to performing the Mann Whitney test for ' + varname1short + '. '
rankprachtml += '</p>'

rankprachtml += '<p>'
rankprachtml += 'Below you will see instructions to perform the Mann Whitney test in SPSS. '
rankprachtml += 'If you follow the instructions you will see the two tabular outputs that are embedded in the explanations below. '
rankprachtml += '</p>'

rankinstrhtml = '<ul class = "instructions">'
rankinstrhtml += '<li>Select <strong>Non Parametric Tests</strong> from the <strong>Analyze</strong> menu.</li>'
rankinstrhtml += '<li>Select <strong>Legacy Dialogs</strong> from the <strong>Non Parametric Tests</strong> sub-menu.</li>'
rankinstrhtml += '<li>Select <strong>2 Independent-Samples...</strong> from the <strong>Legacy Dialogs</strong> sub-menu.</li>'
rankinstrhtml += '<li>Click on the <strong>Reset</strong> button.</li>'
rankinstrhtml += '<li>Copy the <strong>' + varname1long + '</strong> variable into the <strong>Test Variable List:</strong> box.</li>'
rankinstrhtml += '<li>Copy the <strong>' + varname2long + '</strong> variable into the <strong>Grouping Variable:</strong> box.</li>'
rankinstrhtml += '<li>Click on the <strong>Define Groups...</strong> button.</li>'
rankinstrhtml += '<li>Type <strong>' + str(val1) + '</strong> into the <strong>Group 1</strong> box.</li>'
rankinstrhtml += '<li>Type <strong>' + str(val2) + '</strong> into the <strong>Group 2</strong> box.</li>'
rankinstrhtml += '<li>Click on the <strong>Continue</strong> button.</li>'
rankinstrhtml += '<li>Click on the <strong>Exact...</strong> button.</li>'
rankinstrhtml += '<li>On the screen that appears select the <strong>Exact</strong> button.</li>'
rankinstrhtml += '<li>Click on the <strong>Continue</strong> button.</li>'
rankinstrhtml += '<li>Click on the <strong>OK</strong> button.</li>'
rankinstrhtml += '</ul>'

rankprachtml += rankinstrhtml

rankprachtml += '<p>'
rankprachtml += 'The first SPSS output table contains a summary of the rankings for the 2 groups and can be seen below:'
rankprachtml += '</p>'

rankprachtml += outputs['Table_Mann Whitney Ranks.htm'].content

rankprachtml += '<p>'
rankprachtml += 'The Mann Whitney test works by firstly constructing a ranked list of the observations labelled in their two groups. '
rankprachtml += 'It will then work from the lowest observation and give that observation rank 1 and the next rank 2 and so on right up to the largest observation which in this case will have rank ' + str(totN[0]) + '. '
rankprachtml += 'If there are observations with the same value then they are given the same rank that is an average of the ranks available (for example if three observations have the 9th smallest rank then rather than giving them ranks 9, 10 and 11 respectively they will each be given rank 10 (9+10+11)/3 = 10). '
rankprachtml += 'The test works by comparing the sum of the ranks in the two groups. '
rankprachtml += '</p>'

rankprachtml += '<p>'
rankprachtml += 'The statistics required for the test are constructed from the ranks and shown in the table. '
rankprachtml += 'Here we see that for ' + varname2short + ' category ' + str(cats[0]) + ' we have ' + str(N[0]) + ' observations whose total sum of ranks is ' + sums_str[0] + '. '
rankprachtml += 'This results in a mean rank of ' + means_str[0] + '. '
rankprachtml += 'By contrast for ' + varname2short + ' category ' + str(cats[1]) + ' we have ' + str(N[1]) + ' observations whose total sum of ranks is ' + sums_str[1] + '. '
rankprachtml += 'This results in a mean rank of ' + means_str[1] + '. '
if means_num[0] > means_num[1]:
   rankprachtml += 'So ' + varname2short + ' category ' + str(cats[0]) + ' has a larger mean rank than ' + varname2short + ' category ' + str(cats[1]) + ' and thus tends to take larger values. '
if means_num[1] > means_num[0]:
   rankprachtml += 'So ' + varname2short + ' category ' + str(cats[1]) + ' has a larger mean rank than ' + varname2short + ' category ' + str(cats[0]) + ' and thus tends to take larger values. '
if means_num[1] == means_num[0]:
   rankprachtml += 'Very unusually both groups have identical mean ranks. '
rankprachtml += '</p>'
outputs['rank.html'] = HTMLOutput(rankprachtml, "Rank text")

rankquesthtml = '<p>'
rankquesthtml += 'Next perform the Mann Whitney test in SPSS to test for differences in the distribution of ' + varname1long + ' between groups defined by ' + varname2long + ' and answer the following questions:' 
rankquesthtml += '</p>'
rankquesthtml += '<ul>'
rankquesthtml += '<li>Question: Which of the two groups has the highest mean rank?</li>'
rankquesthtml += '</ul>'
outputs['rankquestions.html'] = HTMLOutput(rankquesthtml, "Rank questions text")

ranksolhtml = '<p>'
ranksolhtml += 'To perform the Mann Whitney test in SPSS: '
ranksolhtml += '</p>'
ranksolhtml += rankinstrhtml
ranksolhtml += '<ul>'
ranksolhtml += '<li>Question: Which of the two groups has the highest mean rank?</li>'
ranksolhtml += '</ul>'
ranksolhtml += '<p>'
ranksolhtml += 'Solution: The output from SPSS is as follows:'
ranksolhtml += '</p>'
ranksolhtml += outputs['Table_Mann Whitney Ranks.htm'].content
ranksolhtml += '<p>'
ranksolhtml += 'Here we see that for ' + varname2short + ' category ' + str(cats[0]) + ' we have ' + str(N[0]) + ' observations whose total sum of ranks is ' + sums_str[0] + '. '
ranksolhtml += 'This results in a mean rank of ' + means_str[0] + '. '
ranksolhtml += 'By contrast for ' + varname2short + ' category ' + str(cats[1]) + ' we have ' + str(N[1]) + ' observations whose total sum of ranks is ' + sums_str[1] + '. '
ranksolhtml += 'This results in a mean rank of ' + means_str[1] + '. '
if means_num[0] > means_num[1]:
   ranksolhtml += 'So ' + varname2short + ' category ' + str(cats[0]) + ' has a larger mean rank than ' + varname2short + ' category ' + str(cats[1]) + ' and thus tends to take larger values. '
if means_num[1] > means_num[0]:
   ranksolhtml += 'So ' + varname2short + ' category ' + str(cats[1]) + ' has a larger mean rank than ' + varname2short + ' category ' + str(cats[0]) + ' and thus tends to take larger values. '
if means_num[1] == means_num[0]:
   ranksolhtml += 'Very unusually both groups have identical mean ranks. '
ranksolhtml += '</p>'

outputs['rankanswers.html'] = HTMLOutput(ranksolhtml, "Rank answers text")

testprachtml = '<p>'
testprachtml += 'The Mann Whitney test will now decide on whether this difference in mean ranks is significant or not as is illustrated in the second table.'
testprachtml += '</p>'

testprachtml += '<p>'
testprachtml += 'The second SPSS output table contains details of the test itself and can be seen below:'
testprachtml += '</p>'

testprachtml += outputs['Table_Mann Whitney Test Statistics.htm'].content

mw1 = sums_num[0] - (N[0]*(N[0]+1)/2)
mw2 = sums_num[1] - (N[1]*(N[1]+1)/2)

testprachtml += '<p>'
testprachtml += 'The output here consists of test statistics and their significance as calculated in several ways. '
testprachtml += 'We are considering the Mann Whitney U statistic and to calculate this we need to consider the sums of the rankings and compare them with what we would expect if these two groups came from the same distribution. '
testprachtml += 'We consider each group in turn and work out for each group a U statistic. '
testprachtml += 'The formula here is the sum of the ranks - N x (N+1)/2 for each group. '
testprachtml += 'So for ' + varname2short + ' category ' + str(cats[0]) + ' we have U1 = ' + sums_str[0] + '-' + str(N[0]) + 'x(' + str(N[0]) + '+1)/2 = ' + str(mw1) + ' and for ' + varname2short + ' category ' + str(cats[1]) + ' we have U2 = ' + sums_str[1] + '-' + str(N[1]) + 'x(' + str(N[1]) + '+1)/2 = ' + str(mw2) + '. '
if mw1 < mw2:
    testprachtml += 'Here U1 is less than U2 and it is the lower of the two U statistics that we report when giving the results. So here the value ' + str(mw1) + ' is the U test statistic as shown in the table. '
if mw1 > mw2:
    testprachtml += 'Here U2 is less than U1 and it is the lower of the two U statistics that we report when giving the results. So here the value ' + str(mw2) + ' is the U test statistic as shown in the table. '
if mw1 == mw2:
    testprachtml += 'Here U1 and U2 are equal and we can therefore use either value as our test statistic as shown in the table. '
testprachtml += '</p>'

testprachtml += '<p>'
testprachtml += 'A related approach that uses ranks is the Wilcoxon W statistic which is quoted here and is the maximum of the two rank sums but it is less used than the Mann-Whitney so we do not describe it further. '
testprachtml += 'One way to interpret the Mann-Whitney U statistic is to convert it to a normal score by subtracting its mean and dividing by its standard error and that is done in the Z row. '
testprachtml += 'Here we see that Z = ' + Z_str + ' and this can be compared with a standard normal distribution to get a sense of the magnitude by which the groups differ. '
testprachtml += '</p>'

testprachtml += '<p>'
if asympsig2_num > 0.05:
    testprachtml += 'Here we see that the p value, quoted next to Asymp. Sig. (2-tailed), is ' + asympsig2_str + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the medians of the two groups are the same. '
else:
    if asympsig2_num < 0.001:
        testprachtml += 'Here we see that the p value, quoted next to Asymp. Sig. (2-tailed), is ' + asympsig2_str + ' (reported as p &lt; .001) which is less than 0.05. '
    else:
        testprachtml += 'Here we see that the p value, quoted next to Asymp. Sig. (2-tailed) is ' + asympsig2_str + ' which is less than 0.05. '
    testprachtml += 'We therefore have significant evidence to reject the null hypothesis that the distribution of ' + varname1short + ' is the same in the two groups. '
    if means_num[0] > means_num[1]:
        testprachtml += 'In this case ' + varname2short + ' category ' + str(cats[0]) + ' has a higher mean rank than ' + varname2short + ' category ' + str(cats[1]) + '. '
    if means_num[1] > means_num[0]:
        testprachtml += 'In this case ' + varname2short + ' category ' + str(cats[1]) + ' has a higher mean rank than ' + varname2short + ' category ' + str(cats[0]) + '. '
if exactsig2_str != '':
    testprachtml += 'The normal approximation used above is only an approximation to the p value and it is possible to construct the exact p value. '
    testprachtml += 'This is given in the next row and we see that the exact p value is ' + exactsig2_str + ' whilst the asymptotic p value is ' + asympsig2_str + '. '
    if asympsig2_num > 0.05:
        if exactsig2_num > 0.05:
             testprachtml += 'The exact p value agrees with the asymptotic p value that the null hypothesis cannot be rejected. '
        else:
             testprachtml += 'Unusually the exact p value disagrees with the asymptotic p value and suggests that the null hypothesis can be rejected. In this case we would use the exact result. '
    if asympsig2_num < 0.05:
        if exactsig2_num < 0.05:
             testprachtml += 'The exact p value agrees with the asymptotic p value that the null hypothesis can be rejected. '
        else:
             testprachtml += 'Unusually the exact p value disagrees with the asymptotic p value and suggests that the null hypothesis cannot be rejected. In this case we would use the exact result. '
    testprachtml += 'For completeness the table also gives a p value for a 1-sided test and a point probability but we will ignore these here. '
else:
    testprachtml += 'The normal approximation used above is only an approximation to the p value and SPSS offers a computational procedure to construct an exact p value. '
    testprachtml += 'However in this case the sample sizes are so large that the exact test was too computationally-intensive to complete, although the asymptotic p value should be accurate enough. '
testprachtml += '</p>'
outputs['test.html'] = HTMLOutput(testprachtml, "Test statistics text")

testquesthtml = '<ul>'
testquesthtml += '<li>Question: Looking at the second SPSS table does it look like the two groups are significantly different? </li>'
testquesthtml += '</ul>'
outputs['testquestions.html'] = HTMLOutput(testquesthtml, "Test statistics questions text")

testsolhtml = '<ul>'
testsolhtml += '<li>Question: Looking at the second SPSS table does it look like the two groups are significantly different? </li>'
testsolhtml += '</ul>'
testsolhtml += '<p>'
testsolhtml += 'Solution: The output from SPSS is as follows:'
testsolhtml += '</p>'
testsolhtml += outputs['Table_Mann Whitney Test Statistics.htm'].content
testsolhtml += '<p>'
if asympsig2_num > 0.05:
    testsolhtml += 'Here we see that the p value, quoted next to Asymp. Sig. (2-tailed) is ' + asympsig2_str + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the medians of the two groups are the same. '
else:
    if asympsig2_num < 0.001:
        testsolhtml += 'Here we see that the p value, quoted next to Asymp. Sig. (2-tailed) is ' + asympsig2_str + ' (reported as p &lt; .001) which is less than 0.05. '
    else:
        testsolhtml += 'Here we see that the p value, quoted next to Asymp. Sig. (2-tailed) is ' + asympsig2_str + ' which is less than 0.05. '
    testsolhtml += 'We therefore have significant evidence to reject the null hypothesis that the distribution of ' + varname1short + ' is the same for both groups. '
    if means_num[0] > means_num[1]:
        testsolhtml += 'In this case ' + varname2short + ' category ' + str(cats[0]) + ' has a higher mean rank than ' + varname2short + ' category ' + str(cats[1]) + ', meaning more scores in a higher position. '
    if means_num[1] > means_num[0]:
        testsolhtml += 'In this case ' + varname2short + ' category ' + str(cats[1]) + ' has a higher mean rank than ' + varname2short + ' category ' + str(cats[0]) + ', meaning more scores in a higher position. '
if exactsig2_str != '':
    testsolhtml += 'The asymptotic significance used above is only an approximation to the p value and it is possible to construct the exact p value. '
    testsolhtml += 'This is given in the next row and we see that the exact p value is ' 
    if exactsig2_num < 0.001:
        testsolhtml += exactsig2_str + ' (reported as p &lt; .001)'
    else:
        testsolhtml += exactsig2_str
    testsolhtml += ' whilst the asymptotic p value is also reported as ' + asympsig2_str + '. '
    if asympsig2_num > 0.05:
        if exactsig2_num > 0.05:
             testsolhtml += 'The exact p value agrees with the asymptotic p value that the null hypothesis cannot be rejected. '
        else:
             testsolhtml += 'Unusually the exact p value disagrees with the asymptotic p value and suggests that the null hypothesis can be rejected. In this case we would use the exact result. '
    if asympsig2_num < 0.05:
        if exactsig2_num < 0.05:
             testsolhtml += 'The exact p value agrees with the asymptotic p value that the null hypothesis can be rejected. '
        else:
             testsolhtml += 'Unusually the exact p value disagrees with the asymptotic p value and suggests that the null hypothesis cannot be rejected. In this case we would use the exact result. '
else:
    testsolhtml += 'The normal approximation used above is only an approximation to the p value and SPSS offers a computational procedure to construct an exact p value. '
    testsolhtml += 'However in this case the sample sizes are so large that the exact test was too computational to complete but the asymptotic p value should be accurate enough. '
testsolhtml += '</p>'
outputs['testanswers.html'] = HTMLOutput(testsolhtml, "Test statistics answers text")

conchtml = '<p>'
conchtml += 'In conclusion, we could report this to a reader as follows:'
conchtml += '</p>'

conchtml += '<p>'
conchtml += 'A comparison of the mean of the distribution of the variable ' + str(var) + ' was desired for ' + varname2short + ' categories ' + str(cats[0]) + ' and ' + str(cats[1]) + ' but due to the non-normality of the variable a Mann Whitney test was carried out. '
if means_num[0] > means_num[1]:
   conchtml += varname2short + ' category ' + str(cats[0]) + ' (N= ' + str(N[0]) + ') has a larger mean rank ('+ means_str[0] + ') than ' + varname2short + ' category ' + str(cats[1]) + ' (N= ' + str(N[1]) + ') with mean rank ('+ means_str[1] + ') and thus tends to take larger values. '
if means_num[1] > means_num[0]:
   conchtml += varname2short + ' category ' + str(cats[1]) + ' (N= ' + str(N[1]) + ') has a larger mean rank ('+ means_str[1] + ') than ' + varname2short + ' category ' + str(cats[0]) + ' (N= ' + str(N[0]) + ') with mean rank ('+ means_str[0] + ') and thus tends to take larger values. '
if means_num[1] == means_num[0]:
   conchtml += 'Very unusually both groups have identical mean ranks. '
conchtml += 'A statistically significant difference was '
if exactsig2_str != '':
   if exactsig2_num < 0.05:
       conchtml += 'found (U = ' + MannWhitneyU_str + ', '
   else:
       conchtml += 'not found (U = ' + MannWhitneyU_str + ', '
   if exactsig2_num < 0.001:
       conchtml += 'p &lt; .001'
   else:
       conchtml += 'p = ' + exactsig2_str
else:

   if asympsig2_num < 0.05:
       conchtml += 'found (U = ' + MannWhitneyU_str + ', '
   else:
       conchtml += 'found (U = ' + MannWhitneyU_str + ', '
   if asympsig2_num < 0.001:
       conchtml += 'p  &lt; .001'
   else:
       conchtml += 'p = ' + asympsig2_str
conchtml += '). '
conchtml += '</p>'
    
outputs['conclusion.html'] = HTMLOutput(conchtml, "Conclusion text")

concquesthtml = '<ul>'
concquesthtml += '<li>Question: How would you write up these findings?</li>'
concquesthtml += '</ul>'
outputs['conclusionquestions.html'] = HTMLOutput(concquesthtml, "Conclusion question text")

concsolhtml = concquesthtml
concsolhtml += '<p>Solution: </p>'
concsolhtml += conchtml
outputs['conclusionsolutions.html'] = HTMLOutput(concsolhtml, "Conclusion solutions text")

combinedprachtml = outputs['introduction.html'].content
combinedprachtml += outputs['normality.html'].content
combinedprachtml += outputs['rank.html'].content
combinedprachtml += outputs['test.html'].content
combinedprachtml += outputs['conclusion.html'].content

outputs['allpractical.html'] = HTMLOutput(combinedprachtml, "Combined practical HTML outputs")

combinedquesthtml = outputs['introduction.html'].content
combinedquesthtml += outputs['normalityquestions.html'].content
combinedquesthtml += outputs['rankquestions.html'].content
combinedquesthtml += outputs['testquestions.html'].content
combinedquesthtml += outputs['conclusionquestions.html'].content

outputs['allquestions.html'] = HTMLOutput(combinedquesthtml, "Combined questions HTML outputs")

combinedsolhtml = outputs['normalityanswers.html'].content
combinedsolhtml += outputs['rankanswers.html'].content
combinedsolhtml += outputs['testanswers.html'].content
combinedsolhtml += outputs['conclusionsolutions.html'].content

outputs['allanswers.html'] = HTMLOutput(combinedsolhtml, "Combined solutions HTML outputs")
'''

    spssgui = r'''
\begin{aligned}
&\text{Select }\textbf{Non Parametric Tests}\text{ from the }\textbf{Analyze}\text{ menu.} \\ 
&\text{Select }\textbf{Legacy Dialogs}\text{ from the }\textbf{Non Parametric Tests}\text{ sub-menu.} \\ 
&\text{Select }\textbf{2 Independent-Samples...}\text{ from the }\textbf{Legacy Dialogs}\text{ sub-menu.} \\ 
&\text{Click on the } \textbf{Reset}\text{ button } \\ 
&\text{Copy the } \textbf{${datafile.variables[var]['varlabel'] + ' ['+var+']'}}\text{ variable into the } \textbf{Test Variable List:}\text{ box. } \\ 
&\text{Copy the } \textbf{${datafile.variables[group]['varlabel'] + ' ['+group+']'}}\text{ variable into the } \textbf{Grouping Variable:}\text{ box. } \\ 
&\text{Click on the } \textbf{Define Groups...}\text{ button } \\ 
&\text{Type } \textbf{${val1}}\text{ into the } \textbf{Group 1}\text{ box. } \\ 
&\text{Type } \textbf{${val2}}\text{ into the } \textbf{Group 2}\text{ box. } \\ 
&\text{Click on the } \textbf{Continue}\text{ button } \\ 
&\text{Click on the } \textbf{Exact...}\text{ button } \\ 
&\text{On the screen that appears select the } \textbf{Exact}\text{ button } \\ 
&\text{Click on the } \textbf{Continue}\text{ button } \\ 
&\text{Click on the } \textbf{OK}\text{ button } \\  
\end{aligned}

'''