# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class Regression2(Template):
    'Calculate Descriptive Statistics in SPSS (equivalent to Analyze->Descriptive Statistics->Descriptives... in SPSS 23)'

    __version__ = '1.0.0'

    tags = [ 'SPSS']
    engines = ['SPSS_script']

    inputs = '''
y = DataVector('Response variable: ')
pred = DataVector('Predictor variable: ')
'''

    spssscript = '''
DESCRIPTIVES VARIABLES=${y} ${pred}.

GRAPH
  /SCATTERPLOT(BIVAR)=${pred} WITH ${y}
  /MISSING=LISTWISE.

CORRELATIONS
  /VARIABLES=${y} ${pred}
  /PRINT=TWOTAIL NOSIG
  /STATISTICS DESCRIPTIVES
  /MISSING=PAIRWISE.

REGRESSION
  /MISSING LISTWISE
  /STATISTICS COEFF OUTS CI(95) R ANOVA
  /CRITERIA=PIN(.05) POUT(.10)
  /NOORIGIN 
  /DEPENDENT ${y}
  /METHOD=ENTER ${pred}
  /SCATTERPLOT=(*ZRESID, ${pred})
  /RESIDUALS HISTOGRAM(ZRESID) NORMPROB(ZRESID).

IGRAPH
  /VIEWNAME ='Scatterplot'
  /X1 = VAR(${pred}) TYPE=SCALE
  /Y = VAR(${y}) TYPE=SCALE
  /FITLINE METHOD=REGRESSION LINEAR LINE=TOTAL SPIKE=OFF
  /SCATTER COINCIDENT=NONE.

'''

    processoutputs = '''
import EStat
from EStat.Templating import *
import base64

from lxml import etree
from io import BytesIO

ns = {'x': 'http://www.ibm.com/software/analytics/spss/xml/oms'}

def str2float(s):
    try:
        return float(s)
    except ValueError:
        return float('nan')

anovaout = etree.parse(BytesIO(outputs['Table_ANOVA.xml'].content))
print(etree.tostring(anovaout, pretty_print=True))

path = "//x:dimension[@axis='column']/x:category[@text='Sum of Squares']/x:cell/@number"
aovss_num = [str2float(x) for x in anovaout.xpath(path, namespaces=ns)]
print(aovss_num)

path = "//x:dimension[@axis='column']/x:category[@text='Sum of Squares']/x:cell/@text"
aovss_str = anovaout.xpath(path, namespaces=ns)
print(aovss_str)

path = "//x:dimension[@axis='column']/x:category[@text='Mean Square']/x:cell/@number"
aovms_num = [str2float(x) for x in anovaout.xpath(path, namespaces=ns)]
print(aovms_num)

path = "//x:dimension[@axis='column']/x:category[@text='Mean Square']/x:cell/@text"
aovms_str = anovaout.xpath(path, namespaces=ns)
print(aovms_str)

path = "//x:dimension[@axis='column']/x:category[@text='df']/x:cell/@number"
aovdf = [int(x) for x in anovaout.xpath(path, namespaces=ns)]
print(aovdf)

path = "//x:dimension[@axis='column']/x:category[@text='F']/x:cell/@number"
aovf_num = [str2float(x) for x in anovaout.xpath(path, namespaces=ns)]
print(aovf_num)

path = "//x:dimension[@axis='column']/x:category[@text='F']/x:cell/@text"
aovf_str = anovaout.xpath(path, namespaces=ns)
print(aovf_str)

path = "//x:dimension[@axis='column']/x:category[@text='Sig.']/x:cell/@number"
aovsig_num = [str2float(x) for x in anovaout.xpath(path, namespaces=ns)]
print(aovsig_num)

path = "//x:dimension[@axis='column']/x:category[@text='Sig.']/x:cell/@text"
aovsig_str = anovaout.xpath(path, namespaces=ns)
print(aovsig_str)

coefout = etree.parse(BytesIO(outputs['Table_Coefficients.xml'].content))
print(etree.tostring(coefout, pretty_print=True))

path = "//x:dimension[@axis='column']/x:group/x:category[@text='B']/x:cell/@number"
coefb_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coefb_num)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='B']/x:cell/@text"
coefb_str = coefout.xpath(path, namespaces=ns)
print(coefb_str)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Std. Error']/x:cell/@number"
coefse_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coefse_num)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Std. Error']/x:cell/@text"
coefse_str = coefout.xpath(path, namespaces=ns)
print(coefse_str)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Beta']/x:cell/@number"
coefbeta_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coefbeta_num)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Beta']/x:cell/@text"
coefbeta_str = coefout.xpath(path, namespaces=ns)
print(coefbeta_str)

path = "//x:dimension[@axis='column']/x:category[@text='t']/x:cell/@number"
coeft_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coeft_num)

path = "//x:dimension[@axis='column']/x:category[@text='t']/x:cell/@text"
coeft_str = coefout.xpath(path, namespaces=ns)
print(coeft_str)

path = "//x:dimension[@axis='column']/x:category[@text='Sig.']/x:cell/@number"
coefsig_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coefsig_num)

path = "//x:dimension[@axis='column']/x:category[@text='Sig.']/x:cell/@text"
coefsig_str = coefout.xpath(path, namespaces=ns)
print(coefsig_str)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Lower Bound']/x:cell/@number"
coeflb_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coeflb_num)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Lower Bound']/x:cell/@text"
coeflb_str = coefout.xpath(path, namespaces=ns)
print(coeflb_str)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Upper Bound']/x:cell/@number"
coefub_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coefub_num)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Upper Bound']/x:cell/@text"
coefub_str = coefout.xpath(path, namespaces=ns)
print(coefub_str)

corrout = etree.parse(BytesIO(outputs['Table_Correlations.xml'].content))
print(etree.tostring(corrout, pretty_print=True))

statsout = etree.parse(BytesIO(outputs['Table_Descriptive Statistics.xml'].content))
print(etree.tostring(statsout, pretty_print=True))

path = "//x:dimension[@text='Variables']/x:category/@text"
ylab = statsout.xpath(path, namespaces=ns)[0]
print(ylab)

path = "//x:dimension[@text='Variables']/x:category/@text"
xlab = statsout.xpath(path, namespaces=ns)[1]
print(xlab)

path = "//x:dimension[@axis='column']/x:category[@text='Mean']/x:cell/@number"
means_num = [str2float(x) for x in statsout.xpath(path, namespaces=ns)]
print(means_num)

path = "//x:dimension[@axis='column']/x:category[@text='Mean']/x:cell/@text"
means_str = statsout.xpath(path, namespaces=ns)
print(means_str)

path = "//x:dimension[@axis='column']/x:category[@text='Std. Deviation']/x:cell/@number"
sds_num = [str2float(x) for x in statsout.xpath(path, namespaces=ns)]
print(sds_num)

path = "//x:dimension[@axis='column']/x:category[@text='Std. Deviation']/x:cell/@text"
sds_str = statsout.xpath(path, namespaces=ns)
print(sds_str)

path = "//x:dimension[@axis='column']/x:category[@text='Minimum']/x:cell/@number"
mins_num = [str2float(x) for x in statsout.xpath(path, namespaces=ns)]
print(mins_num)

path = "//x:dimension[@axis='column']/x:category[@text='Minimum']/x:cell/@text"
mins_str = statsout.xpath(path, namespaces=ns)
print(mins_str)

path = "//x:dimension[@axis='column']/x:category[@text='Maximum']/x:cell/@number"
maxs_num = [str2float(x) for x in statsout.xpath(path, namespaces=ns)]
print(maxs_num)

path = "//x:dimension[@axis='column']/x:category[@text='Maximum']/x:cell/@text"
maxs_str = statsout.xpath(path, namespaces=ns)
print(maxs_str)

modelout = etree.parse(BytesIO(outputs['Table_Model Summary.xml'].content))
print(etree.tostring(modelout, pretty_print=True))

path = "//x:dimension[@axis='column']/x:category[@text='R']/x:cell/@number"
r_num = [str2float(x) for x in modelout.xpath(path, namespaces=ns)]
print(r_num)

path = "//x:dimension[@axis='column']/x:category[@text='R']/x:cell/@text"
r_str = modelout.xpath(path, namespaces=ns)
print(r_str)

path = "//x:dimension[@axis='column']/x:category[@text='R Square']/x:cell/@number"
r2_num = [str2float(x) for x in modelout.xpath(path, namespaces=ns)]
print(r2_num)

path = "//x:dimension[@axis='column']/x:category[@text='R Square']/x:cell/@text"
r2_str = modelout.xpath(path, namespaces=ns)
print(r2_str)

path = "//x:dimension[@axis='column']/x:category[@text='Adjusted R Square']/x:cell/@number"
adjr2_num = [str2float(x) for x in modelout.xpath(path, namespaces=ns)]
print(adjr2_num)

path = "//x:dimension[@axis='column']/x:category[@text='Adjusted R Square']/x:cell/@text"
adjr2_str = modelout.xpath(path, namespaces=ns)
print(adjr2_str)

resiout = etree.parse(BytesIO(outputs['Table_Residuals Statistics.xml'].content))
print(etree.tostring(resiout, pretty_print=True))

explout = etree.parse(BytesIO(outputs['Table_Variables Entered Removed.xml'].content))
print(etree.tostring(explout, pretty_print=True))

# Outputs:
# Chart_000.png : Scatter plot
# Chart_1_000.png : histogram (standardised residual)
# Chart_2_000.png : Normal P-P (standardised residual)
# Chart_3_000.png : scatterplot (standardised residual vs standardised predicted values)
# Chart_4_000.png : Scatter plot with fitted line
# Table_ANOVA.htm : Anova table
# Table_Coefficients.htm : Regression coefficients
# Table_Correlations.htm : variable correlations
# Table_Descriptive Statistics.htm : descriptive statistics
# Table_Descriptive Statistics_1.htm : descriptive statistics
# Table_Model Summary.htm : Fit statistics
# Table_Notes.htm : Notes for DESCRIPTIVES command
# Table_Notes.htm : Notes for scatterplot command
# Table_Notes.htm : Notes for correlation command
# Table_Notes.htm : Notes for regression command
# Table_Residuals Statistics.htm : summary statistics for prediction and residual
# Table_Variables Entered Removed.htm : details of dependent variables aded or removed

respnameshort = str(y)
respnamelong = str(y)
if ylab != '':
    respnamelong = ylab + '[' + respnamelong + ']'

respnameshort = '<strong>' + respnameshort + '</strong>'
respnamelong = '<strong>' + respnamelong + '</strong>'

varnameshort = str(pred)
varnamelong = str(pred)
if xlab != '':
    varnamelong = xlab + '[' + varnamelong + ']'

varnameshort = '<strong>' + varnameshort + '</strong>'
varnamelong = '<strong>' + varnamelong + '</strong>'


introhtml = '<h2>Regression practical</h2>'
introhtml += '<p>'
introhtml += 'In this practical we will look at regressing one variable on another variable to explore the relationship between them. '
introhtml += 'This work builds on the concept of correlation but here we specify one variable as a response (or dependent) variable and look at the effect of another predictor (or independent) variable upon it. '
introhtml += '</p>'

outputs['introduction.html'] = HTMLOutput(introhtml, "Introduction text")

descriptprachtml = '<p>'
descriptprachtml += 'In this example ' + respnameshort + ' is the response variable and ' + varnameshort + ' is the predictor variable. '
descriptprachtml += 'To begin with we will simply look at some basic summary information about the variables and plot them in a scatterplot in SPSS which is done as follows:'
descriptprachtml += '</p>'

descriptinstrhtml = '<ul class = "instructions">'
descriptinstrhtml += '<li>Select <strong>Descriptives</strong> from the <strong>Descriptive Statistics</strong> submenu available from the <strong>Analyze</strong> menu.</li>'
descriptinstrhtml += '<li>Copy the ' + respnamelong + ' and ' + varnamelong + ' variables into the <strong>Variable(s)</strong> box.</li>'
descriptinstrhtml += '<li>Click on the <strong>Options</strong> button.</li>'
descriptinstrhtml += '<li>Ensure that the <strong>Mean, Std. deviation, Minimum</strong> and <strong>Maximum</strong> options are selected only.</li>'
descriptinstrhtml += '<li>Click on the <strong>Continue</strong> button to return to the main window.</li>'
descriptinstrhtml += '<li>Click on the <strong>OK</strong> button to run the command.</li>'
descriptinstrhtml += '</ul>'

descriptprachtml += descriptinstrhtml

descriptprachtml += '<p>'
descriptprachtml += 'The descriptive statistics will then appear as shown below:'
descriptprachtml += '</p>'

descriptprachtml += outputs['Table_Descriptive Statistics.htm'].content

descriptprachtml += '<p>'
descriptprachtml += 'Here we see a row in the table for each variable. ' + respnameshort + ' is the response variable and takes values between ' + mins_str[0] + ' and ' + maxs_str[0] + ' with a mean of ' + means_str[0] + '. '
descriptprachtml += varnameshort + ' is the predictor variable and takes values between ' + mins_str[1] + ' and ' + maxs_str[1] + ' with a mean of ' + means_str[1] + '. '
descriptprachtml += '</p>'

outputs['descriptive.html'] = HTMLOutput(descriptprachtml, "After descriptive statistics")

descriptquesthtml = '<p>'
descriptquesthtml += 'Begin by looking at some basic summary information about the response variable ' + respnameshort + ' and the predictor variable ' + varnameshort + '. '
descriptquesthtml += '</p>'
descriptquesthtml += '<ul>'
descriptquesthtml += '<li>Question: What is the range of the predictor?</li>'
descriptquesthtml += '<li>Question: What is the mean of the predictor?</li>'
descriptquesthtml += '</ul>'

outputs['descriptivequestions.html'] = HTMLOutput(descriptquesthtml, "After descriptive statistics questions")

descriptsolhtml = '<p>'
descriptsolhtml += 'The instructions for creating the required descriptive statistics in SPSS are as follows: '
descriptsolhtml += '</p>'
descriptsolhtml += descriptinstrhtml
descriptsolhtml += '<ul>'
descriptsolhtml += '<li>Question: What is the range of the predictor?</li>'
descriptsolhtml += '<li>Question: What is the mean of the predictor?</li>'
descriptsolhtml += '</ul>'
descriptsolhtml += '<p>'
descriptsolhtml += 'Solutions: The output from SPSS is as follows:'
descriptsolhtml += '</p>'
descriptsolhtml += outputs['Table_Descriptive Statistics.htm'].content
descriptsolhtml += '<p>'
descriptsolhtml += 'The answers are as follows:'
descriptsolhtml += '</p>'
descriptsolhtml += '<ul>'
descriptsolhtml += '<li>The predictor takes values between ' + mins_str[1] + ' and ' + maxs_str[1] + '.</li>'
descriptsolhtml += '<li>The mean of the predictor is ' + means_str[1] + '.</li>'
descriptsolhtml += '</ul>'

outputs['descriptiveanswers.html'] = HTMLOutput(descriptsolhtml, "After descriptive statistics answers")

scatterprachtml = '<p>'
scatterprachtml += 'We can next plot these variables against each other following instructions below:'
scatterprachtml += '</p>'

scatterinstrhtml = '<ul class = "instructions">'
scatterinstrhtml += '<li>Select <strong>Scatter/Dot</strong> from the <strong>Legacy diagnostics</strong> available from the <strong>Graphs</strong> menu.</li>'
scatterinstrhtml += '<li>Select <strong>Simple Scatter</strong> and click on Define to bring up the Simple Scatterplot window.</li>'
scatterinstrhtml += '<li>Copy the ' + respnamelong + ' variable into the <strong>Y Axis</strong> box.</li>'
scatterinstrhtml += '<li>Copy the ' + varnamelong + ' variable into the <strong>X Axis</strong> box.</li>'
scatterinstrhtml += '<li>Click on the <strong> OK </strong> button.<br/></p>'
scatterinstrhtml += '</ul>'

scatterprachtml += scatterinstrhtml

scatterprachtml += '<p>'
scatterprachtml += 'SPSS will then draw a scatterplot of the two variables which can be seen below:'
scatterprachtml += '</p>'

scatterprachtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_000.png'].content)  + '">'

scatterprachtml += '<p>'
scatterprachtml += 'The scatterplot gives a general idea of the relationship between the two variables and we can look by eye to see if a linear relationship is suitable. '
if coefb_num[1] > 0.2:
    scatterprachtml += 'In this example there appears to be a positive relationship as there are more points in the bottom-left and top-right quarters of the plot than in the top-left and bottom-right corners. '
elif coefb_num[1] < -0.2:
    scatterprachtml += 'In this example there appears to be a negative relationship as there are more points in the top-left and bottom-right quarters of the plot than in the bottom-left and top-right corners. '
else:
    scatterprachtml += 'In this example it is difficult to discern a relationship because the points appear as a random scatter in the plot. '
scatterprachtml += '</p>'

outputs['scatter.html'] = HTMLOutput(scatterprachtml, "Scatter plot text")

scatterquesthtml = '<p>'
scatterquesthtml += 'Next use a scatter plot to visualise the association between ' + varnameshort + ' and ' + respnameshort + ', and add a regression line to the plot to show the linear relationship .'
scatterquesthtml += '</p>'
scatterquesthtml += '<ul>'
scatterquesthtml += '<li>Question: What does the chart tell you about the relationship?</li>'
scatterquesthtml += '</ul>'

outputs['scatterquestions.html'] = HTMLOutput(scatterquesthtml, "Scatter plot text questions")

scattersolhtml = '<p>'
scattersolhtml += 'The instructions for creating the required scatterplot in SPSS are as follows: '
scattersolhtml += '</p>'
scattersolhtml += scatterinstrhtml
scattersolhtml += '<ul class = "instructions">'
scattersolhtml += '<li>Double click with the left mouse button on the plot and it will pop out into a Chart Editor window.</li>'
scattersolhtml += '<li>On the window click on the 5th button from the left on the bottom row of icons (It will say <strong>Add Fit Line at Total</strong> if you hover the mouse over it)</li>'
scattersolhtml += '<li>On the <strong>Properties</strong> window that appears remove the tick next to <strong>Attach label to line</strong> as otherwise the equation is superimposed on the plot which looks untidy.</li>'
scattersolhtml += '<li>Click on the <strong>Close</strong> button and the line will be added in the Chart Editor window.</li>'
scattersolhtml += '<li>Finally close the <strong>Chart Editor</strong> window and the graph in the output window will now have the fixed line as shown below:</li>' 
scattersolhtml += '</ul>'

scattersolhtml += '<ul>'
scattersolhtml += '<li>Question: What does the chart tell you about the relationship?</li>'
scattersolhtml += '</ul>'
scattersolhtml += '<p>'
scattersolhtml += 'Solution: The output from SPSS is as follows:'
scattersolhtml += '</p>'
scattersolhtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_4_000.png'].content)  + '">'
scattersolhtml += '<p>'
if aovsig_num[0] > 0.05:
    if coefb_num[1] > 0:
        scattersolhtml += 'The scatterplot appears to indicate a positive relationship but not an obvious one.'
    else:
        scattersolhtml += 'The scatterplot appears to indicate a negative relationship but not an obvious one.'
else:
    if coefb_num[1] > 0:
        scattersolhtml += 'The scatterplot appears to indicate a significant positive relationship.'
    else:
        scattersolhtml += 'The scatterplot appears to indicate a significant negative relationship.'
scattersolhtml += '</p>'

outputs['scatteranswers.html'] = HTMLOutput(scattersolhtml, "Scatter plot text answers")

regressprachtml = '<p>'
regressprachtml += 'We now need to actually run the linear regression to look at if there is a significant (linear) effect of ' + varnameshort + ' on ' + respnameshort + '. '
regressprachtml += 'This is done in SPSS as follows:'
regressprachtml += '</p>'

regressinstrhtml = '<ul class = "instructions">'
regressinstrhtml += '<li>Select <strong>Linear</strong> from the <strong>Regression</strong> submenu available from the <strong>Analyze</strong> menu.</li>'
regressinstrhtml += '<li>Copy the ' + respnamelong + ' variable into the <strong>Dependent</strong> box.</li>'
regressinstrhtml += '<li>Copy the ' + varnameshort + ' variable into the <strong>Independent(s)</strong> box.</li>'
regressinstrhtml += '<li>Click on the <strong>Statistics</strong> button.</li>'
regressinstrhtml += '<li>On the screen appears add the tick for <strong>Confidence Interval</strong> to those for <strong> Estimates </strong> and <strong> Model fit </strong>.</li>'
regressinstrhtml += '<li>Click on the <strong>Continue</strong> button to return to the main window.</li>'
regressinstrhtml += '<li>Click on the <strong>Save</strong> button.</li>'
regressinstrhtml += '<li>On the screen appears select the tick for <strong>Standardized</strong> found under <strong> Residuals </strong>.</li>'
regressinstrhtml += '<li>Click on the <strong>Continue</strong> button to return to the main window.</li>'
regressinstrhtml += '<li>Click on the <strong>OK</strong> button to run the command.</li>'
regressinstrhtml += '</ul>'

regressprachtml += regressinstrhtml

regressprachtml += '<p>'
regressprachtml += 'The command will run and five output tables will be presented. '
regressprachtml += 'The first of which is the <strong>Variables Entered/Removed</strong> table but as it is only useful when we do multiple regression we will not show it. '
regressprachtml += '</p>'

regressprachtml += '<p>'
regressprachtml += 'The next table is the <strong>Model Summary</strong> table. '
regressprachtml += '</p>'

regressprachtml += outputs['Table_Model Summary.htm'].content

regressprachtml += '<p>'
regressprachtml += 'Here we see some fit statistics for the overall model. '
regressprachtml += 'The statistic R here takes the value ' + r_str[0] + ' and is equivalent to the Pearson correlation coefficient for a simple linear regression, that is, a regression with only one predictor variable. '
regressprachtml += 'R square (' + r2_str[0] + ') is simply the value of R squared (R multiplied by itself) and represents the proportion of variance in the response variable, ' + respnameshort  + ' explained by the predictor variables. ' 
regressprachtml += 'The table also includes an adjusted R square measure which here takes value ' + adjr2_str[0] + ' and is a version of R squared that is adjusted to take account of the number of predictors (one in the case of a simple linear regression) that are in the model. '
regressprachtml += '</p>'

regressprachtml += '<p>'
regressprachtml += 'The next table is the <strong>ANOVA</strong> table. '
regressprachtml += '</p>'

regressprachtml += outputs['Table_ANOVA.htm'].content

regressprachtml += '<p>'
regressprachtml += 'The ANOVA (Analysis of Variance) table is used to look at how well the predictors as a whole can account for differences in the response variable. '
regressprachtml += 'It is used in SPSS for many models to show how the variability in the response variable is partitioned between different explanatory components (which it measures in terms of sums of squares). '
regressprachtml += '</p>'

regressprachtml += '<p>'
regressprachtml += 'Here we see in the sum of squares column that the total sum of squares, which captures the total variance in ' + respnameshort + ' is ' + aovss_str[2] + '. '
regressprachtml += 'This can be split into two parts. '
regressprachtml += 'The regression sum of squares (' + aovss_str[0] + ') captures the variability in the values of ' + respnameshort + ' that would be predicted on the basis of the predictor variables alone, so here on the basis of ' + varnameshort + '. '
regressprachtml += 'The residual sum of squares (' + aovss_str[1] + ') is the variation in the dependent variable that remains unexplained by the predictor variables. ' 
regressprachtml += 'The R-squared that we saw in the earlier table is related to the sum of squares - it expresses the regression (or explained) SS as a fraction of the total SS i.e. ' + aovss_str[0] + '/' + aovss_str[2] + '=' + str(numpy.round(aovss_num[0]/aovss_num[2], 3)) + '. '
regressprachtml += '</p>'

regressprachtml += '<p>'
regressprachtml += 'These sums of squares have associated degrees of freedom (df). '
regressprachtml += 'For the total sum of squares the df is one less than the number of observations (N - 1, here ' + str(aovdf[2]) + ') due to fitting a mean to the data. '
regressprachtml += 'The regression sum of squares has df = 1 to account for the 1 predictor in the model. '
regressprachtml += 'The residual df is then the difference between the total df and the regression df = ' + str(aovdf[1]) + '. '
regressprachtml += 'The next column is the mean squares (sums of squares adjusted for dfs) which are used to construct a test statistic, F, shown in the fifth column. '
regressprachtml += 'Here we see that F takes value ' + aovf_str[0] + ' and can be used to test the null hypothesis that there is no linear relationship between the predictors and the response variable. '
regressprachtml += 'To do this the value of F needs to be compared with an F distribution with ' + str(aovdf[0]) + ' and ' + str(aovdf[1]) + ' degrees of freedom. '
regressprachtml += 'This test results in a p value that is given in the Sig. column. '
if aovsig_num[0] > 0.05:
    regressprachtml += 'The p value (quoted under Sig.) is ' + aovsig_str[0] + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that there is no regression. '
else:
    if aovsig_num[0] < 0.001:
        regressprachtml += 'Here p is quoted as ' + aovsig_str[0] + ' (reported as p &lt; .001) which is less than the conventional .05 level used to judge significance. '
    else:
        regressprachtml += 'The p value (quoted under Sig.) is ' + aovsig_str[0] + ' which is less than 0.05. '
    regressprachtml += 'We therefore have sufficient evidence to reject the null hypothesis that the predictors have no effect. '
regressprachtml += '</p>'

regressprachtml += '<p>'
regressprachtml +='The next table is the <strong>Coefficients</strong> table. '
regressprachtml += '</p>'

regressprachtml += outputs['Table_Coefficients.htm'].content

regressprachtml += '<p>'
regressprachtml += 'This table gives the most interesting information about the regression model. '
regressprachtml += 'We begin with the coefficients that form the regression equation. '
regressprachtml += 'The regression intercept (labelled Constant in SPSS) takes value ' + coefb_str[0] + ' and is the value of the regression line when ' + varnameshort + ' takes value 0. '
regressprachtml += 'The regression slope, the B, takes value ' + coefb_str[1] + ' and is the amount by which we predict that ' + respnameshort + ' changes for an increase in ' + varnameshort + ' of one unit. '
regressprachtml += '</p>'

regressprachtml += '<p>'
regressprachtml += 'Both coefficients have associated standard errors that can be used to assess their significance and also in the case of the slope, to construct a standardised coefficient. '
regressprachtml += 'This can be seen under the Beta column and takes value ' + coefbeta_str[0] + ' which represents the predicted change in ' + respnameshort + ' in standard deviation units for an increase of one standard deviation in ' + varnameshort + '. '
regressprachtml += 'The standardised coefficient beta can be interpreted as a &quot;unit-free&quot; measure of effect size, one that can be used to compare the manitude of effects of predictors measured in different units. '
regressprachtml += '</p>'

regressprachtml += '<p>'
regressprachtml += 'To test for the significance of the coefficients we need to form test statistics which are reported under the t column and are simply B / Std. Error. '
regressprachtml += 'For the slope the t statistic is ' + coeft_str[1] + ' and this value can be compared with a t distribution to test the null hypothesis that B = 0. '
regressprachtml += 'We can see the resulting p value for the test under the Sig. column. '
if coefsig_num[1] > 0.05:
    regressprachtml += 'The p value (quoted under Sig.) is ' + coefsig_str[1] + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the slope is zero. '
else:
    if coefsig_num[1] < 0.001:
        regressprachtml += 'Here p is quoted as ' + coefsig_str[1] + ' (reported as p &lt; .001) so we have sufficient evidence to reject the null hypothesis that the slope coefficient on ' + varnameshort + ' is zero . '
    else:
        regressprachtml += 'The p value (quoted under Sig.) is ' + coefsig_str[1] + ' which is less than 0.05. '
    regressprachtml += 'We therefore have significant evidence to reject the null hypothesis that the slope is zero. '
regressprachtml += '</p>'

regressprachtml += '<p>'
regressprachtml += 'We can also check if the intercept is different from zero though this is often of less interest. '
regressprachtml += 'For the intercept the t statistic is ' + coeft_str[0] + ' and this value can be compared with a t distribution to test the null hypothesis that the intercept B = 0. '
regressprachtml += 'We can see the resulting p value for the test under the Sig. column. '
if coefsig_num[0] > 0.05:
    regressprachtml += 'The p value (quoted under Sig.) is ' + coefsig_str[0] + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the intercept is zero. '
else:
    if coefsig_num[0] < 0.001:
        regressprachtml += 'Here p is quoted as ' + coefsig_str[0] + ' (reported as p &lt; .001) so we have sufficient evidence to reject the null hypothesis that the intercept is zero .'
    else:
        regressprachtml += 'The p value (quoted under Sig.) is ' + coefsig_str[0] + ' which is less than 0.05. '
    regressprachtml += 'We therefore have significant evidence to reject the null hypothesis that the intercept is zero. '
regressprachtml += '</p>'

regressprachtml += '<p>'
regressprachtml += 'The final two columns give 95% confidence intervals for the coefficients, which show a lower bound of ' + coeflb_str[0] + ' and an upper bound of ' + coefub_str[0] + ' for our estimate of the intercept. '
regressprachtml += '</p>'

regressprachtml += '<p>'
regressprachtml += 'Similarly the 95% confidence interval bounds for the ' + varnameshort + ' B coefficient are ' + coeflb_str[1] + ' and ' + coefub_str[1] + '. '
if coefsig_num[1] > 0.05:
    regressprachtml += 'Zero does not lie between these values, so we cannot reject the null hypothesis that B = 0. '
else:
    regressprachtml += 'Zero does not lie between these values, consistent with our rejection of the null hypothesis that B = 0. '
regressprachtml += '</p>'

regressprachtml += '<p>'
regressprachtml += 'The final table is the <strong>Residuals statistics</strong> table. '
regressprachtml += '</p>'

regressprachtml += outputs['Table_Residuals Statistics.htm'].content

regressprachtml += '<p>'
regressprachtml += 'This table just summarises the predictions and residuals that come out of the regression and it is perhaps easier to look at these via plots. '
regressprachtml += '</p>'

outputs['regress.html'] = HTMLOutput(regressprachtml, "Regression text")

regressquesthtml = '<p>'
regressquesthtml += 'Next run the linear regression to formally estimate the effect of ' + varnameshort + ' on ' + respnameshort +  '. '
regressquesthtml += 'Make sure that you select the option to save the standardized residuals from the regression so that you can explore these with some plots. '
regressquesthtml += '</p>'
regressquesthtml += '<ul>'
regressquesthtml += '<li>Question: Looking at the model summary table what proportion of the variance in the response is explained by the predictor?</li>'
regressquesthtml += '<li>Question: Looking at the ANOVA table is the regression significant?</li>'
regressquesthtml += '<li>Question: Looking at the Coefficients table what is the value of the intercept?</li>'
regressquesthtml += '<li>Question: Looking at the Coefficients table what is the value of the slope?</li>'
regressquesthtml += '<li>Question: What is the standardised slope coefficient and what does it mean?</li>'
regressquesthtml += '<li>Question: Is the slope significant?</li>'
regressquesthtml += '<li>Question: What is the 95% CI for the intercept?</li>'
regressquesthtml += '<li>Question: What is the 95% CI for the slope?</li>'
regressquesthtml += '</ul>'

outputs['regressquestions.html'] = HTMLOutput(regressquesthtml, "Regression text questions")

regresssolhtml = '<p>'
regresssolhtml += 'The instructions for running the required regression in SPSS are as follows: '
regresssolhtml += '</p>'
regresssolhtml += regressinstrhtml
regresssolhtml += '<ul>'
regresssolhtml += '<li>Question: Looking at the model summary table what proportion of the variance in the response is explained by the predictor?</li>'
regresssolhtml += '<li>Question: Looking at the ANOVA table is the regression significant?</li>'
regresssolhtml += '<li>Question: Looking at the Coefficients table what is the value of the intercept?</li>'
regresssolhtml += '<li>Question: Looking at the Coefficients table what is the value of the slope?</li>'
regresssolhtml += '<li>Question: What is the standardised slope coefficient and what does it mean?</li>'
regresssolhtml += '<li>Question: Is the slope significant?</li>'
regresssolhtml += '<li>Question: What is the 95% CI for the intercept?</li>'
regresssolhtml += '<li>Question: What is the 95% CI for the slope?</li>'
regresssolhtml += '</ul>'
regresssolhtml += '<p>'
regresssolhtml += 'Solutions: The output from SPSS is as follows:'
regresssolhtml += '</p>'
regresssolhtml += outputs['Table_Model Summary.htm'].content
regresssolhtml += outputs['Table_ANOVA.htm'].content
regresssolhtml += outputs['Table_Coefficients.htm'].content
regresssolhtml += '<ul>'
regresssolhtml += '<li>The R squared provides the proportion of the variance explained by the regression and has the value ' + r2_str[0] + '. </li>'
regresssolhtml += '<li>'
if aovsig_num[0] > 0.05:
    regresssolhtml += 'The p value (quoted under Sig.) is ' + aovsig_str[0] + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that there is no regression.'
else:
    if aovsig_num[0] < 0.001:
        regresssolhtml += 'Here p is quoted as ' + aovsig_str[0] + ' (reported as p &lt; .001) which is less than the conventional .05 level used to judge significance. '
    else:
        regresssolhtml += 'The p value (quoted under Sig.) is ' + aovsig_str[0] + ' which is less than 0.05. '
    regresssolhtml += 'We therefore have sufficient evidence to reject the null hypothesis that the predictors have no effect. '
regresssolhtml += '</li>'
regresssolhtml += '<li>The regression intercept takes value ' + coefb_str[0] + '.</li>'
regresssolhtml += '<li>The regression slope takes value ' + coefb_str[1] + '.</li>'
regresssolhtml += '<li>The standardised slope coefficient takes value ' + coefbeta_str[0] + ' which represents the predicted change in ' + respnameshort + ' for an increase of 1 standard deviation in ' + varnameshort + '.</li>'
if coefsig_num[1] > 0.05:
    regresssolhtml += '<li>The p value (quoted under Sig.) is ' + coefsig_str[1] + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the slope is zero.</li>'
else:
    regresssolhtml += '<li>'
    if coefsig_num[1] < 0.001:
        regresssolhtml += 'Here p is quoted as ' + coefsig_str[1] + ' (reported as p &lt; .001) so we have sufficient evidence to reject the null hypothesis that the slope coefficient on ' + varnameshort + ' is zero . '
    else:
        regresssolhtml += 'The p value (quoted under Sig.) is ' + coefsig_str[1] + ' which is less than 0.05. '
    regresssolhtml += 'We therefore have significant evidence to reject the null hypothesis that the slope is zero. '
    regresssolhtml += '</li>'
regresssolhtml += '<li>The confidence interval for the intercept is ' + coeflb_str[0] + ' to ' + coefub_str[0] + '.</li>'
regresssolhtml += '<li>The confidence interval for the slope is ' + coeflb_str[1] + ' to ' + coefub_str[1] + '.</li>'
regresssolhtml += '</ul>'

outputs['regressanswers.html'] = HTMLOutput(regresssolhtml, "Regression text answers")

resihistprachtml = '<p>'
resihistprachtml += 'As we ticked the box to request that standardised residuals were saved this has resulted in an additional variable, named <strong>ZRE_1</strong>, being stored in the dataset at the end of the existing variables (You can see this by viewing the dataset in the SPSS Data Editor Window). '
resihistprachtml += 'We can use this variable to create some residual plots to assess the fit of the model. '
resihistprachtml += 'We will firstly plot a histogram of the residuals to check their normality which can be done in SPSS as follows:'
resihistprachtml += '</p>'

resihistinstrhtml = '<ul class = "instructions">'
resihistinstrhtml += '<li>Select <strong>Histogram</strong> from the <strong>Legacy dialogs</strong> available from the <strong>Graphs</strong> menu.</li>'
resihistinstrhtml += '<li>Copy the <strong>Standardized Residual [ZRE_1]</strong> variable into the <strong>Variable</strong> box.</li>'
resihistinstrhtml += '<li>Click on the <strong>Display normal curve</strong> tick box.</li>'
resihistinstrhtml += '<li>Click on the <strong>OK</strong> button to produce the graph.</li>'
resihistinstrhtml += '</ul>'

resihistprachtml += resihistinstrhtml

resihistprachtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_1_000.png'].content)  + '">'

resihistprachtml += '<p>'
resihistprachtml += 'Here we hope to see the histogram of residuals roughly following the shape of the normal curve that is superimposed over them. '
resihistprachtml += '</p>'
outputs['residualhistogram.html'] = HTMLOutput(resihistprachtml, "Residual histogram text")

resihistquesthtml = '<p>'
resihistquesthtml += 'Next plot a histogram of the residuals from the regression model. '
resihistquesthtml += '</p>'
resihistquesthtml += '<ul>'
resihistquesthtml += '<li>Question: Do the residuals look approximately Normal?</li>'
resihistquesthtml += '</ul>'

outputs['residualhistogramquestions.html'] = HTMLOutput(resihistquesthtml, "Residual histogram text questions")

resihistsolhtml = '<p>'
resihistsolhtml += 'The instructions for creating the residual histogram in SPSS are as follows: '
resihistsolhtml += '</p>'
resihistsolhtml += resihistinstrhtml
resihistsolhtml += '<ul>'
resihistsolhtml += '<li>Question: Do the residuals look approximately Normal?</li>'
resihistsolhtml += '</ul>'
resihistsolhtml += '<p>'
resihistsolhtml += 'Solution: The output from SPSS is as follows:'
resihistsolhtml += '</p>'
resihistsolhtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_1_000.png'].content)  + '">'
resihistsolhtml += '<p>We hope to see the histogram of residuals roughly following the shape of the normal curve that is superimposed over them.</p>'

outputs['residualhistogramanswers.html'] = HTMLOutput(resihistsolhtml, "Residual histogram text answers")

resiscatterprachtml = '<p>'
resiscatterprachtml += 'We can also look at how the distribution of the residuals interacts with the predictor variable to check there is no relationship. '
resiscatterprachtml += 'We do this via a scatterplot which can be produced in SPSS as follows:'
resiscatterprachtml += '</p>'

resiscatterinstrhtml = '<ul class = "instructions">'
resiscatterinstrhtml += '<li>Select <strong>Scatter/Dot</strong> from the <strong>Legacy dialogs</strong> available from the <strong>Graphs</strong> menu.</li>'
resiscatterinstrhtml += '<li>Select Simple Scatter and click on Define to bring up the Simple Scatterplot window.</li>'
resiscatterinstrhtml += '<li>Copy the <strong>Standardized Residual [ZRE_1]</strong> variable into the <strong>Y Axis</strong> box.</li>'
resiscatterinstrhtml += '<li>Copy the ' + varnamelong + ' variable into the <strong>X Axis</strong> box.</li>'
resiscatterinstrhtml += '<li>Click on the <strong>OK</strong> button and the plot will appear.</li>' 
resiscatterinstrhtml += '</ul>'

resiscatterprachtml += resiscatterinstrhtml

resiscatterprachtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_3_000.png'].content)  + '">'

resiscatterprachtml += '<p>'
resiscatterprachtml += 'Here we hope that the residuals show a random scatter when plotted against the predictor variable and also that their variability is constant across different values of the predictor variable. '
resiscatterprachtml += '</p>'
outputs['residualscatter.html'] = HTMLOutput(resiscatterprachtml, "Residual scatter plot text")


resiscatterquesthtml = '<p>'
resiscatterquesthtml += 'Now use a scatterplot to check whether there appears to be any relationship between the residuals and the predictor variable ' + varnameshort + '. '
resiscatterquesthtml += '</p>'
resiscatterquesthtml += '<ul>'
resiscatterquesthtml += '<li>Question: Is there any specific pattern in the plot?</li>'
resiscatterquesthtml += '</ul>'

outputs['residualscatterquestions.html'] = HTMLOutput(resiscatterquesthtml, "Residual scatter plot text questions")

resiscattersolhtml = '<p>'
resiscattersolhtml += 'The instructions for creating the required scatterplot in SPSS are as follows: '
resiscattersolhtml += '</p>'
resiscattersolhtml += resiscatterinstrhtml
resiscattersolhtml += '<ul>'
resiscattersolhtml += '<li>Question: Is there any specific pattern in the plot?</li>'
resiscattersolhtml += '</ul>'
resiscattersolhtml += '<p>'
resiscattersolhtml += 'Solution: The output from SPSS is as follows:'
resiscattersolhtml += '</p>'
resiscattersolhtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_3_000.png'].content)  + '">'
resiscattersolhtml += '<p>We hope that the residuals show a random scatter when plotted against the predictor variable and also that their variability is constant across different values of the predictor variable.</p>'

outputs['residualscatteranswers.html'] = HTMLOutput(resiscattersolhtml, "Residual scatter plot text answers")


resiplotprachtml = '<p>'
resiplotprachtml += 'Finally we might like to superimpose the regression line onto the scatterplot we drew earlier of the response against the predictor, so that the strength of the linear relationship is clear. '
resiplotprachtml += 'To do this we will need to use the <strong>Chart Editor</strong> in SPSS so follow the following instructions:'
resiplotprachtml += '</p>'

resiplotinstrhtml = '<ul class = "instructions">'
resiplotinstrhtml += '<li>Locate the earlier scatterplot in the SPSS output window noting you may need to scroll up to find it.</li>'
resiplotinstrhtml += '<li>Double click with the left mouse button on the plot and it will pop out into a Chart Editor window.</li>'
resiplotinstrhtml += '<li>On the window click on the 5th button from the left on the bottom row of icons (It will say <strong>Add Fit Line at Total</strong> if you hover the mouse over it)</li>'
resiplotinstrhtml += '<li>On the <strong>Properties</strong> window that appears remove the tick next to <strong>Attach label to line</strong> as otherwise the equation is superimposed on the plot which looks untidy.</li>'
resiplotinstrhtml += '<li>Click on the <strong>Close</strong> button and the line will be added in the Chart Editor window.</li>'
resiplotinstrhtml += '<li>Finally close the <strong>Chart Editor</strong> window and the graph in the output window will now have the fixed line as shown below:</li>' 
resiplotinstrhtml += '</ul>'

resiplotprachtml += resiplotinstrhtml

resiplotprachtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_4_000.png'].content)  + '">'

resiplotprachtml += '<p>'
resiplotprachtml += 'Note that the scatterplot now also contains the R-squared value which corresponds to the R-squared value we saw in the regression fit earlier. '
resiplotprachtml += '</p>'

outputs['regressionplot.html'] = HTMLOutput(resiplotprachtml, "Regression plot text")

combinedprachtml = outputs['introduction.html'].content
combinedprachtml += outputs['descriptive.html'].content
combinedprachtml += outputs['scatter.html'].content
combinedprachtml += outputs['regress.html'].content
combinedprachtml += outputs['residualhistogram.html'].content
combinedprachtml += outputs['residualscatter.html'].content
combinedprachtml += outputs['regressionplot.html'].content

outputs['allpractical.html'] = HTMLOutput(combinedprachtml, "Combined practical HTML outputs")

combinedquesthtml = outputs['introduction.html'].content
combinedquesthtml += outputs['descriptivequestions.html'].content
combinedquesthtml += outputs['scatterquestions.html'].content
combinedquesthtml += outputs['regressquestions.html'].content
combinedquesthtml += outputs['residualhistogramquestions.html'].content
combinedquesthtml += outputs['residualscatterquestions.html'].content

outputs['allquestions.html'] = HTMLOutput(combinedquesthtml, "Combined questions HTML outputs")

combinedsolhtml = outputs['descriptiveanswers.html'].content
combinedsolhtml += outputs['scatteranswers.html'].content
combinedsolhtml += outputs['regressanswers.html'].content
combinedsolhtml += outputs['residualhistogramanswers.html'].content
combinedsolhtml += outputs['residualscatteranswers.html'].content

outputs['allanswers.html'] = HTMLOutput(combinedsolhtml, "Combined solutions HTML outputs")

'''